package Derytas;

import java.util.*;
import java.awt.*;
import java.awt.List;
import java.awt.event.*;

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// Clase: clsAtributo                                                         //
// Mtodos: - public clsAtributo: Constructor de la clase                     //
//          - private void AbrirAtributo: Abre una nueva ventana para crear   //
//            un atributo (para atributos compuestos)                         //
//          - private void ActualizarTitulo: Actualiza el ttulo de la        //
//            ventana                                                         //
//          - private boolean bValidar: Comprueba que los datos introducidos  //
//            para la creacin del atributo son correctos                     //
//          - private void CheckStateChanged: Gestiona el cambio de seleccin //
//            de la restriccin Check                                         //
//          - private void CrearAtributo: Crea el atributo                    //
//          - private void DefaultStateChanged: Gestiona el cambio de         //
//            seleccin del valor por defecto                                 //
//          - private void LimpiarVentana: Establece los valores iniciales de //
//            la ventana                                                      //
//          - private void NotNullStateChanged: Gestiona el cambio de         //
//            seleccin de la restriccin Not Null                            //
//          - private void Ocultar: Cierra la ventana                         //
//          - private String sConstruirDominio: Construye el dominio          //
//          - private void UniqueStateChanged: Gestiona el cambio de          //
//            seleccin de la restriccion Unique                              //
// Descripcin: Clase para crear los atributos de entidades, relaciones y     //
//              otros atributos (para atributos compuestos)                   //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////
public class clsAtributo
   extends Frame {

   // Variables de clase
   private clsGeneral oGeneral;
   private clsConfig oConfig;
   private clsMensajes oMensajes;

   // Componentes de la ventana
   private Label lblNombre; // Etiqueta para el nombre del atributo
   private TextField txtNombre; // Campo para escribir el nombre del atributo
   private Label lblTipo; // Etiqueta para el tipo de atributo
   private CheckboxGroup cbgTipo; // CheckboxGroup para el tipo de atributo
   private Checkbox chbAtributoSimple; // Opcin atributo simple
   private Checkbox chbAtributoDerivado; // Opcin atributo derivado
   private Checkbox chbAtributoLlave; // Opcin atributo llave
   private Checkbox chbAtributoLlaveParcial; // Opcin atributo llave parcial
   private Checkbox chbAtributoMultivaluado; // Opcin atributo multivaluado
   private Label lblDominio; // Etiqueta para el dominio del atributo
   private Label lblLongitud; // Campo para la longitud del dominio
   private Label lblEscala; // Campo para la escala del dominio
   private Choice choDominio; // Campo para el tipo del dominio
   private TextField txtLongitud; // Campo para la longitud del dominio
   private TextField txtEscala; // Campo para la escala del dominio
   private Label lblTituloDefault; // Etiqueta para el valor por defecto
   private Checkbox chbDefault; // Opcin valor por defecto
   private Label lblDefault; // Etiqueta para indicar el valor por defecto
   private TextField txtDefault; // Campo con el valor por defecto
   private Label lblRestricciones; // Etiquta para las restricciones
   private Checkbox chbNotNull; // Opcin restriccin Not Null
   private Label lblNotNull; // Etiqueta para la restriccin Not Null
   private TextField txtNotNull; // Campo para el nombre de la restriccin Not Null
   private Checkbox chbUnique; // Opcin restriccin Unique
   private Label lblUnique; // Etiqueta para la restriccin Unique
   private TextField txtUnique; // Campo con el nombre de la restriccin Unique
   private Checkbox chbCheck; // Opcin restriccin Check
   private Label lblCheck; // Etiqueta para la restriccin Check
   private TextField txtCheck; // Campo para la restriccin Check
   private Label lblNombreCheck; // Etiqueta para la restriccin Check
   private TextField txtNombreCheck; // Campo con el nombre de la restriccin Check
   private Checkbox chbCerrar; // Opcin para cerrar la ventana
   private Button btnCrear; // Botn para crear el atributo
   private Button btnCerrar; // Opcin para no crear el atributo
   private Frame fraVentanaProp; // Ventana propietaria del atributo
   private Vector vAtributosProp; // Vector de atributos de la ventana propietaria
   private List lstAtributosProp; // Lista de atributos de la ventana propietaria
   private Label lblAtributos; // Etiqueta para los atributos
   private Button btnSubir; // Botn para subir un atributo
   private Button btnBajar; // Opcin para bajar un atributo
   private List lstAtributos; // Lista de atributos
   private Button btnAtributos; // Botn para crear atributos
   private Button btnElimAtrib; // Botn para eliminar un atributo
   private Vector vAtributos; // Vector de atributos
   private String sPropietario; // Entidad o relacin propietarias del atributo
   private Label lblComentario; // Etiqueta para el comentario
   private TextArea txaComentario; // Comentario del atributo

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: clsAtributo                                              //
   // Parmetros: - pfraVentanaProp (Frame): Ventana propietaria del atributo //
   //             - pvAtributosProp (Vector): Vector de atributos de la       //
   //               ventana propietaria                                       //
   //             - lstaAtributosProp (List): Lista de atributos de la        //
   //               ventana propietaria                                       //
   //             - poGeneral (clsGeneral): Objeto de la clase clsGeneral     //
   //               para poder acceder a los mtodos de carcter general      //
   //             - poConfig (clsConfig): Objeto de la clase clsConfig para   //
   //               poder acceder a las variables de configuracin            //
   // Descripcin: Constructor de la clase                                    //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   public clsAtributo(Frame pfraVentanaProp,
                      Vector pvAtributosProp,
                      List lstaAtributosProp,
                      clsGeneral poGeneral,
                      clsConfig poConfig) {

      // Inicializacin de variables

      // Asignar la ventana propietaria
      fraVentanaProp = pfraVentanaProp;
      // Asignar el vector de atributos de la ventana propietaria
      vAtributosProp = pvAtributosProp;
      // Asignar la lista de atributos de la ventana propietaria
      lstAtributosProp = lstaAtributosProp;
      // Asignar el objeto de la clase clsGeneral
      oGeneral = poGeneral;
      // Asignar el objeto de la clase clsConfig
      oConfig = poConfig;
      // Crear el objeto  de la clase clsMensajes
      oMensajes = new clsMensajes(oGeneral);
      // Definir el tamao para algunos componentes de la ventana
      int nAncho_chbRestriccion = 70;
      int nAncho_txtLongitud = 50;
      int nAncho_txtEscala = 50;
      int nAncho_choDominio = 120;
      int nAnchoEtiquetas = clsConstantes.nANCHO_LABEL - 50;
      // Guardar la jerarqua de propietarios
      sPropietario = fraVentanaProp.getTitle();
      sPropietario = sPropietario.replaceFirst("Entidad: ", "");
      sPropietario = sPropietario.replaceFirst("Relacin: ", "");

      // Crear los componentes de la ventana
      lblNombre = new Label("Nombre del Atributo");
      txtNombre = new TextField();
      lblTipo = new Label("Tipo de Atributo");
      cbgTipo = new CheckboxGroup();
      chbAtributoSimple = new Checkbox(clsConstantes.sATRIBUTO_SIMPLE,
                                       cbgTipo,
                                       true);
      chbAtributoDerivado = new Checkbox(clsConstantes.sATRIBUTO_DERIVADO,
                                         cbgTipo,
                                         false);
      chbAtributoLlave = new Checkbox(clsConstantes.sATRIBUTO_LLAVE,
                                      cbgTipo,
                                      false);
      chbAtributoLlaveParcial = new Checkbox(clsConstantes.
                                             sATRIBUTO_LLAVE_PARCIAL,
                                             cbgTipo,
                                             false);
      chbAtributoMultivaluado = new Checkbox(clsConstantes.
                                             sATRIBUTO_MULTIVALUADO,
                                             cbgTipo,
                                             false);
      lblDominio = new Label("Dominio");
      lblLongitud = new Label("Longitud");
      lblEscala = new Label("Escala");
      choDominio = new Choice();
      txtLongitud = new TextField();
      txtEscala = new TextField();
      lblTituloDefault = new Label("Valor por Defecto (Opcional)");
      chbDefault = new Checkbox("DEFAULT");
      lblDefault = new Label("Valor:");
      txtDefault = new TextField();
      lblRestricciones = new Label("Restricciones");
      chbNotNull = new Checkbox("NOT NULL");
      lblNotNull = new Label("Nombre de la Restriccin NOT NULL");
      txtNotNull = new TextField();
      chbUnique = new Checkbox("UNIQUE");
      lblUnique = new Label("Nombre de la Restriccin UNIQUE");
      txtUnique = new TextField();
      chbCheck = new Checkbox("CHECK");
      lblNombreCheck = new Label("Nombre de la Restriccin CHECK");
      txtNombreCheck = new TextField();
      lblCheck = new Label("Condicin CHECK");
      txtCheck = new TextField();
      chbCerrar = new Checkbox("Cerrar despus de Crear");
      btnCrear = new Button("Crear Atributo");
      btnCerrar = new Button("Cancelar");
      lblAtributos = new Label("Lista de Atributos");
      btnSubir = new Button("Subir");
      btnBajar = new Button("Bajar");
      lstAtributos = new List();
      btnAtributos = new Button("Crear Atributos");
      btnElimAtrib = new Button("Eliminar Atributo");
      lblComentario = new Label("Comentario");
      txaComentario = new TextArea("", 0, 0, TextArea.SCROLLBARS_NONE);
      vAtributos = new Vector();

      // Propiedades de los componentes de la ventana

      // lblNombre
      // Establecer tamao y posicin
      lblNombre.setBounds(new Rectangle(clsConstantes.nSEP_VHOR,
                                        clsConstantes.nSEP_VVER,
                                        clsConstantes.nANCHO_LABEL,
                                        clsConstantes.nALTO_COM));

      // txtNombre
      // Establecer tamao y posicin
      txtNombre.setBounds(new Rectangle(clsConstantes.nSEP_VHOR,
                                        lblNombre.getY() +
                                        lblNombre.getHeight(),
                                        clsConstantes.nANCHO_TEXTFIELD,
                                        clsConstantes.nALTO_COM));
      // Controlar el teclado
      txtNombre.addKeyListener(new KeyListener() {

         String sNombre; // Nombre del atributo

         public void keyTyped(KeyEvent e) {
            //  No hace falta
         }; // keyTyped

         public void keyPressed(KeyEvent e) {
            if ( (e.getKeyCode() == e.VK_ENTER) &&
                (! (txtNombre.getText().equals("")))) {
               // Crear el atributo cuando se pulsa Enter sobre el nombre
               CrearAtributo();
            } // if
            else {
               // Deshabilitar el botn Crear si no hay nombre escrito
               btnCrear.setEnabled(! (txtNombre.getText().equals("")));
               // Habilitar el botn para crear atributos cuando lo est Crear
               btnAtributos.setEnabled(btnCrear.isEnabled());
               // Comprobar si se excede el mximo de caracteres para el nombre
               if (txtNombre.getText().length() <= clsConstantes.nMAX_NOMBRE) {
                  // Guardar el nombre escrito hasta el momento
                  sNombre = txtNombre.getText();
                  // Actualizar el ttulo de la ventana
                  ActualizarTitulo();
               } // if
               else {
                  txtNombre.setText(sNombre);
               } // else
            } // else
         } // keyPressed

         public void keyReleased(KeyEvent e) {
            // Deshabilitar el botn Crear si no hay nombre escrito
            btnCrear.setEnabled(! (txtNombre.getText().equals("")));
            // Habilitar el botn para crear atributos cuando lo est Crear
            btnAtributos.setEnabled(btnCrear.isEnabled());
            // Comprobar si se excede el mximo de caracteres para el nombre
            if (txtNombre.getText().length() <= clsConstantes.nMAX_NOMBRE) {
               // Guardar el nombre escrito hasta el momento
               sNombre = txtNombre.getText();
               // Actualizar el ttulo de la ventana
               ActualizarTitulo();
            } // if
            else {
               txtNombre.setText(sNombre);
            } // else
         } // keyReleased
      }); // addKeyListener

      // lblTipo
      // Establecer tamao y posicin
      lblTipo.setBounds(new Rectangle(clsConstantes.nSEP_VHOR,
                                      txtNombre.getY() +
                                      txtNombre.getHeight() +
                                      clsConstantes.nSEP_COM,
                                      nAnchoEtiquetas,
                                      clsConstantes.nALTO_COM));

      // chbAtributoSimple
      // Establecer tamao y posicin
      chbAtributoSimple.setBounds(new Rectangle(clsConstantes.nSEP_VHOR,
                                                lblTipo.getY() +
                                                lblTipo.getHeight(),
                                                nAnchoEtiquetas,
                                                clsConstantes.nALTO_COM));

      // chbAtributoDerivado
      // Establecer tamao y posicin
      chbAtributoDerivado.setBounds(new Rectangle(clsConstantes.nSEP_VHOR,
                                                  chbAtributoSimple.getY() +
                                                  chbAtributoSimple.getHeight() +
                                                  clsConstantes.nSEP_COM,
                                                  nAnchoEtiquetas,
                                                  clsConstantes.nALTO_COM));

      // chbAtributoLlave
      // Establecer tamao y posicin
      chbAtributoLlave.setBounds(new Rectangle(clsConstantes.nSEP_VHOR,
                                               chbAtributoDerivado.getY() +
                                               chbAtributoDerivado.getHeight() +
                                               clsConstantes.nSEP_COM,
                                               nAnchoEtiquetas,
                                               clsConstantes.nALTO_COM));

      // chbAtributoLlaveParcial
      // Establecer tamao y posicin
      chbAtributoLlaveParcial.setBounds(new Rectangle(clsConstantes.nSEP_VHOR,
                                                      chbAtributoLlave.getY() +
                                                      chbAtributoLlave.getHeight() +
                                                      clsConstantes.nSEP_COM,
                                                      nAnchoEtiquetas,
                                                      clsConstantes.nALTO_COM));

      // chbAtributoMultivaluado
      // Establecer tamao y posicin
      chbAtributoMultivaluado.setBounds(new Rectangle(clsConstantes.nSEP_VHOR,
                                                      chbAtributoLlaveParcial.getY() +
                                                      chbAtributoLlaveParcial.getHeight() +
                                                      clsConstantes.nSEP_COM,
                                                      nAnchoEtiquetas,
                                                      clsConstantes.nALTO_COM));

      // lblDominio
      // Establecer tamao y posicin
      lblDominio.setBounds(new Rectangle(txtNombre.getX() +
                                         txtNombre.getWidth() +
                                         clsConstantes.nSEP_COM,
                                         lblTipo.getY(),
                                         100,
                                         clsConstantes.nALTO_COM));

      // choDominio
      // Establecer tamao y posicin
      choDominio.setBounds(new Rectangle(lblDominio.getX(),
                                         lblDominio.getY() +
                                         lblDominio.getHeight(),
                                         nAncho_choDominio,
                                         clsConstantes.nALTO_COM));
      // Cargar los valores para el dominio
      oGeneral.CargarChoiceDominio(choDominio);
      // Aadir el componente al listener para cuando cambie la seleccin
      choDominio.addItemListener(new ItemListener() {
         public void itemStateChanged(ItemEvent e) {
            // Llamar al mtodo que gestiona el evento
            CambioDominio();
         } // Evento: itemStateChanged
      }); // addItemListener

      // lblLongitud
      // Establecer tamao y posicin
      lblLongitud.setBounds(new Rectangle(choDominio.getX() +
                                          choDominio.getWidth() +
                                          clsConstantes.nSEP_COM,
                                          lblDominio.getY(),
                                          50,
                                          clsConstantes.nALTO_COM));

      // txtLongitud
      // Establecer tamao y posicin
      txtLongitud.setBounds(new Rectangle(lblLongitud.getX(),
                                          choDominio.getY(),
                                          nAncho_txtLongitud,
                                          clsConstantes.nALTO_COM));

      // lblEscala
      // Establecer tamao y posicin
      lblEscala.setBounds(new Rectangle(lblLongitud.getX() +
                                        lblLongitud.getWidth() +
                                        clsConstantes.nSEP_COM,
                                        lblDominio.getY(),
                                        100,
                                        clsConstantes.nALTO_COM));

      // txtEscala
      // Establecer tamao y posicin
      txtEscala.setBounds(new Rectangle(lblEscala.getX(),
                                        choDominio.getY(),
                                        nAncho_txtEscala,
                                        clsConstantes.nALTO_COM));

      // lblTituloDefault
      // Establecer tamao y posicin
      lblTituloDefault.setBounds(new Rectangle(lblDominio.getX(),
                                               choDominio.getY() +
                                               choDominio.getHeight() +
                                               clsConstantes.nSEP_COM,
                                               clsConstantes.nANCHO_LABEL,
                                               clsConstantes.nALTO_COM));

      // chbDefault
      // Establecer tamao y posicin
      chbDefault.setBounds(new Rectangle(lblTituloDefault.getX(),
                                         lblTituloDefault.getY() +
                                         lblTituloDefault.getHeight(),
                                         nAncho_chbRestriccion,
                                         clsConstantes.nALTO_COM));
      // Aadir el componente al listener para cuando cambie la seleccin
      chbDefault.addItemListener(new ItemListener() {
         public void itemStateChanged(ItemEvent e) {
            // Llamar al mtodo que gestiona el evento
            DefaultStateChanged();
         } // Evento: itemStateChanged
      }); // addItemListener

      // lblDefault
      // Establecer tamao y posicin
      lblDefault.setBounds(new Rectangle(chbDefault.getX(),
                                         chbDefault.getY() +
                                         chbDefault.getHeight(),
                                         clsConstantes.nANCHO_LABEL,
                                         clsConstantes.nALTO_COM));

      // txtDefault
      // Establecer tamao y posicin
      txtDefault.setBounds(new Rectangle(lblDefault.getX(),
                                         lblDefault.getY() +
                                         lblDefault.getHeight(),
                                         clsConstantes.nANCHO_TEXTFIELD,
                                         clsConstantes.nALTO_COM));
      // Deshabilitar el campo
      txtDefault.setEnabled(false);

      // lblRestricciones
      // Establecer tamao y posicin
      lblRestricciones.setBounds(new Rectangle(clsConstantes.nSEP_VHOR,
                                               chbAtributoMultivaluado.getY() +
                                               chbAtributoMultivaluado.
                                               getHeight() +
                                               clsConstantes.nSEP_COM,
                                               clsConstantes.nANCHO_LABEL,
                                               clsConstantes.nALTO_COM));

      // chbNotNull
      // Establecer tamao y posicin
      chbNotNull.setBounds(new Rectangle(clsConstantes.nSEP_VHOR,
                                         lblRestricciones.getY() +
                                         lblRestricciones.getHeight() +
                                         clsConstantes.nSEP_COM,
                                         clsConstantes.nANCHO_CHECKBOX,
                                         clsConstantes.nALTO_COM));
      // Aadir el componente al listener para cuando se cambie la seleccin
      chbNotNull.addItemListener(new ItemListener() {
         public void itemStateChanged(ItemEvent e) {
            // Llamar al mtodo que gestiona el evento
            NotNullStateChanged();
         } // Evento: itemStateChanged
      }); // addItemListener

      // lblNotNull
      // Establecer tamao y posicin
      lblNotNull.setBounds(new Rectangle(clsConstantes.nSEP_VHOR,
                                         chbNotNull.getY() +
                                         chbNotNull.getHeight(),
                                         clsConstantes.nANCHO_LABEL,
                                         clsConstantes.nALTO_COM));

      // txtNotNull
      // Establecer tamao y posicin
      txtNotNull.setBounds(new Rectangle(clsConstantes.nSEP_VHOR,
                                         lblNotNull.getY() +
                                         lblNotNull.getHeight(),
                                         clsConstantes.nANCHO_TEXTFIELD,
                                         clsConstantes.nALTO_COM));
      // Deshabilitar el campo
      txtNotNull.setEnabled(false);

      // chbUnique
      // Establecer tamao y posicin
      chbUnique.setBounds(new Rectangle(clsConstantes.nSEP_VHOR,
                                        txtNotNull.getY() +
                                        txtNotNull.getHeight() +
                                        clsConstantes.nSEP_COM,
                                        nAncho_chbRestriccion,
                                        clsConstantes.nALTO_COM));
      // Aadir el componente al listener para cuando se cambie la seleccin
      chbUnique.addItemListener(new ItemListener() {
         public void itemStateChanged(ItemEvent e) {
            // Llamar al mtodo que gestiona el evento
            UniqueStateChanged();
         } // Evento: itemStateChanged
      }); // addItemListener

      // lblUnique
      // Establecer tamao y posicin
      lblUnique.setBounds(new Rectangle(clsConstantes.nSEP_VHOR,
                                        chbUnique.getY() +
                                        chbUnique.getHeight(),
                                        clsConstantes.nANCHO_LABEL,
                                        clsConstantes.nALTO_COM));

      // txtUnique
      // Establecer tamao y posicin
      txtUnique.setBounds(new Rectangle(clsConstantes.nSEP_VHOR,
                                        lblUnique.getY() +
                                        lblUnique.getHeight(),
                                        clsConstantes.nANCHO_TEXTFIELD,
                                        clsConstantes.nALTO_COM));
      // Deshabilitar el campo
      txtUnique.setEnabled(false);

      // chbCheck
      // Establecer tamao y posicin
      chbCheck.setBounds(new Rectangle(txtDefault.getX(),
                                       chbNotNull.getY(),
                                       nAncho_chbRestriccion,
                                       clsConstantes.nALTO_COM));
      // Aadir el componente al listener para cuando se cambie la seleccin
      chbCheck.addItemListener(new ItemListener() {
         public void itemStateChanged(ItemEvent e) {
            // Llamar al mtodo que gestiona el evento
            CheckStateChanged();
         } // Evento: itemStateChanged
      }); // addItemListener

      // lblNombreCheck
      // Establecer tamao y posicin
      lblNombreCheck.setBounds(new Rectangle(chbCheck.getX(),
                                             chbCheck.getY() +
                                             chbCheck.getHeight(),
                                             clsConstantes.nANCHO_LABEL,
                                             clsConstantes.nALTO_COM));

      // txtNombreCheck
      // Establecer tamao y posicin
      txtNombreCheck.setBounds(new Rectangle(chbCheck.getX(),
                                             lblNombreCheck.getY() +
                                             lblNombreCheck.getHeight(),
                                             clsConstantes.nANCHO_TEXTFIELD,
                                             clsConstantes.nALTO_COM));
      // Deshabilitar el campo
      txtNombreCheck.setEnabled(false);

      // lblCheck
      // Establecer tamao y posicin
      lblCheck.setBounds(new Rectangle(chbCheck.getX(),
                                       txtNombreCheck.getY() +
                                       txtNombreCheck.getHeight() +
                                       clsConstantes.nSEP_COM,
                                       clsConstantes.nANCHO_LABEL,
                                       clsConstantes.nALTO_COM));

      // txtCheck
      // Establecer tamao y posicin
      txtCheck.setBounds(new Rectangle(chbCheck.getX(),
                                       lblCheck.getY() +
                                       lblCheck.getHeight(),
                                       clsConstantes.nANCHO_TEXTFIELD,
                                       clsConstantes.nALTO_COM));
      // Deshabilitar el campo
      txtCheck.setEnabled(false);

      // lblAtributos
      // Establecer tamao y posicin
      lblAtributos.setBounds(new Rectangle(clsConstantes.nSEP_VHOR,
                                           txtUnique.getY() +
                                           txtUnique.getHeight() +
                                           2 * clsConstantes.nSEP_COM,
                                           100,
                                           clsConstantes.nALTO_COM));

      // lstAtributos
      // Establecer tamao y posicin
      lstAtributos.setBounds(new Rectangle(lblAtributos.getX(),
                                           lblAtributos.getY() +
                                           lblAtributos.getHeight(),
                                           clsConstantes.nANCHO_LIST,
                                           clsConstantes.nALTO_LIST));
      // Aadir el componente al listener para cuando se cambie la seleccin
      lstAtributos.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            // Llamar al mtodo que gestiona el evento
            oGeneral.ListaClick(btnElimAtrib,
                                lstAtributos,
                                btnSubir,
                                btnBajar);
         } // Evento: actionPerformed
      }); // addActionListener

      // btnBajar
      // Establecer tamao y posicin
      btnBajar.setBounds(new Rectangle(lblAtributos.getX() +
                                       clsConstantes.nANCHO_LABEL -
                                       clsConstantes.nANCHO_BUTTON / 2,
                                       lblAtributos.getY() -
                                       clsConstantes.nSEP_COM,
                                       clsConstantes.nANCHO_BUTTON / 2,
                                       clsConstantes.nALTO_BUTTON));
      // Deshabilitar el botn
      btnBajar.setEnabled(false);
      // Aadir el componente al listener para cuando se pulse el botn
      btnBajar.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            // Llamar al mtodo que gestiona el evento
            oGeneral.Bajar(lstAtributos,
                           vAtributos,
                           btnSubir,
                           btnBajar);
         } // Evento: actionPerformed
      }); // addActionListener

      // btnSubir
      // Establecer tamao y posicin
      btnSubir.setBounds(new Rectangle(btnBajar.getX() -
                                       clsConstantes.nANCHO_BUTTON / 2 -
                                       clsConstantes.nSEP_COM,
                                       btnBajar.getY(),
                                       clsConstantes.nANCHO_BUTTON / 2,
                                       clsConstantes.nALTO_BUTTON));
      // Deshabilitar el botn
      btnSubir.setEnabled(false);
      // Aadir el componente al listener para cuando se pulse el botn
      btnSubir.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            // Llamar al mtodo que gestiona el evento
            oGeneral.Subir(lstAtributos,
                           vAtributos,
                           btnSubir,
                           btnBajar);
         } // Evento: addActionListener
      }); // addActionListener

      // btnAtributos
      // Establecer tamao y posicin
      btnAtributos.setBounds(new Rectangle(lstAtributos.getX(),
                                           lstAtributos.getY() +
                                           lstAtributos.getHeight(),
                                           clsConstantes.nANCHO_BUTTON_LIST,
                                           clsConstantes.nALTO_BUTTON));
      // Deshabilitar el botn
      btnAtributos.setEnabled(false);
      // Aadir el componente al listener para cuando se pulse el botn
      btnAtributos.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            // Llamar al mtodo que gestiona el evento
            AbrirAtributo();
         } // Evento: actionPerformed
      }); // addActionListener

      // btnElimAtrib
      // Establecer tamao y posicin
      btnElimAtrib.setBounds(new Rectangle(btnAtributos.getX() +
                                           clsConstantes.nANCHO_BUTTON_LIST +
                                           clsConstantes.nSEP_COM,
                                           btnAtributos.getY(),
                                           clsConstantes.nANCHO_BUTTON_LIST,
                                           clsConstantes.nALTO_BUTTON));
      // Deshabilitar el botn
      btnElimAtrib.setEnabled(false);
      // Aadir el componente al listener para cuando se pulse el botn
      btnElimAtrib.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            // Llamar al mtodo que gestiona el evento
            oGeneral.EliminarAtributo(lstAtributos,
                                      vAtributos,
                                      btnElimAtrib);
         } // Evento: actionPerformed
      }); // addActionListener

      // lblComentario
      // Establecer tamao y posicin
      lblComentario.setBounds(new Rectangle(btnBajar.getX() +
                                            btnBajar.getWidth() +
                                            clsConstantes.nSEP_COM,
                                            lblAtributos.getY(),
                                            clsConstantes.nANCHO_LABEL,
                                            clsConstantes.nALTO_COM));

      // txaComentario
      // Establecer tamao y posicin
      txaComentario.setBounds(new Rectangle(lblComentario.getX(),
                                            lblComentario.getY() +
                                            lblComentario.getHeight(),
                                            clsConstantes.nANCHO_LIST,
                                            clsConstantes.nALTO_LIST));

      // btnCerrar
      // Establecer tamao y posicin
      btnCerrar.setBounds(new Rectangle(txtDefault.getX() +
                                        txtDefault.getWidth() -
                                        clsConstantes.nANCHO_BUTTON,
                                        btnAtributos.getY() +
                                        btnAtributos.getHeight() +
                                        clsConstantes.nSEP_COM,
                                        clsConstantes.nANCHO_BUTTON,
                                        clsConstantes.nALTO_BUTTON));
      // Aadir el componente al listener para cuando se pulse el botn
      btnCerrar.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            // Llamar al mtodo que gestiona el evento
            Ocultar();
         } // Evento: actionPerformed
      }); // addActionListener

      // btnCrear
      // Establecer tamao y posicin
      btnCrear.setBounds(new Rectangle(btnCerrar.getX() -
                                       clsConstantes.nANCHO_BUTTON_CREAR -
                                       clsConstantes.nSEP_COM,
                                       btnCerrar.getY(),
                                       clsConstantes.nANCHO_BUTTON_CREAR,
                                       clsConstantes.nALTO_BUTTON));
      // Deshabilitar el botn
      btnCrear.setEnabled(false);
      // Aadir el componente al listener para cuando se pulse el botn
      btnCrear.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            // Llamar al mtodo que gestiona el evento
            CrearAtributo();
         } // Evento: actionPerformed
      }); // addActionListener

      // chbCerrar
      // Establecer tamao y posicin
      chbCerrar.setBounds(new Rectangle(clsConstantes.nSEP_VHOR,
                                        btnCerrar.getY() +
                                        btnCerrar.getHeight() +
                                        clsConstantes.nSEP_COM,
                                        clsConstantes.nANCHO_CHECKBOX,
                                        clsConstantes.nALTO_COM));
      // Obtener la opcin para cerrar de la variable de configuracin
      chbCerrar.setState(oConfig.bCerrar);

      // Propiedades de la ventana
      // Establecer el tamao
      setSize(oGeneral.nAnchoVentana(btnCerrar.getX()),
              oGeneral.nAltoVentana(chbCerrar.getY()));
      // Establecer tipo de Layout
      setLayout(null);
      // Impedir que se redimensione la ventana
      setResizable(false);
      // Establecer el icono de la ventana
      setIconImage(oGeneral.imgLogo);
      // Establecer el ttulo de la ventana
      setTitle(sPropietario + ".Nuevo Atributo");
      // Centrar la ventana
      oGeneral.CentrarVentana(this);
      // Indicar que sta es la ventana abierta
      oGeneral.EstablecerActual(this);
      // Aadir los componentes a la ventana
      add(lblNombre);
      add(txtNombre);
      add(lblTipo);
      add(chbAtributoSimple);
      add(chbAtributoDerivado);
      add(chbAtributoLlave);
      add(chbAtributoLlaveParcial);
      add(chbAtributoMultivaluado);
      add(lblDominio);
      add(lblLongitud);
      add(lblEscala);
      add(lblRestricciones);
      add(choDominio);
      add(txtLongitud);
      add(txtEscala);
      add(chbNotNull);
      add(lblNotNull);
      add(txtNotNull);
      add(chbUnique);
      add(lblUnique);
      add(txtUnique);
      add(lblTituloDefault);
      add(chbDefault);
      add(lblDefault);
      add(txtDefault);
      add(chbCheck);
      add(lblNombreCheck);
      add(txtNombreCheck);
      add(lblCheck);
      add(txtCheck);
      add(chbCerrar);
      add(lblAtributos);
      add(lstAtributos);
      add(btnSubir);
      add(btnBajar);
      add(btnAtributos);
      add(btnElimAtrib);
      add(lblComentario);
      add(txaComentario);
      add(btnCerrar);
      add(btnCrear);
      // Aadir la ventana al listener para cuando se cierre la ventana
      addWindowListener(new WindowAdapter() {
         public void windowClosing(WindowEvent e) {
            // Llamar al mtodo que gestiona el evento
            Ocultar();
         } // windowClosing
      }); // addWindowListener
      // Ocultar la ventana propietaria
      fraVentanaProp.setVisible(false);
      // Visualizar la ventana
      setVisible(true);

   } // EVENTO: actionPerformed

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: LimpiarVentana                                           //
   // Parmetros: No                                                          //
   // Descripcin: Establece los valores iniciales de la ventana              //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private void LimpiarVentana() {
      txtNombre.setText("");
      chbAtributoSimple.setState(true);
      choDominio.select(0);
      CambioDominio();
      lblLongitud.setVisible(true);
      txtLongitud.setText("");
      lblEscala.setVisible(true);
      txtEscala.setText("");
      chbNotNull.setState(false);
      txtNotNull.setText("");
      txtNotNull.setEnabled(false);
      chbUnique.setState(false);
      txtUnique.setText("");
      txtUnique.setEnabled(false);
      chbDefault.setState(false);
      txtDefault.setText("");
      txtDefault.setEnabled(false);
      chbCheck.setState(false);
      txtNombreCheck.setText("");
      txtNombreCheck.setEnabled(false);
      txtCheck.setText("");
      txtCheck.setEnabled(false);
      btnSubir.setEnabled(false);
      btnBajar.setEnabled(false);
      setTitle(sPropietario + ".Nuevo Atributo");
      btnAtributos.setEnabled(false);
      lstAtributos.removeAll();
      vAtributos.removeAllElements();
      btnElimAtrib.setEnabled(false);
      txaComentario.setText("");
      btnCrear.setEnabled(false);
      txtNombre.requestFocus();
   } // Procedimiento: LimpiarVentana

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: ActualizarTitulo                                         //
   // Parmetros: No                                                          //
   // Descripcin: Actualiza el ttulo de la ventana con el valor introducido //
   //              para el nombre despus de aplicarle el filtro              //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private void ActualizarTitulo() {
      if (txtNombre.getText().equals("")) {
         setTitle(sPropietario + ".Nuevo Atributo");
      } // if
      else {
         setTitle(sPropietario + "." +
                  oGeneral.sFiltrarNombre(txtNombre.getText()));
      } // else
   } // Procedimiento: ActualizarTitulo

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: DefaultStateChanged                                      //
   // Parmetros: No                                                          //
   // Descripcin: Gestiona el cambio de seleccin del valor por defecto      //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private void DefaultStateChanged() {

      boolean bDefault; // Indicador Default

      // Obtener el valor del indicador Default
      bDefault = chbDefault.getState();
      // Habilitar/Deshabilitar el valor por defecto
      txtDefault.setEnabled(bDefault);
      // Comprobar si se ha seleccionado la opcin por defecto
      if (!bDefault) {
         // Limpiar el valor por defecto
         txtDefault.setText("");
      } // if
   } // Procedimiento: DefaultStateChanged

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: NotNullStateChanged                                      //
   // Parmetros: No                                                          //
   // Descripcin: Gestiona el cambio de seleccin de la restriccin Not Null //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private void NotNullStateChanged() {

      boolean bNotNull; // Indicador NotNull

      // Obtener el valor del indicador Default
      bNotNull = chbNotNull.getState();
      // Habilitar/Deshabilitar el valor por defecto
      txtNotNull.setEnabled(bNotNull);
      // Comprobar si se ha seleccionado la restriccin Not Null
      if (!bNotNull) {
         // Limpiar el nombre de la restriccin Not Null
         txtNotNull.setText("");
      } // if
   } // Procedimiento: NotNullStateChanged

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: UniqueStateChanged                                       //
   // Parmetros: No                                                          //
   // Descripcin: Gestiona el cambio de seleccin de la restriccin Unique   //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private void UniqueStateChanged() {

      boolean bUnique; // Indicador Unique

      // Obtener el valor del indicador Unique
      bUnique = chbUnique.getState();
      // Habilitar/Deshabilitar el valor de la condicin Unique
      txtUnique.setEnabled(bUnique);
      // Comprobar si se ha seleccionado la restriccin Unique
      if (!bUnique) {
         // Limpiar el nombre de la restriccin Unique
         txtUnique.setText("");
      } // if
   } // Procedimiento: UniqueStateChanged

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: CheckStateChanged                                        //
   // Parmetros: No                                                          //
   // Descripcin: Gestiona el cambio de seleccin de la restriccin Check    //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private void CheckStateChanged() {

      boolean bCheck; // Indicador Check

      // Obtener el valor del indicador Check
      bCheck = chbCheck.getState();
      // Habilitar/Deshabilitar el nombre de la restriccin Check
      txtNombreCheck.setEnabled(bCheck);
      // Habilitar/Deshabilitar el valor de la condicin Check
      txtCheck.setEnabled(bCheck);
      // Comprobar si se ha seleccionado la restriccin Check
      if (!bCheck) {
         // Limpiar el nombre de la restriccin Check
         txtNombreCheck.setText("");
         // Limpiar el valor de la condicin Check
         txtCheck.setText("");
      } // if
   } // Procedimiento: CheckStateChanged

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: CrearAtributo                                            //
   // Parmetros: No                                                          //
   // Descripcin: Crea el atributo con los datos introducidos en la ventana  //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private void CrearAtributo() {

      String sNombre; // Nombre del atributo
      int nTipo; // Tipo del atributo
      String sDominio; // Dominio
      boolean bDefault; // Indicador del valor por defecto
      boolean bCheck; // Indicador de la restriccin Check
      clsElemento oAtributo; // Objeto para el atributo
      String sTipo; // Nombre del tipo del atributo

      // Obtener el nombre del atributo
      sNombre = oGeneral.sFiltrarNombre(txtNombre.getText());
      // Obtener el nombre del tipo de atributo
      sTipo = cbgTipo.getSelectedCheckbox().getLabel();
      // Obtener el dominio
      sDominio = sConstruirDominio();
      // Obtener el indicador del valor por defecto
      bDefault = chbDefault.getState();
      // Obtener el indicador de Check
      bCheck = chbCheck.getState();
      // Obtener el tipo de atributo
      if (sTipo.equals(clsConstantes.sATRIBUTO_SIMPLE)) {
         nTipo = clsConstantes.nATRIBUTO_SIMPLE;
      } // if
      else if (sTipo.equals(clsConstantes.sATRIBUTO_MULTIVALUADO)) {
         nTipo = clsConstantes.nATRIBUTO_MULTIVALUADO;
      } // else if
      else if (sTipo.equals(clsConstantes.sATRIBUTO_LLAVE)) {
         nTipo = clsConstantes.nATRIBUTO_LLAVE;
      } // else if
      else if (sTipo.equals(clsConstantes.sATRIBUTO_LLAVE_PARCIAL)) {
         nTipo = clsConstantes.nATRIBUTO_LLAVE_PARCIAL;
      } // else if
      else {
         nTipo = clsConstantes.nATRIBUTO_DERIVADO;
      } // else

      // Aadir un nuevo atributo si se pasan las validaciones
      if (bValidar(sNombre,
                   nTipo)) {
         // Comprobar si el atributo es llave
         if ( (nTipo == clsConstantes.nATRIBUTO_LLAVE) ||
             (nTipo == clsConstantes.nATRIBUTO_LLAVE_PARCIAL)) {
            // Marcar como superllave los componentes simples del atributo
            oGeneral.AsignarSuperllave(vAtributos);
         } // if
         // Crear el objeto para el atributo
         oAtributo = new clsElemento(oGeneral, oConfig.nVerAtributos);

         // Asignar el nombre
         oAtributo.sNombre = sNombre;
         // Asignar el tipo
         oAtributo.nTipo = nTipo;
         // Asignar la coordenada X de la posicin
         oAtributo.nX = oGeneral.nXInicial;
         // Asignar la coordenada Y de la posicin
         oAtributo.nY = oGeneral.nYInicial;
         // Asignar el ancho
         oAtributo.nAncho = clsConstantes.nANCHO_ATRIBUTO;
         // Asignar el alto
         oAtributo.nAlto = clsConstantes.nALTO_ATRIBUTO;
         // Inicializar el vector de atributos (para atributos compuestos)
         oAtributo.vAtributos = (Vector) vAtributos.clone();
         // Asignar el nombre del dominio
         oAtributo.sDominio = sDominio;
         // Asignar el indicador Not Null
         oAtributo.bNotNull = chbNotNull.getState();
         // Asignar el nombre de la restriccin Not Null
         oAtributo.sNotNull = txtNotNull.getText().replaceAll(" ", "_");
         // Asignar el indicador Unique
         oAtributo.bUnique = chbUnique.getState();
         // Asignar el nombre de la restriccin Unique
         oAtributo.sUnique = txtUnique.getText().replaceAll(" ", "_");
         // Asignar el indicador Default
         oAtributo.bDefault = bDefault;
         // Asignar el valor Default
         oAtributo.sDefault = txtDefault.getText();
         // Asignar el estado de Check
         oAtributo.bCheck = bCheck;
         // Asignar el nombre de la restriccin Check
         oAtributo.sNombreCheck = txtNombreCheck.getText().replaceAll(" ", "_");
         // Asignar el valor Check
         if (txtCheck.getText().equals("")) {
            oAtributo.sCheck = "";
         } // if
         else {
            oAtributo.sCheck = "(" + txtCheck.getText() + ")";
         } // else
         // Asignar el comentario
         oAtributo.sComentario = txaComentario.getText();
         // Indicar si forma parte de una llave. Por defecto no
         oAtributo.bSuperllave = false;
         // Aadir al vector de atributos del propietario del atributo
         vAtributosProp.addElement(oAtributo);
         // Aadir a la lista de atributos del propietario del atributo
         lstAtributosProp.add(sNombre);
         // Actualizar la variable global bCerrar con el valor indicado en la ventana
         oConfig.bCerrar = chbCerrar.getState();
         // Comprobar si se debe cerrar la ventana
         if (oConfig.bCerrar) {
            Ocultar();
         } // if
         else {
            LimpiarVentana();
         } // else
      } // if
   } // Procedimiento: CrearAtributo

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Funcin: bValidar                                                       //
   // Parmetros: - psNombre (String): Nombre del atributo                    //
   //             - pnTipo (int): Tipo de atributo                            //
   // Retorno: Devuelve un valor lgico que indica si se han pasado o no      //
   //          todas las validaciones                                         //
   // Descripcin: Comprueba que los datos para crear el atributo son         //
   //              correctos, mostrando un mensaje de error para los datos    //
   //              que no sean vlidos                                        //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private boolean bValidar(String psNombre,
                            int pnTipo) {

      String sLongitud; // Valor de la longitud
      String sEscala; // Valor de la escala
      String sDefault; // Valor por defecto
      String sCheck; // Valor de la restriccin Check
      String sDominio; // Dominio seleccionado
      boolean bDefault; // Indicador del valor por defecto
      boolean bCheck; // Indicador de la restriccin Check

      // Obtener el indicador del valor por defecto
      bDefault = chbDefault.getState();
      // Obtener el indicador de la restriccin Check
      bCheck = chbCheck.getState();
      // Obtener el valor de la longitud
      sLongitud = txtLongitud.getText();
      // Obtener el valor de la escala
      sEscala = txtEscala.getText();
      // Obtener el valor por defecto
      sDefault = txtDefault.getText();
      // Obtener el valor check
      sCheck = txtCheck.getText();
      // Obtener el dominio
      sDominio = choDominio.getSelectedItem();

      // Validar que el nombre no est vaco
      if (psNombre.equals("")) {
         // Mostrar el mensaje de error
         oMensajes.MostrarMensaje(clsConstantes.sERROR_CAMPO_VACIO,
                                  "Nombre",
                                  this);
         // Indicar que no se pasa la validacin
         return false;
      } // if

      // Validar que el atributo no existe ya
      if (oGeneral.bAtributoExistente(psNombre, vAtributosProp)) {
         // Mostrar el mensaje de error
         oMensajes.MostrarMensaje(clsConstantes.sERROR_ELEMENTO_YA_EXISTE,
                                  psNombre,
                                  this);
         // Indicar que no se pasa la validacin
         return false;
      } // if

      // Validar que, si es atributo llave, sus componentes sean atributos simples
      if ( (pnTipo == clsConstantes.nATRIBUTO_LLAVE) &&
          (!oGeneral.bLlaveCompuestaOK(vAtributos))) {
         // Mostrar el mensaje de error
         oMensajes.MostrarMensaje(clsConstantes.sERROR_LLAVE_COMPUESTA,
                                  this);
         // Indicar que no se pasa la validacin
         return false;
      } // if

      // Validar que se ha introducido un nmero para la longitud
      if (!oGeneral.bEsNumero(sLongitud)) {
         // Mostrar el mensaje de error
         oMensajes.MostrarMensaje(clsConstantes.sERROR_NUMERO_REQUERIDO,
                                  "Longitud",
                                  this);
         // Indicar que no se pasa la validacin
         return false;
      } // if

      // Validar el dominio
      if ( (sDominio.equals(clsConstantes.sDOMINIO_VARCHAR2) ||
            sDominio.equals(clsConstantes.sDOMINIO_NCHAR) ||
            sDominio.equals(clsConstantes.sDOMINIO_NVARCHAR2) ||
            sDominio.equals(clsConstantes.sDOMINIO_UROWID) ||
            sDominio.equals(clsConstantes.sDOMINIO_RAW)) &&
          sLongitud.equals("")) {
         // Mostrar el mensaje de error
         oMensajes.MostrarMensaje(clsConstantes.sERROR_CAMPO_VACIO,
                                  "Longitud",
                                  this);
         // Indicar que no se pasa la validacin
         return false;
      } // if
      else if (sDominio.equals(clsConstantes.sDOMINIO_NUMBER)) {
         if (!sLongitud.equals("")) {
            if (!sEscala.equals("")) {
               if (!oGeneral.bEsNumero(sEscala)) {
                  // Mostrar el mensaje de error
                  oMensajes.MostrarMensaje(clsConstantes.
                                           sERROR_NUMERO_REQUERIDO,
                                           "Escala",
                                           this);
                  // Indicar que no se pasa la validacin
                  return false;
               } // if
            } // if
         } // if
         else if (!sEscala.equals("")) {
            // Mostrar el mensaje de error
            oMensajes.MostrarMensaje(clsConstantes.sERROR_ESCALA_LONGITUD,
                                     this);
            // Indicar que no se pasa la validacin
            return false;
         } // else if
      } // if

      // Comprobar si se ha indicado valor por defecto
      if (bDefault) {
         // Comprobar si se ha escrito el valor por defecto
         if (sDefault.equals("")) {
            // Mostrar el mensaje de error
            oMensajes.MostrarMensaje(clsConstantes.sERROR_VALOR_DEFAULT,
                                     this);
            // Indicar que no se pasa la validacin
            return false;
         } // if
      } // if

      // Comprobar si se ha indicado restriccin Check
      if (bCheck) {
         // Comprobar si se ha escrito el valor  de la restriccin Check
         if (sCheck.equals("")) {
            // Mostrar el mensaje de error
            oMensajes.MostrarMensaje(clsConstantes.sERROR_CONDICION_CHECK,
                                     this);
            // Indicar que no se pasa la validacin
            return false;
         } // if
      } // if

      // Indicar que se han pasado todas las validaciones
      return true;
   } // Funcin: bValidar

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: Ocultar                                                  //
   // Parmetros: No                                                          //
   // Descripcin: Lgica necesaria  para cerrar la ventana                   //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private void Ocultar() {
      // Visualizar la ventana propietaria
      fraVentanaProp.setVisible(true);
      // Establecer como ventana actual la ventana propietaria
      oGeneral.EstablecerActual(fraVentanaProp);
      // Liberar los recursos
      dispose();
   } // Procedimiento: Ocultar

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: AbrirAtributo                                            //
   // Parmetros: No                                                          //
   // Descripcin: Abre una nueva ventana para crear un atributo para el      //
   //              atributo                                                   //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private void AbrirAtributo() {
      // Crear una nueva instancia de esta clase
      new clsAtributo(this,
                      vAtributos,
                      lstAtributos,
                      oGeneral,
                      oConfig);

   } // Procedimiento: AbrirAtributo

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: CambioDominio                                            //
   // Parmetros: No                                                          //
   // Descripcin: Habilita o deshabilita los campos Longitu y Escala segun   //
   //              el dominio seleccionado                                    //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private void CambioDominio() {

      String sDominio; // Guarda el dominio seleccionado

      // Obtener el dominio
      sDominio = choDominio.getSelectedItem();

      // Habilitar o deshabilitar el campo Longitud segn el dominio elegido
      txtLongitud.setEnabled(sDominio.equals(clsConstantes.sDOMINIO_CHAR) ||
                             sDominio.equals(clsConstantes.sDOMINIO_VARCHAR2) ||
                             sDominio.equals(clsConstantes.sDOMINIO_NCHAR) ||
                             sDominio.equals(clsConstantes.sDOMINIO_NVARCHAR2) ||
                             sDominio.equals(clsConstantes.sDOMINIO_NUMBER) ||
                             sDominio.equals(clsConstantes.sDOMINIO_RAW) ||
                             sDominio.equals(clsConstantes.sDOMINIO_UROWID));

      // Habilitar o deshabilitar el campo Longitud segn el dominio elegido
      txtEscala.setEnabled(sDominio.equals(clsConstantes.sDOMINIO_NUMBER));
   } // Procedimiento: CambioDominio

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Funcin: sConstruirDominio                                              //
   // Parmetro: No                                                           //
   // Retorno: Cadena con el dominio completo del atributo                    //
   // Descripcin: Construye una cadena con el dominio completo del atributo, //
   //              indicando el tipo de dominio y sus parmetros              //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private String sConstruirDominio() {

      String sDominio; // Dominio del atributo
      String sLongitud; // Valor de la longitud
      String sEscala; // Valor de la escala

      // Obtener el dominio
      sDominio = choDominio.getSelectedItem();
      // Obtener el valor de la longitud
      sLongitud = txtLongitud.getText();
      // Obtener el valor de la escala
      sEscala = txtEscala.getText();

      // Comprobar el tipo de dominio
      if (sDominio.equals(clsConstantes.sDOMINIO_CHAR)) {
         if (sLongitud.equals("")) {
            // Concatenar parmetros al tipo de dominio
            sDominio += "(1)";
         } // if
         else {
            // Concatenar parmetros al tipo de dominio
            sDominio += "(" + sLongitud + ")";
         } // else
      } // else if
      else if ( (sDominio.equals(clsConstantes.sDOMINIO_VARCHAR2)) ||
               (sDominio.equals(clsConstantes.sDOMINIO_NCHAR)) ||
               (sDominio.equals(clsConstantes.sDOMINIO_NVARCHAR2)) ||
               (sDominio.equals(clsConstantes.sDOMINIO_UROWID)) ||
               (sDominio.equals(clsConstantes.sDOMINIO_RAW))) {
         // Concatenar parmetros al tipo de dominio
         sDominio += "(" + sLongitud + ")";
      } // else if
      else if (sDominio.equals(clsConstantes.sDOMINIO_NUMBER)) {
         if (!sLongitud.equals("")) {
            if (sEscala.equals("")) {
               // Concatenar parmetros al tipo de dominio
               sDominio += "(" + sLongitud + ")";
            } // if
            else {
               // Concatenar parmetros al tipo de dominio
               sDominio += "(" + sLongitud + "," + sEscala + ")";
            } // else
         } // if
      } // if

      // Devolver el dominio
      return sDominio;
   } // Funcin: sConstruirDominio

} // Clase: clsAtributo
