package Derytas;

import java.util.*;
import java.awt.*;

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// Clase: clsElemento                                                         //
// Mtodos: - public clsElemento: Constructor de la clase                     //
//          - public void CambiarPos: Actualiza la posicin del elemento      //
//          - public void Pintar: Pinta el elemento en el esquema             //
//          - public void PintarLineas: Pinta las lneas que relacionan el    //
//            elemento con otros elementos del esquema                        //
//          - private clsEntRel erEntRel: Devuelve el objeto clsEntRel para   //
//            la entidad participante en una relacin                         //
//          - private void PintarLineasEntidad: Pinta las lneas que          //
//            relacionan una entidad con otros elementos del esquema          //
//          - private void PintarLineasEspecializacion: Pinta las lneas que  //
//            relacionan una especializacin con otros elementos del esquema  //
//          - private void PintarLineasRelacion: Pinta las lneas que         //
//            relacionan una relacin con otros elementos del esquema         //
//          - private void PintarLineasTipo_U_I: Pinta las lneas que         //
//            relacionan un tipo unin o un tipo interseccin con otros       //
//            elementos del esquema                                           //
// Descripcin: Representa un elemento del esquema de la base de datos        //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////
public class clsElemento {

   // Variables comunes a todos los tipos de elemento
   String sNombre; // Nombre que aparece en el elemento
   int nTipo; // Tipo de elemento
   int nX; // Coordenada X donde se posiciona el elemento
   int nY; // Coordenada Y donde se posiciona el elemento
   int nAncho; // Ancho del elemento
   int nAlto; // Alto del elemento

   // Variables para entidades
   Vector vRelaciones; // Relaciones en las que participa una entidad
   Vector vEspecializaciones; // Relaciones en las que participa una entidad
   Vector vUniones; // Uniones en las que participa una entidad
   Vector vIntersecciones; // Intersecciones en las que participa una entidad

   // Variables para atributos
   String sDominio; // Dominio del atributo
   String sElementoAtributo; // Elemento al que pertenece un atributo
   boolean bNotNull; // Indicador de restriccin NOT NULL
   String sNotNull; // Nombre de la restriccin NOT NULL a la que pertenece
   boolean bUnique; // Indicador de restriccin UNIQUE
   String sUnique; // Nombre de la restriccin UNIQUE a la que pertenece
   boolean bDefault; // Indicador de valor por defecto DEFAULT
   String sDefault; // Valor por defecto
   boolean bCheck; // Indicador de valor por defecto CHECK
   String sNombreCheck; // Nombre de la restriccin CHECK
   String sCheck; // Cndicin de la restriccin CHECK

   Vector vAtributos; // Atributos para Entidades, Atributos Compuestos y Relaciones

   // Variables para especializaciones
   String sSuperclase; // Entidad Superclase de una Esp/Gen
   Vector vSubclases; // Entidades para Esp/Gen y Un/Cat
   String sAtributoEspecializacion; // Nombre para el atributo de la Especializacin (si tiene)

   Vector vEntRel; // Entidades para Relaciones

   // Variables para tipo unin e interseccin
   Vector vSuperclases; // Superclases de una Union
   String sSubclase; // Subclase de una Unin
   boolean bLlavesIguales; // Indica si las llaves de las superclases de una Unin son iguales

   Vector vLlaveSuperclase; // Atributos llave de la Superclase a la que est asociada la Subclase

   boolean bGenerado; // Indica si el cdigo ya ha sido generado
   boolean bSuperllave; // Indica si el atributo es simple de una llave compuesta
   String sLlave; // Nombre de la llave
   String sLlaveExterna; // Nombre de la llave externa

   String sComentario; // Comentario acerca del elemento (entidades y atributos)

   clsGeneral oGeneral; // Objeto clsGeneral del applet
   clsGraficos oGraficos; // Objeto para los grficos

   // Variables grficas
   boolean bAtribIzq; // Indica si los atributos se pintan a la izquierda o a la derecha de la entidad
   int nVerAtributos; // Opcin para mostrar los atributos

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: clsElemento                                              //
   // Parmetros: - (clsGeneral) poGeneral: Objeto de la clase clsGeneral     //
   //               para acceder a los mtodo que sta proporciona            //
   //             - (int) pnVerAtributos: Opcin de configuracin para ver    //
   //               los atributos                                             //
   // Descripcin: Constructor de la clase                                    //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   public clsElemento(clsGeneral poGeneral, int pnVerAtributos) {
      oGeneral = poGeneral;
      nVerAtributos = pnVerAtributos;
      bGenerado = false;
      oGraficos = new clsGraficos();
   } // Procedimiento: clsElemento

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Funcin: erEntidad                                                      //
   // Parmetros: - psNombre (String) : Nombre de una entidad participante en //
   //               una relacin                                              //
   // Retorno: El objeto de la clase clsEntRel para la entidad                //
   // Descripcin: Devuelve el objeto clsEntRel para la entidad participante  //
   //              en una relacin cuyo nombre se recibe como parmetro       //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private clsEntRel erEntidad(String psNombre) {
      // Contador para el bucle
      int iIndice;

      // Inicializar el contador
      iIndice = 0;
      // Recorrer el vector de entidades participantes hasta encontrarla
      while ( (iIndice < vEntRel.size()) &&
             (! ( (clsEntRel) vEntRel.elementAt(iIndice)).sNombre.equals(
         psNombre))) {
         iIndice++;
      } // while

      // Devolver el objeto con la entidad participante
      return (clsEntRel) vEntRel.elementAt(iIndice);
   } // Funcin: erEntidad

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: PintarLineasEntidad                                      //
   // Parmetros: - g (Graphics) : Objeto grfico                             //
   //             - c (Color): Color para pintar                              //
   //             - pnX (int): Posicin horizontal del elemento               //
   //             - pnY (int): Posicin vertical del elemento                 //
   // Descripcin: Pinta las lneas que relacionan una entidad con otros      //
   //              elementos del esquema                                      //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private void PintarLineasEntidad(Graphics g,
                                    Color c,
                                    int pnX,
                                    int pnY) {

      int iIndice; // ndice para recorrer estructuras de datos
      clsElemento elmElemento; // Objeto para contener elementos de las estructuras de datos
      clsEntRel erEntidad; // Objeto para una entidad participante en una relacin

      // Establecer el color para pintar
      g.setColor(c);
      // Pintar las lineas de la entidad a las relaciones en las que participa
      for (iIndice = 0; iIndice < this.vRelaciones.size(); iIndice++) {
         // Obtener una de las relaciones en las que participa la entidad
         elmElemento = oGeneral.elmElemento( (String) vRelaciones.elementAt(
            iIndice));
         // Obtener el objeto correspondiente a la entidad
         erEntidad = elmElemento.erEntidad(this.sNombre);
         // Comprobar si la participacin es total o parcial
         if (erEntidad.sMin.equals("0")) {
            // Llamar al mtodo de la clase de grficos que pinta la lnea
            oGraficos.TrazarLinea(g,
                                  elmElemento.nX,
                                  elmElemento.nY,
                                  elmElemento.nAncho,
                                  elmElemento.nAlto,
                                  pnX,
                                  pnY,
                                  nAncho,
                                  nAlto,
                                  erEntidad.sMin,
                                  erEntidad.sMax);
         } // if
         else {
            // Llamar al mtodo de la clase de grficos que pinta la lnea
            oGraficos.TrazarLineaDoble(g,
                                       2,
                                       elmElemento.nX,
                                       elmElemento.nY,
                                       elmElemento.nAncho,
                                       elmElemento.nAlto,
                                       pnX,
                                       pnY,
                                       nAncho,
                                       nAlto,
                                       erEntidad.sMin,
                                       erEntidad.sMax);
         } // else
      } // for

      // Pintar las lineas de la entidad a las especializaciones que pertenece
      for (iIndice = 0; iIndice < this.vEspecializaciones.size(); iIndice++) {
         // Obtener una de las especializaciones a la que pertenece la entidad
         elmElemento = oGeneral.elmElemento((String) vEspecializaciones.elementAt(iIndice));

         // Pintar lnea doble si la especializacin es total y la entidad es su superclase
         if (elmElemento.sSuperclase.equals(sNombre)) {
            if ( (elmElemento.nTipo == clsConstantes.nESPECIALIZACION_DISJUNTA_TOTAL) ||
                 (elmElemento.nTipo == clsConstantes.nESPECIALIZACION_SOLAPADA_TOTAL)) {
               // Llamar al mtodo de la clase de grficos que pinta la lnea
               oGraficos.TrazarLineaDoble(g,
                                          2,
                                          elmElemento.nX,
                                          elmElemento.nY,
                                          elmElemento.nAncho,
                                          elmElemento.nAlto,
                                          pnX,
                                          pnY,
                                          nAncho,
                                          nAlto,
                                          false);
            } // if
            else {
               // Llamar al mtodo de la clase de grficos que pinta la lnea
               oGraficos.TrazarLinea(g,
                                     elmElemento.nX,
                                     elmElemento.nY,
                                     elmElemento.nAncho,
                                     elmElemento.nAlto,
                                     pnX,
                                     pnY,
                                     nAncho,
                                     nAlto,
                                     false);
            } // else
         } // if
         else {
            // Llamar al mtodo de la clase de grficos que pinta la lnea
            oGraficos.TrazarLinea(g,
                                  elmElemento.nX,
                                  elmElemento.nY,
                                  elmElemento.nAncho,
                                  elmElemento.nAlto,
                                  pnX,
                                  pnY,
                                  nAncho,
                                  nAlto,
                                  true);
         } // else
      } // for

      // Pintar las lineas de la entidad a los tipos unin a los que pertenece
      for (iIndice = 0; iIndice < this.vUniones.size(); iIndice++) {
         // Obtener uno de los tipos unin a los que pertenece la entidad
         elmElemento = oGeneral.elmElemento( (String) vUniones.elementAt(
            iIndice));

         // Pintar lnea doble si el tipo unin es total y la entidad es su subclase
         if (elmElemento.sSubclase.equals(sNombre)) {
            if (elmElemento.nTipo == clsConstantes.nTIPO_UNION_TOTAL) {
               // Llamar al mtodo de la clase de grficos que pinta la lnea
               oGraficos.TrazarLineaDoble(g,
                                          2,
                                          elmElemento.nX,
                                          elmElemento.nY,
                                          elmElemento.nAncho,
                                          elmElemento.nAlto,
                                          pnX,
                                          pnY,
                                          nAncho,
                                          nAlto,
                                          true);
            } // if
            else {
               // Llamar al mtodo de la clase de grficos que pinta la lnea
               oGraficos.TrazarLinea(g,
                                     elmElemento.nX,
                                     elmElemento.nY,
                                     elmElemento.nAncho,
                                     elmElemento.nAlto,
                                     pnX,
                                     pnY,
                                     nAncho,
                                     nAlto,
                                     true);
            } // else
         } // if
         else {
            // Llamar al mtodo de la clase de grficos que pinta la lnea
            oGraficos.TrazarLinea(g,
                                  elmElemento.nX,
                                  elmElemento.nY,
                                  elmElemento.nAncho,
                                  elmElemento.nAlto,
                                  pnX,
                                  pnY,
                                  nAncho,
                                  nAlto,
                                  false);
         } // else
      } // for

      // Pintar las lineas de la entidad a los tipos interseccin a los que pertenece
      for (iIndice = 0; iIndice < this.vIntersecciones.size(); iIndice++) {
         // Obtener uno de los tipos interseccin a los que pertenece la entidad
         elmElemento = oGeneral.elmElemento( (String) vIntersecciones.elementAt(iIndice));

         // Pintar lnea doble si el tipo interseecin es total y la entidad es su subclase
         if (elmElemento.sSubclase.equals(sNombre)) {
            if (elmElemento.nTipo == clsConstantes.nTIPO_INTERSECCION_TOTAL) {
               // Llamar al mtodo de la clase de grficos que pinta la lnea
               oGraficos.TrazarLineaDoble(g,
                                          2,
                                          elmElemento.nX,
                                          elmElemento.nY,
                                          elmElemento.nAncho,
                                          elmElemento.nAlto,
                                          pnX,
                                          pnY,
                                          nAncho,
                                          nAlto,
                                          true);
            } // if
            else {
               // Llamar al mtodo de la clase de grficos que pinta la lnea
               oGraficos.TrazarLinea(g,
                                     elmElemento.nX,
                                     elmElemento.nY,
                                     elmElemento.nAncho,
                                     elmElemento.nAlto,
                                     pnX,
                                     pnY,
                                     nAncho,
                                     nAlto,
                                     true);
            } // else
         } // if
         else {
            // Llamar al mtodo de la clase de grficos que pinta la lnea
            oGraficos.TrazarLinea(g,
                                  elmElemento.nX,
                                  elmElemento.nY,
                                  elmElemento.nAncho,
                                  elmElemento.nAlto,
                                  pnX,
                                  pnY,
                                  nAncho,
                                  nAlto,
                                  false);
         } // else
      } // for

      // Comprobar si es superclse o subclase de otra entidad directamente
      if (! (sSubclase.equals(""))) {
         // Obtener la subclase
         elmElemento = oGeneral.elmElemento(this.sSubclase);
         // Llamar al mtodo de la clase de grficos que pinta la lnea
         oGraficos.TrazarLinea(g,
                               pnX,
                               pnY,
                               nAncho,
                               nAlto,
                               elmElemento.nX,
                               elmElemento.nY,
                               elmElemento.nAncho,
                               elmElemento.nAlto,
                               true);
      } // if
      else if (! (sSuperclase.equals(""))) {
         // Obtener la superclase
         elmElemento = oGeneral.elmElemento(this.sSuperclase);
         // Llamar al mtodo de la clase de grficos que pinta la lnea
         oGraficos.TrazarLinea(g,
                               elmElemento.nX,
                               elmElemento.nY,
                               elmElemento.nAncho,
                               elmElemento.nAlto,
                               pnX,
                               pnY,
                               nAncho,
                               nAlto,
                               true);
      } // else if
   } // Procedimiento: PintarLineasEntidad

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: PintarLineasEspecializacion                              //
   // Parmetros: - g (Graphics) : Objeto grfico                             //
   //             - c (Color): Color para pintar                              //
   //             - pnX (int): Posicin horizontal del elemento               //
   //             - pnY (int): Posicin vertical del elemento                 //
   // Descripcin: Pinta las lneas que relacionan una especializacin con    //
   //              otros elementos del esquema                                //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private void PintarLineasEspecializacion(Graphics g,
                                            Color c,
                                            int pnX,
                                            int pnY) {

      clsElemento oEntidad; // Objeto para contener una entidad

      // Establecer el color con el que se pinta
      g.setColor(c);

      // Obtener la superclase
      oEntidad = oGeneral.elmElemento(sSuperclase);

      // Pintar la lnea hacia la superclase
      if ( ( (nTipo == clsConstantes.nESPECIALIZACION_DISJUNTA_TOTAL) ||
             (nTipo == clsConstantes.nESPECIALIZACION_SOLAPADA_TOTAL)) &&
           (oEntidad.sNombre.equals(sSuperclase))) {
         // Llamar al mtodo de la clase de grficos que pinta la lnea
         oGraficos.TrazarLineaDoble(g,
                                    2,
                                    pnX,
                                    pnY,
                                    nAncho,
                                    nAlto,
                                    oEntidad.nX,
                                    oEntidad.nY,
                                    oEntidad.nAncho,
                                    oEntidad.nAlto,
                                    false);
      } // if
      else {
         // Llamar al mtodo de la clase de grficos que pinta la lnea
         oGraficos.TrazarLinea(g,
                               pnX,
                               pnY,
                               nAncho,
                               nAlto,
                               oEntidad.nX,
                               oEntidad.nY,
                               oEntidad.nAncho,
                               oEntidad.nAlto,
                               false);
      } // else

      // Pintar las lneas hacia las subclases
      for (int i = 0; i < vSubclases.size(); i++) {
         // Obtener una de las subclase
         oEntidad = oGeneral.elmElemento( (String) vSubclases.elementAt(i));
         // Llamar al mtodo de la clase de grficos que pinta la lnea
         oGraficos.TrazarLinea(g,
                               pnX,
                               pnY,
                               nAncho,
                               nAlto,
                               oEntidad.nX,
                               oEntidad.nY,
                               oEntidad.nAncho,
                               oEntidad.nAlto,
                               true);
      } // for
   } // Procedimiento: PintarLineasEspecializacion

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: PintarLineasTipo_U_I                                     //
   // Parmetros: - g (Graphics) : Objeto grfico                             //
   //             - c (Color): Color para pintar                              //
   //             - pnX (int): Posicin horizontal del elemento               //
   //             - pnY (int): Posicin vertical del elemento                 //
   // Descripcin: Pinta las lneas que relacionan un tipo unin o un tipo    //
   //              interseccin con otro elementos del esquema                //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private void PintarLineasTipo_U_I(Graphics g,
                                     Color c,
                                     int pnX,
                                     int pnY) {

      clsElemento oEntidad; // Objeto para contener una entidad

      // Establecer el color con el que se pinta
      g.setColor(c);

      // Obtener la subclase
      oEntidad = oGeneral.elmElemento(sSubclase);

      if ( (nTipo == clsConstantes.nTIPO_UNION_TOTAL) ||
          (nTipo == clsConstantes.nTIPO_INTERSECCION_TOTAL)) {
         // Llamar al mtodo de la clase de grficos que pinta la lnea
         oGraficos.TrazarLineaDoble(g,
                                    2,
                                    pnX,
                                    pnY,
                                    nAncho,
                                    nAlto,
                                    oEntidad.nX,
                                    oEntidad.nY,
                                    oEntidad.nAncho,
                                    oEntidad.nAlto,
                                    true);
      } // if
      else {
         // Llamar al mtodo de la clase de grficos que pinta la lnea
         oGraficos.TrazarLinea(g,
                               pnX,
                               pnY,
                               nAncho,
                               nAlto,
                               oEntidad.nX,
                               oEntidad.nY,
                               oEntidad.nAncho,
                               oEntidad.nAlto,
                               true);
      } // else

      // Pintar las lneas hacia las superclases
      for (int i = 0; i < vSuperclases.size(); i++) {
         oEntidad = oGeneral.elmElemento( (String) vSuperclases.elementAt(i));
         // Llamar al mtodo de la clase de grficos que pinta la lnea
         oGraficos.TrazarLinea(g,
                               pnX,
                               pnY,
                               nAncho,
                               nAlto,
                               oEntidad.nX,
                               oEntidad.nY,
                               oEntidad.nAncho,
                               oEntidad.nAlto,
                               false);
      } // for
   } // Procedimiento: PintarLineasUnion

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: PintarLineasTipo_U_I                                     //
   // Parmetros: - g (Graphics) : Objeto grfico                             //
   //             - c (Color): Color para pintar                              //
   //             - pnX (int): Posicin horizontal del elemento               //
   //             - pnY (int): Posicin vertical del elemento                 //
   // Descripcin: Pinta las lneas que relacionan una relacin con otros     //
   //              elementos del esquema                                      //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private void PintarLineasRelacion(Graphics g,
                                     Color c,
                                     int pnX,
                                     int pnY) {

      int iIndice; // ndice para recorrer el vector de entidades participantes
      clsEntRel erEntidad; // Objeto para una entidad participante en una relacin
      clsElemento elmEntidad; // Objeto para una entidad

      // Establecer el color con el que pintar
      g.setColor(c);

      // Recorrer el vector de entidades participantes
      for (iIndice = 0; iIndice < vEntRel.size(); iIndice++) {
         // Obtener una entidad participante
         erEntidad = (clsEntRel) vEntRel.elementAt(iIndice);
         // Obtener la entidad correspondiente a la entidad participante
         elmEntidad = oGeneral.elmElemento(erEntidad.sNombre);
         // Comprobar si la participacin es parcial o total
         if (erEntidad.sMin.equals("0")) {
            // Llamar al mtodo de la clase de grficos que pinta la lnea
            oGraficos.TrazarLinea(g,
                                  pnX,
                                  pnY,
                                  nAncho,
                                  nAlto,
                                  elmEntidad.nX,
                                  elmEntidad.nY,
                                  elmEntidad.nAncho,
                                  elmEntidad.nAlto,
                                  erEntidad.sMin,
                                  erEntidad.sMax);
         } // if
         else {
            // Llamar al mtodo de la clase de grficos que pinta la lnea
            oGraficos.TrazarLineaDoble(g,
                                       2,
                                       pnX,
                                       pnY,
                                       nAncho,
                                       nAlto,
                                       elmEntidad.nX,
                                       elmEntidad.nY,
                                       elmEntidad.nAncho,
                                       elmEntidad.nAlto,
                                       erEntidad.sMin,
                                       erEntidad.sMax);
         } // else
      } // for
   } // Procedimiento: PintarLineasRelacion

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: Pintar                                                   //
   // Parmetros: - g (Graphics) : Objeto grfico                             //
   //             - c (Color): Color para pintar                              //
   // Descripcin: Pinta el elemento en el color indicado                     //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   public void Pintar(Graphics g,
                      Color c) {
      // Comprobar el tipo de elemento
      if (nTipo == clsConstantes.nENTIDAD) {
         // Llamar al mtodo de la clase de grficos que pinta el elemento
         oGraficos.PintarEntidad(g,
                                 c,
                                 nX,
                                 nY,
                                 sNombre,
                                 nVerAtributos,
                                 vAtributos,
                                 bAtribIzq);
      } // if
      else if (nTipo == clsConstantes.nRELACION) {
         // Llamar al mtodo de la clase de grficos que pinta el elemento
         oGraficos.PintarRelacion(g,
                                  c,
                                  nX,
                                  nY,
                                  sNombre,
                                  nVerAtributos,
                                  vAtributos);
      } // else if
      else if (nTipo == clsConstantes.nENTIDAD_DEBIL) {
         // Llamar al mtodo de la clase de grficos que pinta el elemento
         oGraficos.PintarEntidadDebil(g,
                                      c,
                                      nX,
                                      nY,
                                      sNombre,
                                      nVerAtributos,
                                      vAtributos,
                                      bAtribIzq);
      } // else if
      else if (nTipo == clsConstantes.nRELACION_IDENTIFICATIVA) {
         // Llamar al mtodo de la clase de grficos que pinta el elemento
         oGraficos.PintarRelacionIdentificativa(g,
                                                c,
                                                nX,
                                                nY,
                                                sNombre,
                                                nVerAtributos,
                                                vAtributos);
      } // else if
      else if ( (nTipo == clsConstantes.nESPECIALIZACION_DISJUNTA_TOTAL) ||
                (nTipo == clsConstantes.nESPECIALIZACION_DISJUNTA_PARCIAL)) {
         // Llamar al mtodo de la clase de grficos que pinta el elemento
         oGraficos.PintarEspecializacion(g,
                                         c,
                                         nX,
                                         nY,
                                         "d",
                                         sAtributoEspecializacion);
      } // else if
      else if ( (nTipo == clsConstantes.nESPECIALIZACION_SOLAPADA_TOTAL) ||
                (nTipo == clsConstantes.nESPECIALIZACION_SOLAPADA_PARCIAL)) {
         // Llamar al mtodo de la clase de grficos que pinta el elemento
         oGraficos.PintarEspecializacion(g,
                                         c,
                                         nX,
                                         nY,
                                         "o",
                                         sAtributoEspecializacion);
      } // else if
      else if ( (nTipo == clsConstantes.nTIPO_UNION_TOTAL) ||
                (nTipo == clsConstantes.nTIPO_UNION_PARCIAL) ||
                (nTipo == clsConstantes.nTIPO_INTERSECCION_TOTAL) ||
                (nTipo == clsConstantes.nTIPO_INTERSECCION_PARCIAL)) {
         // Llamar al mtodo de la clase de grficos que pinta el elemento
         oGraficos.PintarTipo_U_I(g,
                                  c,
                                  nX,
                                  nY,
                                  nTipo);
      } // else if
   } // Procedimiento: Pintar

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: PintarLineas                                             //
   // Parmetros: - g (Graphics) : Objeto grfico                             //
   //             - c (Color): Color para pintar                              //
   // Descripcin: Pinta las lneas que relacionan el elemento con otros      //
   //              elementos del esquema                                      //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   public void PintarLineas(Graphics g, Color c) {
      // Comprobar el tipo de elemento
      if ( (nTipo == clsConstantes.nENTIDAD) ||
           (nTipo == clsConstantes.nENTIDAD_DEBIL)) {
         // Llamar al mtodo que pinta las lneas del elemento
         PintarLineasEntidad(g,
                             c,
                             this.nX,
                             this.nY);
      } // if
      else if ( (nTipo == clsConstantes.nRELACION) ||
                (nTipo == clsConstantes.nRELACION_IDENTIFICATIVA)) {
         // Llamar al mtodo que pinta las lneas del elemento
         PintarLineasRelacion(g,
                              c,
                              this.nX,
                              this.nY);
      } // else if
      else if ( (nTipo == clsConstantes.nESPECIALIZACION_DISJUNTA_TOTAL) ||
                (nTipo == clsConstantes.nESPECIALIZACION_DISJUNTA_PARCIAL)) {
         // Llamar al mtodo que pinta las lneas del elemento
         PintarLineasEspecializacion(g,
                                     c,
                                     this.nX,
                                     this.nY);
      } // else if
      else if ( (nTipo == clsConstantes.nESPECIALIZACION_SOLAPADA_TOTAL) ||
                (nTipo == clsConstantes.nESPECIALIZACION_SOLAPADA_PARCIAL)) {
         // Llamar al mtodo que pinta las lneas del elemento
         PintarLineasEspecializacion(g,
                                     c,
                                     this.nX,
                                     this.nY);
      } // else if
      else if ( (nTipo == clsConstantes.nTIPO_UNION_TOTAL) ||
                (nTipo == clsConstantes.nTIPO_UNION_PARCIAL) ||
                (nTipo == clsConstantes.nTIPO_INTERSECCION_PARCIAL) ||
                (nTipo == clsConstantes.nTIPO_INTERSECCION_TOTAL)) {
         // Llamar al mtodo que pinta las lneas del elemento
         PintarLineasTipo_U_I(g,
                              c,
                              this.nX,
                              this.nY);
      } // else if
   } // Procedimiento: PintarLineas

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: CambiarPos                                               //
   // Parmetros: - pnX (int) : Coordenada X para el elemento                 //
   //             - pnY (int) : Coordenada Y para el elemento                 //
   // Descripcin : Procedimiento que actualiza la posicin del elemento con  //
   //               las coordenadas que se le pasan como parmetros           //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   public void CambiarPos(int pnX, int pnY) {
      // Actualizacin de la coordenada X
      nX = pnX;
      // Actualizacin de la coordenada Y
      nY = pnY;
   } // Procedimiento: CambiarPos

} // Clase: clsElemento