package Derytas;

import java.util.*;
import java.awt.*;
import java.awt.List;
import java.awt.event.*;

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// Clase: clsEntidad                                                          //
// Mtodos: - public clsEntidad: Constructor de la clase                      //
//          - private void AbrirAtributo: Abre una nueva ventana para crear   //
//            un atributo                                                     //
//          - private void ActualizarTitulo: Actualiza el ttulo de la        //
//            ventana                                                         //
//          - private boolean bSubclaseConLlave: Comprueba si el tipo de      //
//            atributos es correcto para el tipo de entidad                   //
//          - private boolean bValidar: Comprueba que los datos introducidos  //
//            para la creacin de la entidad son correctos                    //
//          - private void ComprobarEliminarAtributo: Comprobar si se puede   //
//            eliminar el atributo seleccionado al modificar la entidad       //
//          - private void CrearEntidad: Crea la entidad                      //
//          - private void LimpiarVentana: Establece los valores iniciales de //
//            la ventana                                                      //
//          - private void Mostrar: Muestra la ventana segn el modo de       //
//            apertura indicado                                               //
//          - private void Ocultar: Cierra la ventana                         //
// Descripcin: Clase para crear y modificar entidades                        //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////
public class clsEntidad
   extends Frame {

   // Variables de clase
   private clsGeneral oGeneral;
   private clsConfig oConfig;
   private clsMensajes oMensajes;
   private int nModo; // Modo de apertura de la ventana

   // Componentes de la ventana
   private Label lblNombre; // Etiqueta del nombre de la entidad
   private TextField txtNombre; // Nombre de la entidad
   private Checkbox chbDebil; // Indicador de entidad dbil
   private Label lblAtributos; //Etiqueta para los atributos
   private Button btnSubir; // Botn para subir atributos
   private Button btnBajar; // Botn para bajar atributos
   private List lstAtributos; // Lista para los atributos de la entidad
   private Button btnAtributos; // Botn para crear atributos
   private Button btnElimAtrib; // Botn para elminiar atributos
   private Label lblPintarAtributos; // Etiqueta para pintar los atributos
   private CheckboxGroup cbgAtributos; // Indicador para pintar los atributos
   private Checkbox chbAtributoIzq; // Indicador para pintar los atributos a la izquierda
   private Checkbox chbAtributoDer; // Indicador para pintar los atributos a la derecha
   private Label lblComentario; // Etiqueta para los comentarios
   private TextArea txaComentario; // Comentario de la entidad
   private Button btnCrear; // Botn para crear la entidad
   private Button btnCerrar; // Botn para cerrar la ventana
   private Checkbox chbCerrar; // Indicador para cerrar la ventana
   private Vector vAtributos; // Contenedor de los atributos de la entidad
   private String sNombreAntiguo = ""; // Nombre de la entidad antes de modificar

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: clsEntidad                                               //
   // Parmetros: - poGeneral (clsGeneral): Objeto de la clase clsGeneral     //
   //               para poder acceder a los mtodos de carcter general      //
   //             - poConfig (clsConfig): Objeto de la clase clsConfig para   //
   //               poder acceder a las variables de configuracin            //
   //             - pnModo (int): Modo de acceso a la ventana                 //
   // Descripcin: Constructor de la clase                                    //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   public clsEntidad(clsGeneral poGeneral,
                     clsConfig poConfig,
                     int pnModo) {

      // Inicializacin de variables

      // Asignar el objeto de la clase clsGeneral
      oGeneral = poGeneral;
      // Asignar el objeto de la clase clsConfig
      oConfig = poConfig;
      // Obtener el modo de acceso
      nModo = pnModo;
      // Crear el objeto para la clase de mensajes
      oMensajes = new clsMensajes(oGeneral);

      // Crear los componentes de la ventana
      lblNombre = new Label("Nombre de la Entidad");
      txtNombre = new TextField();
      chbDebil = new Checkbox("Entidad Dbil");
      lblAtributos = new Label("Lista de Atributos");
      btnSubir = new Button("Subir");
      btnBajar = new Button("Bajar");
      lstAtributos = new List();
      chbCerrar = new Checkbox("Cerrar despus de Crear");
      btnCrear = new Button("Crear Entidad");
      btnCerrar = new Button("Cancelar");
      btnAtributos = new Button("Crear Atributos");
      btnElimAtrib = new Button("Eliminar Atributo");
      lblPintarAtributos = new Label("Pintar Atributos");
      cbgAtributos = new CheckboxGroup();
      chbAtributoIzq = new Checkbox(clsConstantes.sATRIBUTO_IZQUIERDA,
                                    cbgAtributos, true);
      chbAtributoDer = new Checkbox(clsConstantes.sATRIBUTO_DERECHA,
                                    cbgAtributos, false);
      lblComentario = new Label("Comentario");
      txaComentario = new TextArea("", 0, 0, TextArea.SCROLLBARS_NONE);
      // Iniciar el vector de atributos segn el modo de acceso
      if (nModo == clsConstantes.nMODO_ALTA) {
         vAtributos = new Vector();
      } // if
      else {
         vAtributos = (Vector) oGeneral.oModificar.vAtributos.clone();
      } // else

      // Propiedades de los componentes de la ventana

      // lblNombre
      // Establecer tamao y posicin
      lblNombre.setBounds(new Rectangle(clsConstantes.nSEP_VHOR,
                                        clsConstantes.nSEP_VVER,
                                        clsConstantes.nANCHO_LABEL,
                                        clsConstantes.nALTO_COM));

      // txtNombre
      // Establecer tamao y posicin
      txtNombre.setBounds(new Rectangle(lblNombre.getX(),
                                        lblNombre.getY() +
                                        lblNombre.getHeight(),
                                        clsConstantes.nANCHO_TEXTFIELD,
                                        clsConstantes.nALTO_COM));
      // Controlar el teclado
      txtNombre.addKeyListener(new KeyListener() {

         String sNombre; // Nombre de la entidad

         public void keyTyped(KeyEvent e) {
            //  No hace falta
         }; // keyTyped

         public void keyPressed(KeyEvent e) {
            if ( (e.getKeyCode() == e.VK_ENTER) &&
                (! (txtNombre.getText().equals("")))) {
               // Crear la entidad cuando se pulsa ENTER sobre el nombre
               CrearEntidad();
            } // if
            else {
               // Deshabilitar el botn Crear si no hay nombre escrito
               btnCrear.setEnabled(! (txtNombre.getText().equals("")));
               // Habilitar el botn para crear atributos cuando lo est Crear
               btnAtributos.setEnabled(btnCrear.isEnabled());
               // Comprobar si se excede el mximo de caracteres para el nombre
               if (txtNombre.getText().length() <= clsConstantes.nMAX_NOMBRE) {
                  // Guardar el nombre escrito hasta el momento
                  sNombre = txtNombre.getText();
                  // Actualizar el ttulo de la ventana
                  ActualizarTitulo();
               } // if
               else {
                  txtNombre.setText(sNombre);
               } // else
            } // else
         } // keyPressed

         public void keyReleased(KeyEvent e) {
            // Deshabilitar el botn Crear si no hay nombre escrito
            btnCrear.setEnabled(! (txtNombre.getText().equals("")));
            // Habilitar el botn para crear atributos cuando lo est Crear
            btnAtributos.setEnabled(btnCrear.isEnabled());
            // Comprobar si se excede el mximo de caracteres para el nombre
            if (txtNombre.getText().length() <= clsConstantes.nMAX_NOMBRE) {
               // Guardar el nombre escrito hasta el momento
               sNombre = txtNombre.getText();
               // Actualizar el ttulo de la ventana
               ActualizarTitulo();
            } // if
            else {
               txtNombre.setText(sNombre);
            } // else
         } // keyReleased
      });  // addKeyListener

      // chbDebil
      // Establecer tamao y posicin
      chbDebil.setBounds(new Rectangle(clsConstantes.nSEP_VHOR,
                                       txtNombre.getY() +
                                       txtNombre.getHeight() +
                                       clsConstantes.nSEP_COM,
                                       clsConstantes.nANCHO_CHECKBOX,
                                       clsConstantes.nALTO_COM));

      // lblAtributos
      // Establecer tamao y posicin
      lblAtributos.setBounds(new Rectangle(clsConstantes.nSEP_VHOR,
                                           chbDebil.getY() +
                                           chbDebil.getHeight() +
                                           clsConstantes.nSEP_COM,
                                           100,
                                           clsConstantes.nALTO_COM));

      // btnBajar
      // Establecer tamao y posicin
      btnBajar.setBounds(new Rectangle(clsConstantes.nSEP_VHOR +
                                       clsConstantes.nANCHO_LIST -
                                       clsConstantes.nANCHO_BUTTON / 2,
                                       lblAtributos.getY() -
                                       clsConstantes.nSEP_COM,
                                       clsConstantes.nANCHO_BUTTON / 2,
                                       clsConstantes.nALTO_BUTTON));
      // Deshabilitar el botn
      btnBajar.setEnabled(false);
      // Aadir el componente al listener para cuando se pulse el botn
      btnBajar.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            // Llamar al mtodo que gestiona el evento
            oGeneral.Bajar(lstAtributos,
                           vAtributos,
                           btnSubir,
                           btnBajar);
         } // Evento: actionPerformed
      }); // addActionListener

      // btnSubir
      btnSubir.setBounds(new Rectangle(btnBajar.getX() -
                                       clsConstantes.nANCHO_BUTTON / 2 -
                                       clsConstantes.nSEP_COM,
                                       lblAtributos.getY() -
                                       clsConstantes.nSEP_COM,
                                       clsConstantes.nANCHO_BUTTON / 2,
                                       clsConstantes.nALTO_BUTTON));
      // Deshabilitar el botn
      btnSubir.setEnabled(false);
      // Aadir el componente al listener para cuando se pulse el botn
      btnSubir.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            // Llamar al mtodo que gestiona el evento
            oGeneral.Subir(lstAtributos,
                           vAtributos,
                           btnSubir,
                           btnBajar);
         } // Evento: actionPerformed
      });  // addActionListener

      // lstAtributos
      // Establecer tamao y posicin
      lstAtributos.setBounds(new Rectangle(clsConstantes.nSEP_VHOR,
                                           lblAtributos.getY() +
                                           lblAtributos.getHeight(),
                                           clsConstantes.nANCHO_LIST,
                                           clsConstantes.nALTO_LIST));
      // Aadir el componente al listener para cuando se cambie la seleccin
      lstAtributos.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            // Llamar al mtodo que gestiona el evento
            oGeneral.ListaClick(btnElimAtrib,
                                lstAtributos,
                                btnSubir,
                                btnBajar);
         } // Evento: actionPerformed
      }); // addActionListener

      // btnAtributos
      // Establecer tamao y posicin
      btnAtributos.setBounds(new Rectangle(clsConstantes.nSEP_VHOR,
                                           lstAtributos.getY() +
                                           lstAtributos.getHeight(),
                                           clsConstantes.
                                           nANCHO_BUTTON_LIST,
                                           clsConstantes.nALTO_BUTTON));
      // Deshabilitar el botn
      btnAtributos.setEnabled(false);
      // Aadir el componente al listener para cuando se pulse el botn
      btnAtributos.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            // Llamar al mtodo que gestiona el evento
            AbrirAtributo();
         } // Evento: actionPerformed
      }); // addActionListener

      // btnElimAtrib
      btnElimAtrib.setBounds(new Rectangle(btnAtributos.getX() +
                                           clsConstantes.
                                           nANCHO_BUTTON_LIST +
                                           clsConstantes.nSEP_COM,
                                           btnAtributos.getY(),
                                           clsConstantes.
                                           nANCHO_BUTTON_LIST,
                                           clsConstantes.nALTO_BUTTON));
      // Deshabilitar el botn
      btnElimAtrib.setEnabled(false);
      // Aadir el componente al listener para cuando se pulse el botn
      btnElimAtrib.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            // Llamar al mtodo que gestiona el evento
            ComprobarEliminarAtributo();
         } // Evento: actionPerformed
      }); // addActionListener

      // lblPintarAtributos
      // Establecer tamao y posicin
      lblPintarAtributos.setBounds(new Rectangle(clsConstantes.nSEP_VHOR,
                                                 btnAtributos.getY() +
                                                 btnAtributos.getHeight() +
                                                 clsConstantes.nSEP_COM,
                                                 clsConstantes.nANCHO_LABEL,
                                                 clsConstantes.nALTO_COM));

      // chbAtributoIzq
      // Establecer tamao y posicin
      chbAtributoIzq.setBounds(new Rectangle(clsConstantes.nSEP_VHOR,
                                             lblPintarAtributos.getY() +
                                             lblPintarAtributos.getHeight(),
                                             100,
                                             clsConstantes.nALTO_COM));

      // chbAtributoDer
      // Establecer tamao y posicin
      chbAtributoDer.setBounds(new Rectangle(btnElimAtrib.getX(),
                                             chbAtributoIzq.getY(),
                                             100,
                                             clsConstantes.nALTO_COM));

      // lblComentario
      // Establecer tamao y posicin
      lblComentario.setBounds(new Rectangle(clsConstantes.nSEP_VHOR,
                                            chbAtributoDer.getY() +
                                            chbAtributoDer.getHeight() +
                                            clsConstantes.nSEP_COM,
                                            100,
                                            clsConstantes.nALTO_COM));
      // txaComentario
      // Establecer tamao y posicin
      txaComentario.setBounds(new Rectangle(lblComentario.getX(),
                                            lblComentario.getY() +
                                            lblComentario.getHeight(),
                                            clsConstantes.nANCHO_LIST,
                                            clsConstantes.nALTO_LIST));

      // btnCerrar
      // Establecer tamao y posicin
      btnCerrar.setBounds(new Rectangle(clsConstantes.nSEP_VHOR +
                                        clsConstantes.nANCHO_LIST -
                                        clsConstantes.nANCHO_BUTTON,
                                        txaComentario.getY() +
                                        txaComentario.getHeight() +
                                        clsConstantes.nSEP_COM,
                                        clsConstantes.nANCHO_BUTTON,
                                        clsConstantes.nALTO_BUTTON));
      // Aadir el componente al listener para cuando se pulse el botn
      btnCerrar.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            // Llamar al mtodo que gestiona el evento
            Ocultar();
         } // Evento: actionPerformed
      }); // addActionListener

      // btnCrear
      // Establecer tamao y posicin
      btnCrear.setBounds(new Rectangle(btnCerrar.getX() -
                                       clsConstantes.nANCHO_BUTTON_CREAR -
                                       clsConstantes.nSEP_COM,
                                       btnCerrar.getY(),
                                       clsConstantes.nANCHO_BUTTON_CREAR,
                                       clsConstantes.nALTO_BUTTON));
      // Aadir el componente al listener para cuando se pulse el botn
      btnCrear.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            // Llamar al mtodo que gestiona el evento
            CrearEntidad();
         } // Evento: actionPerformed
      });  // addActionListener

      // chbCerrar
      // Establecer tamao y posicin
      chbCerrar.setBounds(new Rectangle(clsConstantes.nSEP_VHOR,
                                        btnCerrar.getY() +
                                        btnCerrar.getHeight() +
                                        clsConstantes.nSEP_COM,
                                        clsConstantes.nANCHO_CHECKBOX,
                                        clsConstantes.nALTO_COM));
      // Obtener la opcin para cerrar de la variable de configuracin
      chbCerrar.setState(oConfig.bCerrar);

      // Propiedades de la ventana
      // Establecer el tamao
      setSize(oGeneral.nAnchoVentana(btnCerrar.getX()),
              oGeneral.nAltoVentana(chbCerrar.getY()));
      // Establecer tipo de Layout
      setLayout(null);
      // Impedir que se redimensione la ventana
      setResizable(false);
      // Establecer el icono de la ventana
      setIconImage(oGeneral.imgLogo);
      // Centrar la ventana
      oGeneral.CentrarVentana(this);
      // Indicar que sta es la ventana actual
      oGeneral.EstablecerActual(this);
      // Aadir los componenentes a la ventana
      add(lblNombre);
      add(txtNombre);
      add(chbDebil);
      add(lblAtributos);
      add(btnSubir);
      add(btnBajar);
      add(lstAtributos);
      add(btnAtributos);
      add(btnElimAtrib);
      add(lblPintarAtributos);
      add(chbAtributoIzq);
      add(chbAtributoDer);
      add(lblComentario);
      add(txaComentario);
      add(btnCrear);
      add(btnCerrar);
      add(chbCerrar);
      // Aadir la ventana al listener para cuando se cierre la ventana
      addWindowListener(new WindowAdapter() {
         public void windowClosing(WindowEvent e) {
            // Llamar al mtodo que gestiona el evento
            Ocultar();
         } // windowClosing
      }); // addWindowListener
      // Mostrar los datos de la ventana segn el modo de acceso
      Mostrar();
      // Mostrar la ventana
      setVisible(true);
   } // Procedimiento: clsEntidad

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: Mostrar                                                  //
   // Parmetros: No                                                          //
   // Descripcin: Limpia los campos de la ventana o los informa con los      //
   //              datos de una entidad segun el modo de acceso               //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private void Mostrar() {
      // Indicar que hay una ventana abierta
      oGeneral.bVentanaAbierta = true;
      // Comprobar el modo de acceso
      if (nModo == clsConstantes.nMODO_ALTA) {
         // Limpiar la ventana
         LimpiarVentana();
      } // if
      else {
         // Cambiar el ttulo del botn de crear
         btnCrear.setLabel("Modificar Entidad");
         // Cambiar el ttulo de la check de cerrar la ventana
         chbCerrar.setLabel("Cerrar despus de Modificar");
         // Cerrar la ventana tras modificar
         chbCerrar.setState(true);
         // Impedir que no se cierre la ventana, inhabilitando la check
         chbCerrar.setEnabled(false);
         // Guardar el nombre original
         sNombreAntiguo = oGeneral.oModificar.sNombre;
         // Mostrar el Nombre
         txtNombre.setText(sNombreAntiguo);
         // Identificar si es o no Entidad Dbil
         if (oGeneral.oModificar.nTipo == clsConstantes.nENTIDAD_DEBIL) {
            chbDebil.setState(true);
         } // if
         // Mostrar los atributos
         for (int i = 0; i < vAtributos.size(); i++) {
            lstAtributos.add( ( (clsElemento) vAtributos.elementAt(i)).
                             sNombre);
         } // for
         btnAtributos.setEnabled(true);
         // Marcar el dibujo de los atributos
         if (oGeneral.oModificar.bAtribIzq) {
            chbAtributoIzq.setState(true);
         } // if
         else {
            chbAtributoDer.setState(true);
         } // else
         // Mostrar las observaciones
         txaComentario.setText(oGeneral.oModificar.sComentario);
         // Habilitar el botn para crear la entidad
         btnCrear.setEnabled(true);
         // Cambiar el ttulo de la ventana
         setTitle("Entidad: " + oGeneral.oModificar.sNombre);
      } // if
   } // Procedimiento: Mostrar

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: Ocultar                                                  //
   // Parmetros: No                                                          //
   // Descripcin: Lgica necesaria  para cerrar la ventana                   //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private void Ocultar() {
      // Limpiar la ventana actual
      oGeneral.EstablecerActual(null);
      // Guardar si las ventanas deben cerrarse o no despus de crear
      oConfig.bCerrar = chbCerrar.getState();
      // Indicar que no hay ninguna ventana abierta
      oGeneral.bVentanaAbierta = false;
      // Ocultar la ventana
      this.dispose();
   } // Procedimiento: Ocultar

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: LimpiarVentana                                           //
   // Parmetros: No                                                          //
   // Descripcin: Establece los valores iniciales de la ventana              //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private void LimpiarVentana() {
      // Limpiar el nombre de la entidad
      txtNombre.setText("");
      // Limpiar el indicador de entidad dbil
      chbDebil.setState(false);
      // Iniciar el ttulo de la ventana
      setTitle("Nueva Entidad");
      // Deshabilitar el botn para crear atributos
      btnAtributos.setEnabled(false);
      // Deshabilitar el botn para subir atributos
      btnSubir.setEnabled(false);
      // Deshabilitar el botn para bajar atributos
      btnBajar.setEnabled(false);
      // Limpiar la lista de atributos
      lstAtributos.removeAll();
      // Limpiar el comentario de la entidad
      txaComentario.setText("");
      // Deshabilitar el botn para crear la entidad
      btnCrear.setEnabled(false);
      // Deshabilitar el botn para eliminar atributos
      btnElimAtrib.setEnabled(false);
      // Limpiar el vector de atributos
      vAtributos.removeAllElements();
      // Poner el foco en el nombre de la entidad
      txtNombre.requestFocus();
   } // Procedimiento: LimpiarVentana

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: ActualizarTitulo                                         //
   // Parmetros: No                                                          //
   // Descripcin: Actualiza el ttulo de la ventana con el valor introducido //
   //              para el nombre despus de aplicarle el filtro              //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private void ActualizarTitulo() {
      if (txtNombre.getText().equals("")) {
         setTitle("Nueva Entidad");
      } // if
      else {
         setTitle("Entidad: " +
                  oGeneral.sFiltrarNombre(txtNombre.getText()));
      } // else
   } // Procedimiento: ActualizarTitulo

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: CrearEntidad                                             //
   // Parmetros: No                                                          //
   // Descripcin: Crea la entidad con los datos introducidos en la ventana   //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private void CrearEntidad() {

      String sNombre; // Nombre de la entidad
      int nTipo; // Tipo de entidad
      clsElemento elmElemento; // Elemento con la nueva entidad
      clsEntRel erEntidad; // Objeto para entidad participante en relacin
      int iBuscar; // ndice para buscar la entidad
      boolean bBuscar; // Indica si se sigue buscando la entidad

      // Obtener el nombre de la entidad
      sNombre = oGeneral.sFiltrarNombre(txtNombre.getText());

      // Determinar el tipo de entidad
      if (!chbDebil.getState()) {
         nTipo = clsConstantes.nENTIDAD;
      } // if
      else {
         nTipo = clsConstantes.nENTIDAD_DEBIL;
      } // else

      // Aadir una nueva entidad si pasa las validaciones
      if (bValidar(sNombre,
                   nTipo)) {
         // Asignar la entidad como propietaria de los atributos
         oGeneral.AsignarPropietario(sNombre,
                                     vAtributos);

         if (nModo == clsConstantes.nMODO_ALTA) {
            // Crear el objeto para la nueva entidad
            elmElemento = new clsElemento(oGeneral,
                                          oConfig.nVerAtributos);
            // Asignar el nombre del elemento
            elmElemento.sNombre = sNombre;
            // Asignar el tipo de elemento
            elmElemento.nTipo = nTipo;
            // Asignar la coordenada X de la posicin del elemento
            elmElemento.nX = oGeneral.nXInicial;
            // Asignar la coordenada Y de la posicin del elemento
            elmElemento.nY = oGeneral.nYInicial;
            // Buscar una posicin libre en el esquema
            while (oGeneral.bEnElemento(elmElemento.nX,
                                        elmElemento.nY,
                                        false)) {
               // Asigna la coordenada X de la posicin del elemento
               elmElemento.nX += clsConstantes.nSEP_COM;
               // Asigna la coordenada Y de la posicin del elemento
               elmElemento.nY += clsConstantes.nSEP_COM;
            } // while

            // Asignar el ancho del elemento
            elmElemento.nAncho = clsConstantes.nANCHO_ENTIDAD;
            // Asignar el alto del elemento
            elmElemento.nAlto = clsConstantes.nALTO_ENTIDAD;
            // Inicializar el vector de atributos
            elmElemento.vAtributos = (Vector) vAtributos.clone();
            // Inicializar el vector de elementos relacionados
            elmElemento.vRelaciones = new Vector();
            elmElemento.vEspecializaciones = new Vector();
            elmElemento.vUniones = new Vector();
            elmElemento.vIntersecciones = new Vector();
            elmElemento.vLlaveSuperclase = new Vector();
            elmElemento.sSubclase = "";
            elmElemento.sSuperclase = "";
            elmElemento.sLlave = oGeneral.sIdentificarLlave(elmElemento.vAtributos);
            elmElemento.sLlaveExterna = "";
            elmElemento.bAtribIzq = (cbgAtributos.getSelectedCheckbox().
                                     getLabel().equals(clsConstantes.
               sATRIBUTO_IZQUIERDA));
            // Almacenar el comentario
            elmElemento.sComentario = txaComentario.getText();
            // Aadir la entidad al vector de elementos del esquema
            oGeneral.vElementos.addElement(elmElemento);
         } // if
         else {
            // Recuperar los datos de la ventana
            oGeneral.oModificar.sNombre = sNombre;
            oGeneral.oModificar.nTipo = nTipo;
            oGeneral.oModificar.vAtributos = (Vector) vAtributos.clone();
            oGeneral.oModificar.sLlave = oGeneral.sIdentificarLlave(oGeneral.
               oModificar.
               vAtributos);
            oGeneral.oModificar.sLlaveExterna = "";
            oGeneral.oModificar.bAtribIzq = (cbgAtributos.getSelectedCheckbox().
                                               getLabel().equals(clsConstantes.
               sATRIBUTO_IZQUIERDA));
            oGeneral.oModificar.sComentario = txaComentario.getText();

            // Comprobar si se ha modificado el nombre
            if (!sNombre.equals(sNombreAntiguo)) {
               // Actualizar el nombre en las relaciones en las que participa
               for (int iIndice = 0; iIndice < oGeneral.oModificar.vRelaciones.size(); iIndice++) {
                  // Obtener la relacin
                  elmElemento = oGeneral.elmElemento( (String) oGeneral.oModificar.vRelaciones.elementAt(iIndice));
                  // Obtener el objeto con la entidad participante
                  erEntidad = oGeneral.erObtenerEntRel(elmElemento.vEntRel,
                                                       sNombreAntiguo);
                  // Actualizar el nombre
                  erEntidad.sNombre = sNombre;
               } // for

               // Actualizar el nombre en las especializaciones a las que pertenece
               for (int iIndice = 0; iIndice < oGeneral.oModificar.vEspecializaciones.size(); iIndice++) {
                  // Obtener la especializacin
                  elmElemento = oGeneral.elmElemento( (String) oGeneral.oModificar.vEspecializaciones.elementAt(iIndice));
                  // Actualizar el nombre
                  if (elmElemento.sSuperclase.equals(sNombreAntiguo)) {
                     elmElemento.sSuperclase = sNombre;
                  } // if
                  else {
                     iBuscar = 0;
                     bBuscar = true;
                     while ( (bBuscar) && (iBuscar < elmElemento.vSubclases.size())) {
                        if ( ( (String) elmElemento.vSubclases.elementAt(
                           iBuscar)).
                            equals(sNombreAntiguo)) {
                           bBuscar = false;
                           elmElemento.vSubclases.removeElementAt(iBuscar);
                           elmElemento.vSubclases.add(sNombre);
                        } // if
                        iBuscar++;
                     } // while
                  } // else
               } // for

               // Actualizar el nombre en los tipos unin a los que pertenece
               for (int iIndice = 0; iIndice < oGeneral.oModificar.vUniones.size(); iIndice++) {
                  // Obtener el tipo unin
                  elmElemento = oGeneral.elmElemento( (String) oGeneral.oModificar.vUniones.elementAt(iIndice));
                  // Actualizar el nombre
                  if (elmElemento.sSubclase.equals(sNombreAntiguo)) {
                     elmElemento.sSubclase = sNombre;
                  } // if
                  else {
                     iBuscar = 0;
                     bBuscar = true;
                     while ( (bBuscar) &&
                            (iBuscar < elmElemento.vSuperclases.size())) {
                        if ( ( (String) elmElemento.vSuperclases.
                              elementAt(iBuscar)).
                            equals(sNombreAntiguo)) {
                           bBuscar = false;
                           elmElemento.vSuperclases.removeElementAt(iBuscar);
                           elmElemento.vSuperclases.add(sNombre);
                        } // if
                        iBuscar++;
                     } // while
                  } // else
               } // for

               // Actualizar el nombre en los tipos interseccin a los que pertenece
               for (int iIndice = 0; iIndice < oGeneral.oModificar.vIntersecciones.size();
                    iIndice++) {
                  // Obtener el tipo interseccin
                  elmElemento = oGeneral.elmElemento( (String) oGeneral.oModificar.vIntersecciones.elementAt(iIndice));
                  // Actualizar el nombre
                  if (elmElemento.sSubclase.equals(sNombreAntiguo)) {
                     elmElemento.sSubclase = sNombre;
                  } // if
                  else {
                     iBuscar = 0;
                     bBuscar = true;
                     while ( (bBuscar) &&
                            (iBuscar < elmElemento.vSuperclases.size())) {
                        if ( ( (String) elmElemento.vSuperclases.
                              elementAt(iBuscar)).
                            equals(sNombreAntiguo)) {
                           bBuscar = false;
                           elmElemento.vSuperclases.removeElementAt(iBuscar);
                           elmElemento.vSuperclases.add(sNombre);
                        } // if
                        iBuscar++;
                     } // while
                  } // else
               } // for
            } // if
         } // else
         oGeneral.oModificar = null;

         // Actualizar la variable global bCerrar con el valor de la ventana
         oConfig.bCerrar = chbCerrar.getState();

         // Comprobar si se debe ocultar la ventana
         if (oConfig.bCerrar) {
            // Ocultar la ventana
            Ocultar();
         } // if
         else {
            // Limpiar la ventana
            LimpiarVentana();
         } // else

         // Actualizar el esquema
         oGeneral.Repintar();
      } // if
   } // Procedimiento: CrearEntidad

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Funcin: bValidar                                                       //
   // Parmetros: - psNombre (String): Nombre del atributo                    //
   //             - pnTipo (int): Tipo de entidad                             //
   // Retorno: Devuelve un valor lgico que indica si se han pasado o no      //
   //          todas las validaciones                                         //
   // Descripcin: Comprueba que los datos para crear la entidad son          //
   //              correctos, mostrando un mensaje de error para los datos    //
   //              que no sean vlidos                                        //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private boolean bValidar(String psNombre,
                            int nTipo) {

      int iIndice; // ndice para recorrer el vector de atributos
      clsElemento oAtributo; // Objeto para contener un atributo

      psNombre = txtNombre.getText();

      // Validar que el nombre no est vaco
      if (psNombre.equals("")) {
         oMensajes.MostrarMensaje(clsConstantes.sERROR_CAMPO_VACIO,
                                  "Nombre",
                                  this);
      } // if

      // Validar que el elemento no exista ya (si no es modificacin)
      if ( (oGeneral.oModificar == null) ||
          ( (oGeneral.oModificar != null) && (!psNombre.equals(sNombreAntiguo)))) {
         if (oGeneral.bExisteElemento(psNombre, nTipo, oGeneral.vElementos)) {
            oMensajes.MostrarMensaje(clsConstantes.sERROR_ELEMENTO_YA_EXISTE,
                                     psNombre,
                                     this);
            return false;
         }
      } // if

      // Validar que los tipos de atributos son correctos para el tipo de entidad
      for (iIndice = 0; iIndice < vAtributos.size(); iIndice++) {
         oAtributo = (clsElemento) vAtributos.elementAt(iIndice);
         if (nTipo == clsConstantes.nENTIDAD){
             if(oAtributo.nTipo == clsConstantes.nATRIBUTO_LLAVE_PARCIAL) {
                oMensajes.MostrarMensaje(clsConstantes.sERROR_LLAVE_PARCIAL, this);
                return false;
             } // if
             // Comprobar si se est modificando
             if (nModo == clsConstantes.nMODO_MODIFICAR){
                if (bSubclaseConLlave(oAtributo.nTipo)){
                   oMensajes.MostrarMensaje(clsConstantes.sERROR_SUBCLASE_CON_LLAVE, this);
                   return false;
                } // if
             } // if
         } // if
         if ( (nTipo == clsConstantes.nENTIDAD_DEBIL) &&
             (oAtributo.nTipo == clsConstantes.nATRIBUTO_LLAVE)) {
            oMensajes.MostrarMensaje(clsConstantes.sERROR_LLAVE_PRIMARIA, this);
            return false;
         } // if
      } // for

      // Llegar a este punto implica haber pasado todas la validaciones
      return true;

   } // Funcin: bValidar

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: AbrirAtributo                                            //
   // Parmetros: No                                                          //
   // Descripcin: Abre una nueva ventana para crear un atributo para la      //
   //              entidad                                                    //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private void AbrirAtributo(){
      // Crear una nueva instancia de la clase clsAtributo
      new clsAtributo(this,
                            vAtributos,
                            lstAtributos,
                            oGeneral,
                            oConfig);
   } // Procedimiento: AbrirAtributo

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: ComprobarEliminarAtributo                                //
   // Parmetros: No                                                          //
   // Descripcin: Comprueba si se puede modificar la entidad eliminando el   //
   //              atributo seleccionado                                      //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private void ComprobarEliminarAtributo() {

      clsElemento oAtributo; // Objeto para contener un atributo
      int nLlave = 0; // Indica el nmero de atributos llave (por defecto 0)
      String sAtributo; // Nombre del atributo seleccionado
      int iIndice; // ndice para recorrer el vector de atributos
      boolean bLlave = false; // Indica si el atributo a eliminar es llave

      // Comprobar que la ventana se ha abierto para modificar
      if (this.nModo == clsConstantes.nMODO_MODIFICAR) {
         // Obtener el nombre del atributo seleccionado
         sAtributo = lstAtributos.getSelectedItem();
         iIndice = 0;
         while ( (iIndice < vAtributos.size()) && (nLlave < 2)) {
            // Obtener el atributo actual
            oAtributo = (clsElemento) vAtributos.elementAt(iIndice);
            // Comprobar si es el atributo seleccionado y no es llave
            if (sAtributo.equals(oAtributo.sNombre)) {
               if (oAtributo.nTipo != clsConstantes.nATRIBUTO_LLAVE) {
                  // Indicar que hay ms de un atributo llave para que no siga
                  // comprobando y elimine el atributo
                  nLlave = 2;
               } // if
               else {
                  // Indicar que el atributo es llave
                  bLlave = true;
                  // Incrementar el nmero de atributos llave
                  nLlave++;
               } // else
            } // if
            else {
               // Comprobar si es atributo llave
               if (oAtributo.nTipo == clsConstantes.nATRIBUTO_LLAVE) {
                  // Incrementar el nmero de atributos llave
                  nLlave++;
               } // if
            } // else
            // Incrementar el ndice
            iIndice++;
         } // while

         // Comprobar si es atributo llave
         if (bLlave) {
            // Comprobar que no sea el nico atributo llave
            if (nLlave > 1) {
               // Eliminar el atributo
               oGeneral.EliminarAtributo(lstAtributos,
                                         vAtributos,
                                         btnElimAtrib);
            } // if
            else {
               // Comprobar si la entidad es superclase
               if (!oGeneral.oModificar.sSubclase.equals("") ||
                   (oGeneral.oModificar.vEspecializaciones.size() > 0) ||
                   (oGeneral.oModificar.vUniones.size() > 0) ||
                   (oGeneral.oModificar.vIntersecciones.size() > 0)) {
                  oMensajes.MostrarMensaje(clsConstantes.
                                           sERROR_ELIMINAR_LLAVE_SUPERCLASE, this);
               } // if
               else {
                  // Eliminar el atributo
                  oGeneral.EliminarAtributo(lstAtributos,
                                            vAtributos,
                                            btnElimAtrib);
               } // else
            } // else
         } // if
         else {
            // Eliminar el atributo
            oGeneral.EliminarAtributo(lstAtributos,
                                      vAtributos,
                                      btnElimAtrib);
         } // else
      } // if
   } // Procedimiento: ComprobarEliminarAtributo

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Funcin: ComprobarEliminarAtributo                                      //
   // Parmetros: - pnTipo (int): Tipo de atributo                            //
   // Retorno: Indica si se trata de una subclase con atributo llave          //
   // Descripcin: Comprueba si se est intentando crear un atributo llave a  //
   //              una entidad que es subclase                                //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private boolean bSubclaseConLlave(int pnTipo){

      boolean bRetorno = false; // Indica si se trata de una subclase con llave
      clsElemento oElemento; // Objeto que contiene un elemento del esquema

      // Comprobar si el atributo es llave
      if (pnTipo == clsConstantes.nATRIBUTO_LLAVE){
         if (!oGeneral.oModificar.sSuperclase.equals("")){
            // Actualizar el valor de retorno
            bRetorno = true;
         } // if
         else if (oGeneral.oModificar.vEspecializaciones.size() > 0) {
            // Obtener la especializacin
            oElemento = oGeneral.elmElemento((String) oGeneral.oModificar.vEspecializaciones.elementAt(0));
            // Actualizar el valor de retorno
            bRetorno = !oElemento.sSuperclase.equals(oGeneral.oModificar.sNombre);
         } // else if
         else if (oGeneral.oModificar.vUniones.size() > 0) {
            // Obtener el tipo unin
            oElemento = oGeneral.elmElemento((String) oGeneral.oModificar.vUniones.elementAt(0));
            // Actualizar el valor de retorno
            bRetorno = oElemento.sSubclase.equals(oGeneral.oModificar.sNombre);
         } // else if
         else if (oGeneral.oModificar.vIntersecciones.size() > 0) {
            // Obtener el tipo interseccin
            oElemento = oGeneral.elmElemento((String) oGeneral.oModificar.vIntersecciones.elementAt(0));
            // Actualizar el valor de retorno
            bRetorno = oElemento.sSubclase.equals(oGeneral.oModificar.sNombre);
         } // else if
      } // if

      // Devolver el resultado de la comprobacin
      return bRetorno;
   } // Funcin: bSubclaseConLlave

} // Clase: clsEntidad
