package Derytas;

import java.util.Vector;
import java.awt.*;

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// Clase: clsAtributo                                                         //
// Mtodos: - public clsFichero: Constructor de la clase                      //
//          - public void Abrir: Procedimiento que recibe una cadena con el   //
//            contenido del fichero de un esquema y lo muestra en pantalla    //
//          - public String sGuardar: Funcin que devuelve una cadena con el  //
//            contenido de un esquema para ser guardada en fichero            //
//          - public String sObtenerFicheroEsquema: Devuelve la ruta completa //
//            del fichero                                                     //
//          - public String sObtenerFicheroScript: Devuelve la ruta completa  //
//            del fichero para el script del esquema                          //
//          - private int nLeerAtributos: Lee las propiedes de un atributo    //
//          - private int nLeerConfiguracion: Lee las propiedades de las      //
//            variables de configuracin                                      //
//          - private int nLeerElemento: Lee las propiedades de un elemento   //
//            del esquema                                                     //
//          - private int nLeerEntidad: Lee las propiedades de una entidad    //
//          - private int nLeerEspecializacion: Lee las propiedades de una    //
//            especializacin                                                 //
//          - private int nLeerRelacion: Lee las propiedades de una relacin  //
//          - private int nLeerTipoInterseccion: Lee las propiedades de un    //
//            tipo interseccin                                               //
//          - private int nLeerTipoUnion: Lee las propiedades de un tipo      //
//            unin                                                           //
//          - private String sGuardarAtributo: Construye una cadena para      //
//            guardar un atributo                                             //
//          - private String sGuardarConfiguracion: Construye una cadena para //
//            guardar el valor de las variables de configuracin              //
//          - private String sGuardarElemento: Construye una cadena para      //
//            guardar un elemento                                             //
//          - private String sGuardarEntidad: Construye una cadena para       //
//            guardar una entidad                                             //
//          - private String sGuardarEntRel: Construye una cadena para        //
//            guardar las propiedades de una entidad participante en una      //
//            relacin                                                        //
//          - private String sGuardarEspecializacion: Construye una cadena    //
//            guardar una especializacin                                     //
//          - private String sGuardarRelacion: Construye una cadena para      //
//            guardar una relacin                                            //
//          - private String sGuardarTipoInterseccion: Construye una cadena   //
//            guardar un tipo interseccin                                    //
//          - private String sGuardarTipoUnion: Construye una cadena para     //
//            guardar un tipo unin                                           //
// Descripcin: Clase para abrir y guardar esquemas en fichero                //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////
public class clsFichero {

   // Variables de clase
   private clsGeneral oGeneral;
   private clsConfig oConfig;
   private clsMensajes oMensajes;
   private String[] gsDatos; // Datos ledos del fichero
   private String gsPath; // Ruta de la aplicacin

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: clsFichero                                               //
   // Parmetros: - poGeneral (clsGeneral): Objeto de la clase clsGeneral     //
   //               para poder acceder a los mtodos de carcter general      //
   //             - poConfig (clsConfig): Objeto de la clase clsConfig para   //
   //               poder acceder a las variables de configuracin            //
   //             - pgsPath (String): Ruta de la aplicacin                   //
   // Descripcin: Constructor de la clase                                    //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   public clsFichero(clsGeneral poGeneral,
                     clsConfig poConfig,
                     String pgsPath) {

      // Inicializacin de variables

      // Asignar el objeto de la clase clsGeneral
      oGeneral = poGeneral;
      // Asignar el objeto de la clase clsConfig
      oConfig = poConfig;
      // Crear el objeto de la clase clsGeneral para mostrar mensajes
      oMensajes = new clsMensajes(oGeneral);
      // Asignar la ruta de la aplicacin
      gsPath = pgsPath;
   } // Procedimiento: clsFichero

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Funcin: sGuardar                                                       //
   // Parmetros: No                                                          //
   // Retorno: Construye una cadena con los elementos del esquema para ser    //
   //          guardados en fichero                                           //
   // Descripcin: Constructor de la clase                                    //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   public String sGuardar() {

      String sEsquema = ""; // Cadena con los elementos del esquema

      // Guardar opciones de configuracin
      sEsquema = sGuardarConfiguracion();

      // Guardar elementos del esquema
      for (int i = 0; i < oGeneral.vElementos.size(); i++) {
         sEsquema +=
            sGuardarElemento( (clsElemento) oGeneral.vElementos.elementAt(i));
      } // for

      // Devolver la cadena con los elementos del esquema
      return sEsquema;
   } // Funcin: sGuardar

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Funcin: sGuardarConfiguracion                                          //
   // Parmetros: No                                                          //
   // Retorno: Cadena con la configuracin del esquema                        //
   // Descripcin: Construye una cadena con la configuracin del esquema para //
   //              guardarla en fichero                                       //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private String sGuardarConfiguracion() {

      String sConfiguracion; // Cadena con la configuracin del esquema

      // Guardar el valor de la variable de configuracin bCerrar
      sConfiguracion = String.valueOf(oConfig.bCerrar);
      // Guardar el indicador de los atributos que se visualizan
      sConfiguracion += " " + String.valueOf(oConfig.nVerAtributos);
      // Guardar el indicador de la opcin para generar el cdigo de las especializaciones
      sConfiguracion += " " + String.valueOf(oConfig.nOpcionEsp);
      // Guardar si se generan o no comentarios para el script
      sConfiguracion += " " + String.valueOf(oConfig.bComentScript);
      // Guardar si se generan o no comentarios en el diccionario de la BBDD
      sConfiguracion += " " + String.valueOf(oConfig.bComentDic);
      // Devolver la cadena con la configuracion del esquema
      return sConfiguracion;
   } // Funcin: sGuardarConfiguracion

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Funcin: sGuardarElemento                                               //
   // Parmetros: - oElemento (clsElemento): Elemento a guardar               //
   // Retorno: Cadena con el elemento a guardar                               //
   // Descripcin: Construye una cadena con el elemento que se quiere guardar //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private String sGuardarElemento(clsElemento oElemento) {

      String sElemento = ""; // Cadena con el elemento a guardar
      int nTipo; // Tipo del elemento que se quiere guardar

      // Obtener el tipo del elemento que se quiere guardar
      nTipo = oElemento.nTipo;

      // Comprobar el tipo del elemento a guardar
      if ( (nTipo == clsConstantes.nENTIDAD) ||
          (nTipo == clsConstantes.nENTIDAD_DEBIL)) {
         // Obtener la cadena para guardar una entidad
         sElemento += sGuardarEntidad(oElemento);
      } // if
      else if ( (nTipo == clsConstantes.nRELACION) ||
               (nTipo == clsConstantes.nRELACION_IDENTIFICATIVA)) {
         // Obtener la cadena para guardar una relacin
         sElemento += sGuardarRelacion(oElemento);
      } // else if
      else if ( (nTipo == clsConstantes.nESPECIALIZACION_DISJUNTA_PARCIAL) ||
               (nTipo == clsConstantes.nESPECIALIZACION_DISJUNTA_TOTAL) ||
               (nTipo == clsConstantes.nESPECIALIZACION_SOLAPADA_PARCIAL) ||
               (nTipo == clsConstantes.nESPECIALIZACION_SOLAPADA_TOTAL)) {
         // Obtener la cadena para guardar una especializacin
         sElemento += sGuardarEspecializacion(oElemento);
      } // else if
      else if ( (nTipo == clsConstantes.nTIPO_UNION_PARCIAL) ||
               (nTipo == clsConstantes.nTIPO_UNION_TOTAL)) {
         // Obtener la cadena para guardar un tipo unin
         sElemento += sGuardarTipoUnion(oElemento);
      } // else if
      else if ( (nTipo == clsConstantes.nTIPO_INTERSECCION_PARCIAL) ||
               (nTipo == clsConstantes.nTIPO_INTERSECCION_TOTAL)) {
         // Obtener la cadena para guardar un tipo interseccin
         sElemento += sGuardarTipoInterseccion(oElemento);
      } // else if

      // Devolver la cadena con el elemento a guardar
      return sElemento;
   } // Funcin: sGuardarElemento

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Funcin: sGuardarEntidad                                                //
   // Parmetros: - oElemento (clsElemento): Entidad a guardar                //
   // Retorno: Cadena con la entidad a guardar                                //
   // Descripcin: Construye una cadena con la entidad que se quiere guardar  //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private String sGuardarEntidad(clsElemento oElemento) {

      String sEntidad = ""; // Cadena con la entidad a guardar
      int nAtributos; // Atributos de la entidad
      int nRelaciones; // Relaciones en las que participa la entidad
      int nEspecializaciones; // Especializaciones a las que pertenece la entidad
      int nTiposU; // Tipos unin a los que pertenece la entidad
      int nTiposI; // Tipos interseccin a los que pertenece la entidad
      String sComentario; // Contiene el comentario antes de guardarlo

      // Guardar el tipo de entidad
      sEntidad += " " + String.valueOf(oElemento.nTipo);
      // Guardar el nombre de la entidad
      sEntidad += " " + oElemento.sNombre;
      // Guardar la posicin X de la entidad
      sEntidad += " " + String.valueOf(oElemento.nX);
      // Guardar la posicin Y de la entidad
      sEntidad += " " + String.valueOf(oElemento.nY);

      // Obtener el nmero de atributos de la entidad
      nAtributos = oElemento.vAtributos.size();
      // Guardar el nmero de atributos de la entidad
      sEntidad += " " + String.valueOf(nAtributos);
      // Guardar los atributos de la entidad
      for (int i = 0; i < nAtributos; i++) {
         sEntidad +=
            sGuardarAtributo( (clsElemento) oElemento.vAtributos.elementAt(i));
      } // for

      // Obtener el nmero de relaciones en las que participa la entidad
      nRelaciones = oElemento.vRelaciones.size();
      // Guardar el nmero de relaciones en las que participa la entidad
      sEntidad += " " + String.valueOf(nRelaciones);
      // Guardar las relaciones en las que participa la entidad
      for (int i = 0; i < nRelaciones; i++) {
         sEntidad += " " + (String) oElemento.vRelaciones.elementAt(i);
      } // for

      // Obtener el nmero de especializaciones a las que pertenece la entidad
      nEspecializaciones = oElemento.vEspecializaciones.size();
      // Guardar el nmero de especializaciones a las que pertenece la entidad
      sEntidad += " " + String.valueOf(nEspecializaciones);
      // Guardar las especializaciones a las que pertenece la entidad
      for (int i = 0; i < nEspecializaciones; i++) {
         sEntidad += " " + (String) oElemento.vEspecializaciones.elementAt(i);
      } // for

      // Obtener el nmero de tipos unin a los que pertenece la entidad
      nTiposU = oElemento.vUniones.size();
      // Guardar el nmero de tipos unin a los que pertenece la entidad
      sEntidad += " " + String.valueOf(nTiposU);
      // Guardar los tipos unin a los que pertenece la entidad
      for (int i = 0; i < nTiposU; i++) {
         sEntidad += " " + (String) oElemento.vUniones.elementAt(i);
      } // for

      // Obtener el nmero de tipos interseccin a los que pertenece la entidad
      nTiposI = oElemento.vIntersecciones.size();
      // Guardar el nmero de tipos interseccin a los que pertenece la entidad
      sEntidad += " " + String.valueOf(nTiposI);
      // Guardar los tipos interseccin a los que pertenece la entidad
      for (int i = 0; i < nTiposI; i++) {
         sEntidad += " " + (String) oElemento.vIntersecciones.elementAt(i);
      } // for

      // Comprobar si es superclase o subclase de una entidad
      if (! (oElemento.sSubclase.equals(""))) {
         // Guardar que es superclase
         sEntidad += " " + String.valueOf(clsConstantes.nES_SUPERCLASE);
         // Guardar la subclase
         sEntidad += " " + oElemento.sSubclase;
      } // if
      else if (! (oElemento.sSuperclase.equals(""))) {
         // Guardar que es subclase
         sEntidad += " " + String.valueOf(clsConstantes.nES_SUBCLASE);
         // Guardar la superclase
         sEntidad += " " + oElemento.sSuperclase;
      } // else if
      else {
         // Guardar que no es superclase ni subclase
         sEntidad += " " + String.valueOf(clsConstantes.nNO_SUB_O_SUP);
      } // else

      // Guardar si los atributos se pintan a izquierda o derecha
      sEntidad += " " + String.valueOf(oElemento.bAtribIzq);

      // Obtener el comentario de la entidad
      sComentario = oElemento.sComentario;
      // Guardar si se ha indicado comentario para la entidad
      sEntidad += " " + String.valueOf(!sComentario.equals(""));
      // Comprobar si se ha indicado comentario para la entidad
      if (!sComentario.equals("")) {
         // Rellenar los espacios
         sComentario = sComentario.replaceAll(" ",
                                              clsConstantes.sRELLENO_ESPACIOS);
         // Sustituir los retornos de carro
         sComentario = sComentario.replace(clsConstantes.cRETORNO_CARRO, '@');
         // Guardar el comentario
         sEntidad += " " + sComentario;
      } // if

      // Devolver la cadena con la entidad a guardar
      return sEntidad;
   } // Funcin: sGuardarEntidad

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Funcin: sGuardarAtributo                                               //
   // Parmetros: - oElemento (clsElemento): Atributo a guardar               //
   // Retorno: Cadena con el atributo a guardar                               //
   // Descripcin: Construye una cadena con el atributo que se quiere guardar //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private String sGuardarAtributo(clsElemento oElemento) {

      String sAtributo = ""; // Cadena con el atributo a guardar
      int nAtributos; // Atributos del atributo
      boolean bNombreNotNull; // Indica si tiene nombre para NOT NULL
      boolean bNombreUnique; // Indica si tiene nombre para UNIQUE
      boolean bNombreCheck; // Indica si tiene nombre para CHECK
      boolean bComentario; // Indica si hay comentario
      String sComentario; // Contiene el comentario antes de guardarlo

      // Guardar el tipo de atributo
      sAtributo += " " + String.valueOf(oElemento.nTipo);
      // Guardar el nombre del atributo
      sAtributo += " " + oElemento.sNombre;
      // Guardar el dominio del atributo
      sAtributo += " " + oElemento.sDominio.replaceFirst(" ",
                                              clsConstantes.sRELLENO_ESPACIOS);

      // Guardar si se ha indicado restriccin Not Null
      sAtributo += " " + String.valueOf(oElemento.bNotNull);
      // Comprobar si se ha indicado restriccin Not Null
      if (oElemento.bNotNull) {
         // Obtener si la restriccin Not Null tiene nombre
         bNombreNotNull = (!oElemento.sNotNull.equals(""));
         // Guardar si la restriccin Not Null tiene nombre
         sAtributo += " " + String.valueOf(bNombreNotNull);
         // Comprobar si la restriccin Not Null tiene nombre
         if (bNombreNotNull) {
            // Guardar el nombre de la restriccin Not Null
            sAtributo += " " + oElemento.sNotNull;
         } // if
      } // if

      // Guardar si se ha indicado restriccin Unique
      sAtributo += " " + String.valueOf(oElemento.bUnique);
      // Comprobar si se ha indicado restriccin Unique
      if (oElemento.bUnique) {
         // Obtener si la restriccin Unique tiene nombre
         bNombreUnique = (!oElemento.sUnique.equals(""));
         // Guardar si la restriccin Unique tiene nombre
         sAtributo += " " + String.valueOf(bNombreUnique);
         // Comprobar si la restriccin Unique tiene nombre
         if (bNombreUnique) {
            // Guardar el nombre de la restriccin Unique
            sAtributo += " " + oElemento.sUnique;
         } // if
      } // if

      // Guardar si se ha indicado valor por defecto
      sAtributo += " " + String.valueOf(oElemento.bDefault);
      // Comprobar si se ha indicado valor por defecto
      if (oElemento.bDefault) {
         // Guardar el valor por defecto
         sAtributo += " " +
            oElemento.sDefault.replaceAll(" ", clsConstantes.sRELLENO_ESPACIOS);
      } // if

      // Guardar si se ha indicado restriccin Check
      sAtributo += " " + String.valueOf(oElemento.bCheck);
      // Comprobar si se ha indicado restriccin Check
      if (oElemento.bCheck) {
         // Obtener si la restriccin Check tiene nombre
         bNombreCheck = (!oElemento.sNombreCheck.equals(""));
         // Indicar si la restriccin Check tiene nombre
         sAtributo += " " + String.valueOf(bNombreCheck);
         // Comprobar si la restriccin Check tiene nombre
         if (bNombreCheck) {
            // Guardar el nombre de la restriccin Check
            sAtributo += " " + oElemento.sNombreCheck;
         } // if
         // Guardar el valor de la restriccin Check
         sAtributo += " " +
            oElemento.sCheck.replaceAll(" ", clsConstantes.sRELLENO_ESPACIOS);
      } // if

      // Guardar el elemento al que pertenece el atributo
      sAtributo += " " + oElemento.sElementoAtributo;

      // Obtener el nmero de atributos del atributo
      nAtributos = oElemento.vAtributos.size();
      // Guardar el nmero de atributos del atributo
      sAtributo += " " + String.valueOf(nAtributos);
      // Guardar los atributos del atributo
      for (int i = 0; i < oElemento.vAtributos.size(); i++) {
         sAtributo +=
            sGuardarAtributo( (clsElemento) oElemento.vAtributos.elementAt(i));
      } // for

      // Obtener el comentario
      sComentario = oElemento.sComentario;
      // Obtener si tiene comentario
      bComentario = (!sComentario.equals(""));
      // Guardar si tiene comentario
      sAtributo += " " + String.valueOf(bComentario);
      // Comprobar si tiene comentario
      if (bComentario) {
         // Rellenar los espacios en blanco
         sComentario = sComentario.replaceAll(" ",
                                              clsConstantes.sRELLENO_ESPACIOS);
         // Sustituir el retorno de carro
         sComentario = sComentario.replace(clsConstantes.cRETORNO_CARRO,
                                           '@');
         // Guardar el comentario
         sAtributo += " " + sComentario;
      } // if

      // Devolver la cadena con el atributo a guardar
      return sAtributo;
   } // Funcin: sGuardarAtributo

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Funcin: sGuardarRelacion                                               //
   // Parmetros: - oElemento (clsElemento): Relacin a guardar               //
   // Retorno: Cadena con la relacin a guardar                               //
   // Descripcin: Construye una cadena con la relacin que se quiere guardar //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private String sGuardarRelacion(clsElemento oElemento) {

      String sRelacion = ""; // Cadena con la relacin que se quiere guardar
      int nAtributos; // Atributos de la relacin
      int nEntidades; // Entidades de la relacin

      // Guardar el tipo de relacin
      sRelacion += " " + String.valueOf(oElemento.nTipo);
      // Guardar el nombre de la relacin
      sRelacion += " " + oElemento.sNombre;
      // Guardar la posicin X de la relacin
      sRelacion += " " + String.valueOf(oElemento.nX);
      // Guardar la posicin Y de la relacin
      sRelacion += " " + String.valueOf(oElemento.nY);

      // Obtener el nmero de atributos de la relacin
      nAtributos = oElemento.vAtributos.size();
      // Guardar el nmero de atributos de la relacin
      sRelacion += " " + String.valueOf(nAtributos);
      // Guardar los atributos de la relacin
      for (int i = 0; i < nAtributos; i++) {
         sRelacion +=
            sGuardarAtributo( (clsElemento) oElemento.vAtributos.elementAt(i));
      } // for

      // Obtener el nmero de entidades de la relacin
      nEntidades = oElemento.vEntRel.size();
      // Guardar el nmero de entidades de la relacin
      sRelacion += " " + String.valueOf(nEntidades);
      // Guardar las entidades de la relacin
      for (int i = 0; i < nEntidades; i++) {
         sRelacion += sGuardarEntRel( (clsEntRel) oElemento.vEntRel.elementAt(i));
      } // for

      // Devolver la cadena con la relacin a guardar
      return sRelacion;
   } // Funcin: sGuardarRelacion

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Funcin: sGuardarEntRel                                                 //
   // Parmetros: - oElemento (clsElemento): Entidad participante en una      //
   //               relacin a guardar                                        //
   // Retorno: Cadena con la entidad participante en una relacina guardar    //
   // Descripcin: Construye una cadena para la entidad participante en una   //
   //              relacin a guardar                                         //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private String sGuardarEntRel(clsEntRel oEntRel) {

      // Devolver la cadena para la entidad participante en la relacin que se
      // quiere guardar
      return " " + oEntRel.sNombre +
         " " + oEntRel.sMin +
         " " + oEntRel.sMax;
   } // Funcin: sGuardarEntRel

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Funcin: sGuardarEspecializacion                                        //
   // Parmetros: - oElemento (clsElemento): Especializacin a guardar        //
   // Retorno: Cadena con la especializacin a guardar                        //
   // Descripcin: Construye una cadena para la especializacin a guardar     //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private String sGuardarEspecializacion(clsElemento oElemento) {

      String sEspecializacion = ""; // Cadena con la especializacin a guardar
      int nSubclases; // Subclases de la especializacin

      // Guardar el tipo de especializacin
      sEspecializacion += " " + String.valueOf(oElemento.nTipo);
      // Guardar el nombre de la especializacin
      sEspecializacion += " " + oElemento.sNombre;
      // Guardar la posicin X de la especializacin
      sEspecializacion += " " + String.valueOf(oElemento.nX);
      // Guardar al posicin Y de la especializacin
      sEspecializacion += " " + String.valueOf(oElemento.nY);
      // Guardar la superclase de la especializacin
      sEspecializacion += " " + String.valueOf(oElemento.sSuperclase);

      // Obtener el nmero de subclases de la especializacin
      nSubclases = oElemento.vSubclases.size();
      // Guardar el nmero de subclases de la especializacin
      sEspecializacion += " " + String.valueOf(nSubclases);
      // Guardar las subclases de la especializacin
      for (int i = 0; i < nSubclases; i++) {
         sEspecializacion += " " +
            (String) oElemento.vSubclases.elementAt(i);
      } // for

      // Guardar el atributo de la especializacin
      if (oElemento.sAtributoEspecializacion.equals("")) {
         sEspecializacion += " " + clsConstantes.sNO_ATRIBUTO_ESPECIALIZACION;
      } // if
      else {
         sEspecializacion += " " + oElemento.sAtributoEspecializacion;
      } // else

      // Devolver la cadena con la especializacin a guardar
      return sEspecializacion;
   } // Funcin: sGuardarRelacion

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Funcin: sGuardarTipoUnion                                              //
   // Parmetros: - oElemento (clsElemento): Tipo unin a guardar             //
   // Retorno: Cadena con el tipo unin a guardar                             //
   // Descripcin: Construye una cadena para el tipo unin a guardar          //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private String sGuardarTipoUnion(clsElemento oElemento) {

      String sTipoUnion = ""; // Cadena para el tipo unin a guardar
      int nSuperclases; // Superclases del tipo unin

      // Guardar el tipo de tipo unin
      sTipoUnion += " " + String.valueOf(oElemento.nTipo);
      // Guardar el nombre del tipo unin
      sTipoUnion += " " + oElemento.sNombre;
      // Guardar la posicin X del tipo unin
      sTipoUnion += " " + String.valueOf(oElemento.nX);
      // Guardar la posicin Y del tipo unin
      sTipoUnion += " " + String.valueOf(oElemento.nY);
      // Guardar la subclase del tipo unin
      sTipoUnion += " " + String.valueOf(oElemento.sSubclase);

      // Obtener el nmero de superclases del tipo unin
      nSuperclases = oElemento.vSuperclases.size();
      // Guardar el nmero de superclases del tipo unin
      sTipoUnion += " " + String.valueOf(nSuperclases);
      // Guardar las subclases del tipo unin
      for (int i = 0; i < nSuperclases; i++) {
         sTipoUnion += " " +
            (String) oElemento.vSuperclases.elementAt(i);
      } // for

      // Guardar si las llaves de las superclases son iguales o no
      sTipoUnion += " " + String.valueOf(oElemento.bLlavesIguales);

      // Devolver la cadena con el tipo unin a guardar
      return sTipoUnion;
   } // Funcin: sGuardarTipoUnion

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Funcin: sGuardarTipoInterseccion                                       //
   // Parmetros: - oElemento (clsElemento): Tipo interseccin a guardar      //
   // Retorno: Cadena con el tipo interseccin a guardar                      //
   // Descripcin: Construye una cadena para el tipo interseccin a guardar   //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private String sGuardarTipoInterseccion(clsElemento oElemento) {

      String sInterseccion = ""; // Cadena para el tipo interseccin a guardar
      int nSuperclases; // Superclases del tipo interseccin

      // Guardar el tipo de tipo interseccin
      sInterseccion += " " + String.valueOf(oElemento.nTipo);
      // Guardar el nombre del tipo interseccin
      sInterseccion += " " + oElemento.sNombre;
      // Guaradar la posicin X del tipo interseccin
      sInterseccion += " " + String.valueOf(oElemento.nX);
      // Guardar la posicin Y del tipo interseccin
      sInterseccion += " " + String.valueOf(oElemento.nY);
      // Guardar la subclase del tipo interseccin
      sInterseccion += " " + String.valueOf(oElemento.sSubclase);

      // Obtener el nmero de superclases del tipo interseccin
      nSuperclases = oElemento.vSuperclases.size();
      // Guardar el nmero de superclases del tipo interseccin
      sInterseccion += " " + String.valueOf(nSuperclases);
      // Guardar las subclases del tipo interseccin
      for (int i = 0; i < nSuperclases; i++) {
         sInterseccion += " " +
            (String) oElemento.vSuperclases.elementAt(i);
      } // for

      // Devolver la cadena con el tipo interseccin a guardar
      return sInterseccion;
   } // Funcin: sGuardarTipoInterseccion

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: Abrir                                                    //
   // Parmetros: - psDatos (String): Cadena con el contenido del fichero que //
   //               guarda un esquema                                         //
   // Descripcin: Obtiene las propiedades de los elementos del esquema a     //
   //              partir de la cadena que recibe                             //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   public void Abrir(String psDatos) {

      int iIndice; // ndice para recorrer el array con el esquema

      try {
         // Inicializar el ndice para recorrer los datos
         iIndice = 0;
         // Obtener el array con el esquema
         gsDatos = psDatos.split(" ");
         // Leer los parmetros de configuracin
         iIndice = nLeerConfiguracion(iIndice);

         // Leer los elementos mientras haya alguno por leer
         while (iIndice < gsDatos.length) {
            iIndice = nLeerElemento(iIndice);
         } // while
      } // try

      catch (Throwable t) {
         // Mostrar el mensaje de error por no poder leer el fichero
         oMensajes.MostrarMensaje(clsConstantes.sERROR_FICHERO_INCORRECTO);
      } // catch
   } // Procedimiento: Abrir

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Funcin: nLeerConfiguracion                                             //
   // Parmetros: - piIndice (int): ndice del array de datos por el que se   //
   //               va leyendo                                                //
   // Retorno: ndice del array de datos del siguiente elemento               //
   // Descripcin: Obtiene los valores de las variables de configuracin del  //
   //              esquema                                                    //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private int nLeerConfiguracion(int piIndice) {

      String sVerAtributos; // Valor que indica los atributos que se visualizan
      String sOpcionEsp; // Valor que indica la opcin para generar las espcializaciones

      // Obtener el valor de la variable que indica si las ventanas se cierran
      // o permanecen abiertas tras crear un elemento
      oConfig.bCerrar = gsDatos[piIndice].equals("true");
      piIndice++;

      // Obtener el valor de la variable que indica los atributos visible
      sVerAtributos = gsDatos[piIndice];
      piIndice++;
      if (sVerAtributos.equals(String.valueOf(clsConstantes.nTODOS))) {
         oConfig.nVerAtributos = clsConstantes.nTODOS;
      } // if
      else if (sVerAtributos.equals(String.valueOf(clsConstantes.nINMEDIATOS))) {
         oConfig.nVerAtributos = clsConstantes.nINMEDIATOS;
      } // else if
      else if (sVerAtributos.equals(String.valueOf(clsConstantes.nNINGUNO))) {
         oConfig.nVerAtributos = clsConstantes.nNINGUNO;
      } // else if

      // Obtener el indicador de la opcin para generar el cdigo de las especializaciones
      sOpcionEsp = gsDatos[piIndice];
      piIndice++;
      if (sOpcionEsp.equals(String.valueOf(clsConstantes.nOPCION_A))) {
         oConfig.nOpcionEsp = clsConstantes.nOPCION_A;
      } // if
      else if (sOpcionEsp.equals(String.valueOf(clsConstantes.nOPCION_B))) {
         oConfig.nOpcionEsp = clsConstantes.nOPCION_B;
      } // else if
      else if (sOpcionEsp.equals(String.valueOf(clsConstantes.nOPCION_C))) {
         oConfig.nOpcionEsp = clsConstantes.nOPCION_C;
      } // else if
      else if (sOpcionEsp.equals(String.valueOf(clsConstantes.nOPCION_D))) {
         oConfig.nOpcionEsp = clsConstantes.nOPCION_D;
      } // else if

      // Obtener si se generan o no comentarios para el script
      oConfig.bComentScript = gsDatos[piIndice].equals("true");
      piIndice++;

      // Obtener si se generan o no comentarios en el diccionario de la BBDD
      oConfig.bComentDic = gsDatos[piIndice].equals("true");
      piIndice++;

      // Devolver el ndice del array de elementos
      return piIndice;
   } // Funcin: nLeerConfiguracion

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Funcin: nLeerElemento                                                  //
   // Parmetros: - piIndice (int): ndice del array de datos por el que se   //
   //               va leyendo                                                //
   // Retorno: ndice del array de datos del siguiente elemento               //
   // Descripcin: Obtiene las propiedades del elemento del esquema indicado  //
   //              por el ndice que se recibe como parmetro                 //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private int nLeerElemento(int piIndice) {

      int nTipo; // Tipo de elemento

      // Recuperar el tipo de elemento
      nTipo = Integer.parseInt(gsDatos[piIndice]);

      // Leer el elemento segn el tipo del que se trate
      if ( (nTipo == clsConstantes.nENTIDAD) ||
          (nTipo == clsConstantes.nENTIDAD_DEBIL)) {
         // Leer una entidad
         piIndice = nLeerEntidad(piIndice);
      } // if
      else if ( (nTipo == clsConstantes.nRELACION) ||
               (nTipo == clsConstantes.nRELACION_IDENTIFICATIVA)) {
         // Leer una relacin
         piIndice = nLeerRelacion(piIndice);
      } // else if
      else if ( (nTipo == clsConstantes.nESPECIALIZACION_DISJUNTA_PARCIAL) ||
               (nTipo == clsConstantes.nESPECIALIZACION_DISJUNTA_TOTAL) ||
               (nTipo == clsConstantes.nESPECIALIZACION_SOLAPADA_PARCIAL) ||
               (nTipo == clsConstantes.nESPECIALIZACION_SOLAPADA_TOTAL)) {
         // Leer una especializacin
         piIndice = nLeerEspecializacion(piIndice);
      } // else if
      else if ( (nTipo == clsConstantes.nTIPO_UNION_PARCIAL) ||
               (nTipo == clsConstantes.nTIPO_UNION_TOTAL)) {
         // Leer un tipo unin
         piIndice = nLeerTipoUnion(piIndice);
      } // else if
      else if ( (nTipo == clsConstantes.nTIPO_INTERSECCION_PARCIAL) ||
               (nTipo == clsConstantes.nTIPO_INTERSECCION_TOTAL)) {
         // Leer un tipo interseccin
         piIndice = nLeerTipoInterseccion(piIndice);
      } // else if

      // Devolver el ndice del siguiente elemento
      return piIndice;
   } // Funcin: nLeerElemento

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Funcin: nLeerEntidad                                                   //
   // Parmetros: - piIndice (int): ndice del array de datos por el que se   //
   //               va leyendo                                                //
   // Retorno: ndice del array de datos del siguiente elemento               //
   // Descripcin: Obtiene las propiedades de la entidad del esquema indicada //
   //              por el ndice que se recibe como parmetro                 //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private int nLeerEntidad(int piIndice) {

      int nAtributos; // Atributos de la entidad
      int nRelaciones; // Relaciones en las que participa la entidad
      int nEspecializaciones; // Especializaciones a las que pertenece
      int nTiposU; // Tipos unin a los que pertenece
      int nTiposI; // Tipos interseccin a los que pertenece
      int nSubSup; // Indicador de subclase, superclase o ninguno
      boolean bComentario; // Indicador de existencia de comentario
      clsElemento oEntidad; // Objeto para la entidad

      // Crear el objeto para la entidad
      oEntidad = new clsElemento(oGeneral, oConfig.nVerAtributos);
      // Iniciar el vector de atributos de la entidad
      oEntidad.vAtributos = new Vector();
      // Iniciar el vector de relaciones de la entidad
      oEntidad.vRelaciones = new Vector();
      // Iniciar el vector de especializaciones de la entidad
      oEntidad.vEspecializaciones = new Vector();
      // Iniciar el vector de tpos unin de la entidad
      oEntidad.vUniones = new Vector();
      // Iniciar el vector de tipos interseccin de la entidad
      oEntidad.vIntersecciones = new Vector();

      // Iniciar la subclase de la entidad
      oEntidad.sSubclase = "";
      // Inicia la superclase de la entidad
      oEntidad.sSuperclase = "";
      // Iniciar la llave de la entidad
      oEntidad.sLlave = "";
      // Iniciar la llave externa de la entidd
      oEntidad.sLlaveExterna = "";
      // Iniciar el comentario de la entidad
      oEntidad.sComentario = "";

      // Recuperar el tipo de entidad y aumentar el ndice
      oEntidad.nTipo = Integer.parseInt(gsDatos[piIndice]);
      piIndice++;
      // Recuperar el nombre de la entidad y aumentar el ndice
      oEntidad.sNombre = gsDatos[piIndice];
      piIndice++;
      // Recuperar la posicin X de la entidad y aumentar el ndice
      oEntidad.nX = Integer.parseInt(gsDatos[piIndice]);
      piIndice++;
      // Recuperar la posicin Y de la entidad y aumentar el ndice
      oEntidad.nY = Integer.parseInt(gsDatos[piIndice]);
      piIndice++;

      // Obtener el nmero de atributos y aumentar el ndice
      nAtributos = Integer.parseInt(gsDatos[piIndice]);
      piIndice++;
      // Crear los atributos, si los tiene
      for (int i = 0; i < nAtributos; i++) {
         piIndice = nLeerAtributos(oEntidad.vAtributos, piIndice);
      } // for

      // Obtener el nmero de relaciones en las que participa la entidad y
      // aumentar el ndice
      nRelaciones = Integer.parseInt(gsDatos[piIndice]);
      piIndice++;
      // Guardar el nombre de las relaciones
      for (int i = 0; i < nRelaciones; i++) {
         oEntidad.vRelaciones.addElement(gsDatos[piIndice]);
         piIndice++;
      } // for

      // Obtener el nmero de especializaciones a las que pertenece la entidad
      // y aumentar el ndice
      nEspecializaciones = Integer.parseInt(gsDatos[piIndice]);
      piIndice++;
      // Guardar el nombre de las especializaciones
      for (int i = 0; i < nEspecializaciones; i++) {
         oEntidad.vEspecializaciones.addElement(gsDatos[piIndice]);
         piIndice++;
      } // for

      // Obtener el nmero de tipos unin a los que pertenece la entidad y
      // aumentar el ndice
      nTiposU = Integer.parseInt(gsDatos[piIndice]);
      piIndice++;
      // Guardar el nombre de los tipos unin
      for (int i = 0; i < nTiposU; i++) {
         oEntidad.vUniones.addElement(gsDatos[piIndice]);
         piIndice++;
      } // for

      // Obtener el nmero de tipos interseccin a los que pertenece la entidad
      // y aumentar el ndice
      nTiposI = Integer.parseInt(gsDatos[piIndice]);
      piIndice++;
      // Guardar el nombre de los tipos interseccin
      for (int i = 0; i < nTiposI; i++) {
         oEntidad.vIntersecciones.addElement(gsDatos[piIndice]);
         piIndice++;
      } // for

      // Obtener si es superclase, subclase o ninguno de los dos y aumentar el
      // ndice
      nSubSup = Integer.parseInt(gsDatos[piIndice]);
      piIndice++;
      // Comprobar si es subclase o superclase
      if (nSubSup == clsConstantes.nES_SUBCLASE) {
         // Guardar el nombre de la superclase y aumentar el ndice
         oEntidad.sSuperclase = gsDatos[piIndice];
         piIndice++;
      } // if
      else if (nSubSup == clsConstantes.nES_SUPERCLASE) {
         // Guardar el nombre de la subclase y aumentar el ndice
         oEntidad.sSubclase = gsDatos[piIndice];
         piIndice++;
      } // else if
      // Leer si los atributos se pintan a la izquierda o a la derecha
      oEntidad.bAtribIzq = gsDatos[piIndice].equals("true");
      piIndice++;

      // Obtener si existe comentario
      bComentario = gsDatos[piIndice].equals("true");
      piIndice++;
      // Comprobar si existe comentario
      if (bComentario) {
         // Leer el comentario
         oEntidad.sComentario = gsDatos[piIndice];
         // Reemplaar la constante de espacios por espacios
         oEntidad.sComentario = oEntidad.sComentario.replaceAll(
            clsConstantes.sRELLENO_ESPACIOS, " ");
         // Reemplazar '@' por retorno de carro
         oEntidad.sComentario = oEntidad.sComentario.replace('@',
            clsConstantes.cRETORNO_CARRO);
         // Aumentar el ndice
         piIndice++;
      } // if

      // Asignar el ancho de la entidad
      oEntidad.nAncho = clsConstantes.nANCHO_ENTIDAD;
      // Asignar el alto de la entidad
      oEntidad.nAlto = clsConstantes.nALTO_ENTIDAD;
      // Identificar la llave de la entidad
      oEntidad.sLlave = oGeneral.sIdentificarLlave(oEntidad.vAtributos);

      // Aadir el objeto para la entidad al vector de elementos del esquema
      oGeneral.vElementos.addElement(oEntidad);

      // Devolver el ndice del siguiente elemento a leer
      return piIndice;
   } // Funcin: nLeerEntidad

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Funcin: nLeerAtributos                                                 //
   // Parmetros: - piIndice (int): ndice del array de datos por el que se   //
   //               va leyendo                                                //
   // Retorno: ndice del array de datos del siguiente elemento               //
   // Descripcin: Obtiene las propiedades del atributo del esquema indicado  //
   //              por el ndice que se recibe como parmetro                 //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private int nLeerAtributos(Vector pvAtributos, int piIndice) {

      int nAtributos; // Atributos del atributo
      boolean bNombreNotNull; // Indica si la restriccin Not Null tiene nombre
      boolean bNombreCheck; // Indica si la restriccin Check tiene nombre
      boolean bNombreUnique; // Indica si la restriccin Unique tiene nombre
      boolean bComentario; // Indica si tiene comentario
      clsElemento oAtributo; // Objeto para el atributo

      // Crear el objeto para elatributo
      oAtributo = new clsElemento(oGeneral, oConfig.nVerAtributos);
      // Iniciar el valor de las variables del atributo
      oAtributo.sDefault = "";
      oAtributo.sNotNull = "";
      oAtributo.sCheck = "";
      oAtributo.sNombreCheck = "";
      oAtributo.sUnique = "";
      oAtributo.sComentario = "";

      // Recuperar el tipo de atributo y aumentar el ndice
      oAtributo.nTipo = Integer.parseInt(gsDatos[piIndice]);
      piIndice++;
      // Recuperar el nombre del atributo y aumentar el ndice
      oAtributo.sNombre = gsDatos[piIndice];
      piIndice++;
      // Recuperar el dominio del atributo y aumentar el ndice
      oAtributo.sDominio = gsDatos[piIndice];
      oAtributo.sDominio = oAtributo.sDominio.replaceFirst(clsConstantes.sRELLENO_ESPACIOS, " ");
      piIndice++;

      // Recuperar la restriccin Not Null y aumentar el ndice
      oAtributo.bNotNull = gsDatos[piIndice].equals("true");
      piIndice++;
      // Comprobar si tiene restriccin Not Null
      if (oAtributo.bNotNull) {
         // Recuperar si la restriccin Not Null tiene nombre y aumentar el ndice
         bNombreNotNull = gsDatos[piIndice].equals("true");
         piIndice++;
         // Comprobar si la restriccin Not Null tiene nombre
         if (bNombreNotNull) {
            // Recuperar el nombre de la restriccin Not Null y aumentar el ndice
            oAtributo.sNotNull = gsDatos[piIndice];
            piIndice++;
         } // if
      } // if

      // Recuperar la restriccin Unique y aumentar el ndice
      oAtributo.bUnique = gsDatos[piIndice].equals("true");
      piIndice++;
      // Comprobar si tiene restriccin Unique
      if (oAtributo.bUnique) {
         // Recuperar si la restriccin Unique tiene nombre y aumentar el ndice
         bNombreUnique = gsDatos[piIndice].equals("true");
         piIndice++;
         // Comprobar si la restriccin Unique tiene nombre
         if (bNombreUnique) {
            // Recuperar el nombre de la restriccin Unique y aumentar el ndice
            oAtributo.sUnique = gsDatos[piIndice];
            piIndice++;
         } // if
      } // if

      // Recuperar Default y aumentar el indice
      oAtributo.bDefault = gsDatos[piIndice].equals("true");
      piIndice++;
      // Comprobar si se ha indicado Default
      if (oAtributo.bDefault) {
         // Recuperar el valor Default y aumentar el ndice
         oAtributo.sDefault = gsDatos[piIndice].replaceAll(clsConstantes.
            sRELLENO_ESPACIOS, " ");
         piIndice++;
      } // if

      // Recuperar la restriccin Check y aumentar el ndice
      oAtributo.bCheck = gsDatos[piIndice].equals("true");
      piIndice++;
      // Comprobar si tiene restriccin Check
      if (oAtributo.bCheck) {
         // Recuperar si la restriccin Check tiene nombre y aumentar el ndice
         bNombreCheck = gsDatos[piIndice].equals("true");
         piIndice++;
         // Comprobar si la restriccin Check tiene nombre
         if (bNombreCheck) {
            // Recuperar el nombre de la restriccin Check
            oAtributo.sNombreCheck = gsDatos[piIndice];
            piIndice++;
         } // if
         // Recuperar el valor de la restriccin Check y aumentar el ndice
         oAtributo.sCheck = gsDatos[piIndice].replaceAll(clsConstantes.
            sRELLENO_ESPACIOS, " ");
         piIndice++;
      } // if

      // Recuperar el elemento al que pertenece el atributo y aumentar el ndice
      oAtributo.sElementoAtributo = gsDatos[piIndice];
      piIndice++;

      // Obtener el nmero de atributos y aumentar el ndice
      nAtributos = Integer.parseInt(gsDatos[piIndice]);
      piIndice++;
      // Iniciar el vector de atributos del atributo
      oAtributo.vAtributos = new Vector();
      // Crear los atributos, si los tiene
      for (int i = 0; i < nAtributos; i++) {
         piIndice = nLeerAtributos(oAtributo.vAtributos, piIndice);
      } // for
      // Comprobar si el atributo es llave
      if ( (oAtributo.nTipo == clsConstantes.nATRIBUTO_LLAVE) ||
          (oAtributo.nTipo == clsConstantes.nATRIBUTO_LLAVE_PARCIAL)) {
         // Marcar como superllave los componentes simples del atributo
         oGeneral.AsignarSuperllave(oAtributo.vAtributos);
      } // if

      // Asignar el ancho del atributo
      oAtributo.nAncho = clsConstantes.nANCHO_ATRIBUTO;
      // Asignar el alto del atributo
      oAtributo.nAlto = clsConstantes.nALTO_ATRIBUTO;
      // Aadir el atributo al vector de atributos del elemento al que pertenece
      pvAtributos.addElement(oAtributo);

      // Recuperar si tiene comentario
      bComentario = gsDatos[piIndice].equals("true");
      piIndice++;
      // Comprobar si tiene comentario
      if (bComentario) {
         // Leer el comentario
         oAtributo.sComentario = gsDatos[piIndice];
         // Reemplazar la constante para espacios por espacios
         oAtributo.sComentario = oAtributo.sComentario.replaceAll(
            clsConstantes.sRELLENO_ESPACIOS, " ");
         // Reemplazar '@' por retorno de carro
         oAtributo.sComentario = oAtributo.sComentario.replace('@',
            clsConstantes.cRETORNO_CARRO);
         // Aumentar el ndice
         piIndice++;
      } // if

      // Devolver el ndice del siguiente elemento a leer
      return piIndice;
   } // Funcin: nLeerAtributos

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Funcin: nLeerRelacion                                                  //
   // Parmetros: - piIndice (int): ndice del array de datos por el que se   //
   //               va leyendo                                                //
   // Retorno: ndice del array de datos del siguiente elemento               //
   // Descripcin: Obtiene las propiedades de la relacin del esquema         //
   //              indicada por el ndice que se recibe como parmetro        //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private int nLeerRelacion(int piIndice) {

      int nAtributos; // Atributos de la relacin
      int nEntRels; // Entidades participantes en la relacin
      String sNombreEntRel; // Nombre de entidad participante
      String sMin; // Cardinalidad mnima de entidad participante
      String sMax; // Cardinalidad maxima de entidad participante
      clsElemento oRelacion; // Objeto para la relacin

      // Crear el objeto para la relacin
      oRelacion = new clsElemento(oGeneral, oConfig.nVerAtributos);
      // Iniciar el vector de atributos de la relacin
      oRelacion.vAtributos = new Vector();
      // Iniciar el vector de entidades pertenecientes a la relacin
      oRelacion.vEntRel = new Vector();

      // Recuperar el tipo de relacin y aumentar el ndice
      oRelacion.nTipo = Integer.parseInt(gsDatos[piIndice]);
      piIndice++;
      // Recuperar el nombre de la relacin y aumentar el ndice
      oRelacion.sNombre = gsDatos[piIndice];
      piIndice++;
      // Recuperar la posicin X de la relacin y aumentar el ndice
      oRelacion.nX = Integer.parseInt(gsDatos[piIndice]);
      piIndice++;
      // Recuperar la posicin Y de la relacin y aumentar el ndice
      oRelacion.nY = Integer.parseInt(gsDatos[piIndice]);
      piIndice++;

      // Obtener el nmero de atributos y aumentar el ndice
      nAtributos = Integer.parseInt(gsDatos[piIndice]);
      piIndice++;
      // Crear los atributos, si los tiene
      for (int i = 0; i < nAtributos; i++) {
         piIndice = nLeerAtributos(oRelacion.vAtributos, piIndice);
      }

      // Obtener el nmero de entidades participantes en la relacin y
      // aumentar el ndice
      nEntRels = Integer.parseInt(gsDatos[piIndice]);
      piIndice++;
      // Guardar la entidades participantes
      for (int i = 0; i < nEntRels; i++) {
         // Recuperar el nombre de la entidad participante y aumentar el ndice
         sNombreEntRel = gsDatos[piIndice];
         piIndice++;
         // Recuperar la cardinalidad mnima y aumentar el ndice y aumentar el ndice
         sMin = gsDatos[piIndice];
         piIndice++;
         // Recuperar la cardinalidad mxima
         sMax = gsDatos[piIndice];
         piIndice++;

         // Aadir la entidad al vector de entidades participantes
         oRelacion.vEntRel.addElement(new clsEntRel(sNombreEntRel, sMin, sMax));
      } // for

      // Asignar el ancho de la relacin
      oRelacion.nAncho = clsConstantes.nANCHO_RELACION;
      // Asignar el alto de la relacin
      oRelacion.nAlto = clsConstantes.nALTO_RELACION;

      // Aadir el objeto para la relacin al vector de elementos del esquema
      oGeneral.vElementos.addElement(oRelacion);

      // Devolver el ndice del siguiente elemento a leer
      return piIndice;
   } // Funcin: nLeerRelacion

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Funcin: nLeerEspecializacion                                           //
   // Parmetros: - piIndice (int): ndice del array de datos por el que se   //
   //               va leyendo                                                //
   // Retorno: ndice del array de datos del siguiente elemento               //
   // Descripcin: Obtiene las propiedades de la especializacin del esquema  //
   //              indicada por el ndice que se recibe como parmetro        //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private int nLeerEspecializacion(int piIndice) {

      int nSubclases; // Subclases de la especializacin
      clsElemento elmEspecializacion; // Objeto para la especializacin

      // Crear el objeto para la especializacin
      elmEspecializacion = new clsElemento(oGeneral, oConfig.nVerAtributos);
      // Iniciar el vector de subclases de la especializacin
      elmEspecializacion.vSubclases = new Vector();

      // Recuperar el tipo de especializacin y aumentar el ndice
      elmEspecializacion.nTipo = Integer.parseInt(gsDatos[piIndice]);
      piIndice++;
      // Recuperar el nombre de la especializacin y aumentar el ndice
      elmEspecializacion.sNombre = gsDatos[piIndice];
      piIndice++;
      // Recuperar la posicin X de la especializacin y aumentar el ndice
      elmEspecializacion.nX = Integer.parseInt(gsDatos[piIndice]);
      piIndice++;
      // Recuperar la posicin Y de la especializacin y aumentar el ndice
      elmEspecializacion.nY = Integer.parseInt(gsDatos[piIndice]);
      piIndice++;

      // Obtener la superclase de la especializacin  y aumentar el ndice
      elmEspecializacion.sSuperclase = gsDatos[piIndice];
      piIndice++;

      // Obtener el nmero de subclases de la especializacin y aumentar el ndice
      nSubclases = Integer.parseInt(gsDatos[piIndice]);
      piIndice++;
      // Guardar las subclases de la especializacin y aumentar el ndice
      for (int i = 0; i < nSubclases; i++) {
         elmEspecializacion.vSubclases.addElement(gsDatos[piIndice]);
         piIndice++;
      } // for

      // Recuperar el Atributo de la especializacin
      if (gsDatos[piIndice].equals(clsConstantes.sNO_ATRIBUTO_ESPECIALIZACION)) {
         elmEspecializacion.sAtributoEspecializacion = "";
      } // if
      else {
         elmEspecializacion.sAtributoEspecializacion = gsDatos[piIndice];
      } // else
      // Aumentar el ndice
      piIndice++;

      // Asignar el ancho de la especializacin
      elmEspecializacion.nAncho = clsConstantes.nANCHO_ESPGEN_UNCAT;
      // Asignar el alto de la especializacin
      elmEspecializacion.nAlto = clsConstantes.nALTO_ESPGEN_UNCAT;

      // Aadir el objeto para la especializacin al vector de elementos del esquema
      oGeneral.vElementos.addElement(elmEspecializacion);

      // Devolver el ndice del siguiente elemento a leer
      return piIndice;
   } // Funcin: nLeerEspecializacion

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Funcin: nLeerTipoUnion                                                 //
   // Parmetros: - piIndice (int): ndice del array de datos por el que se   //
   //               va leyendo                                                //
   // Retorno: ndice del array de datos del siguiente elemento               //
   // Descripcin: Obtiene las propiedades del tipo unin del esquema         //
   //              indicado por el ndice que se recibe como parmetro        //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private int nLeerTipoUnion(int piIndice) {

      int nSuperclases; // Superclases del tipo unin
      clsElemento oTipoUnion; // Objeto para el tipo unin

      // Crear el objeto para el tipo unin
      oTipoUnion = new clsElemento(oGeneral, oConfig.nVerAtributos);
      // Iniciar el vector de superclases del tipo unin
      oTipoUnion.vSuperclases = new Vector();

      // Recuperar el tipo de tipo unin y aumentar el ndice
      oTipoUnion.nTipo = Integer.parseInt(gsDatos[piIndice]);
      piIndice++;
      // Recuperar el nombre del tipo unin y aumentar el ndice
      oTipoUnion.sNombre = gsDatos[piIndice];
      piIndice++;
      // Recuperar la posicin X del tipo unin y aumentar el ndice
      oTipoUnion.nX = Integer.parseInt(gsDatos[piIndice]);
      piIndice++;
      // Recuperar la posicin Y de la Especializacin
      oTipoUnion.nY = Integer.parseInt(gsDatos[piIndice]);
      piIndice++;

      // Obtener la subclase del tipo unin y aumentar el ndice
      oTipoUnion.sSubclase = gsDatos[piIndice];
      piIndice++;

      // Obtener el nmero de superclases del tipo unin y aumentar el ndice
      nSuperclases = Integer.parseInt(gsDatos[piIndice]);
      piIndice++;
      // Guardar las superclases del tipo unin
      for (int i = 0; i < nSuperclases; i++) {
         oTipoUnion.vSuperclases.addElement(gsDatos[piIndice]);
         piIndice++;
      } // for

      // Recuperar si las llaves de las superclases son iguales o no y aumentar
      // el ndice
      oTipoUnion.bLlavesIguales = gsDatos[piIndice].equals("true");
      piIndice++;

      // Asignar el ancho del tipo unin
      oTipoUnion.nAncho = clsConstantes.nANCHO_ESPGEN_UNCAT;
      // Asignar el alto del tipo unin
      oTipoUnion.nAlto = clsConstantes.nALTO_ESPGEN_UNCAT;

      // Aadir el objeto para el tipo unin al vector de elementos del esquema
      oGeneral.vElementos.addElement(oTipoUnion);

      // Devolver el ndice del siguiente elemento a leer
      return piIndice;
   } // Funcin: nLeerTipoUnion

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Funcin: nLeerTipoInterseccion                                          //
   // Parmetros: - piIndice (int): ndice del array de datos por el que se   //
   //               va leyendo                                                //
   // Retorno: ndice del array de datos del siguiente elemento               //
   // Descripcin: Obtiene las propiedades del tipo interseccin del esquema  //
   //              indicado por el ndice que se recibe como parmetro        //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private int nLeerTipoInterseccion(int piIndice) {

      int nSuperclases; // Superclases del tipo interseccin
      clsElemento oInterseccion; // Objeto para el tipo interseccin

      // Crear el objeto para el tipo interseccin
      oInterseccion = new clsElemento(oGeneral, oConfig.nVerAtributos);
      // Iniciar el vector de Superclases de la Unin
      oInterseccion.vSuperclases = new Vector();

      // Recuperar el tipo de tipo interseccin y aumentar el ndice
      oInterseccion.nTipo = Integer.parseInt(gsDatos[piIndice]);
      piIndice++;
      // Recuperar el nombre del tipo interseccin y aumentar el ndice
      oInterseccion.sNombre = gsDatos[piIndice];
      piIndice++;
      // Recuperar la posicin X del tipo interseccin y aumentar el ndice
      oInterseccion.nX = Integer.parseInt(gsDatos[piIndice]);
      piIndice++;
      // Recuperar la posicin Y del tipo interseccin y aumentar el ndice
      oInterseccion.nY = Integer.parseInt(gsDatos[piIndice]);
      piIndice++;

      // Obtener la subclase del tipo interseccin y aumentar el ndice
      oInterseccion.sSubclase = gsDatos[piIndice];
      piIndice++;

      // Obtener el nmero de superclase del tipo interseccin y aumentar el ndice
      nSuperclases = Integer.parseInt(gsDatos[piIndice]);
      piIndice++;
      // Guardar las superclases del tipo interseccin
      for (int i = 0; i < nSuperclases; i++) {
         oInterseccion.vSuperclases.addElement(gsDatos[piIndice]);
         piIndice++;
      } // for

      // Asignar el ancho del tipo interseccin
      oInterseccion.nAncho = clsConstantes.nANCHO_ESPGEN_UNCAT;
      //Asignar el alto del tipo interseccin
      oInterseccion.nAlto = clsConstantes.nALTO_ESPGEN_UNCAT;

      // Aadir el objeto para el tipo unin al vector de elementos del esquema
      oGeneral.vElementos.addElement(oInterseccion);

      // Devolver el ndice del siguiente elemento a leer
      return piIndice;
   } // Funcin: nLeerTipoInterseccion

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Funcin: sObtenerFicheroEsquema                                         //
   // Parmetros: - pnOpcion (int): Opcin de apertura del fichero del        //
   //               esquema                                                   //
   // Retorno: ndice del array de datos del siguiente elemento               //
   // Descripcin: Devuelve la ruta completa del fichero del esquema          //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   public String sObtenerFicheroEsquema(int pnOpcion) {

      FileDialog fdDialog; // Dilogo para indicar el fichero
      String sRuta; // Ruta del fichero
      String sFichero; // Nombre del fichero
      String sTitulo; // Ttulo del dilogo
      int nModo; // Modo del dilogo

      // Comprobar el modo del dilogo
      if (pnOpcion == 0) {
         sTitulo = "Abrir Esquema";
         nModo = FileDialog.LOAD;
      } // if
      else {
         sTitulo = "Guardar Esquema";
         nModo = FileDialog.SAVE;
      } // else

      // Crear el dilogo
      fdDialog = new FileDialog(new Frame(), sTitulo, nModo);
      // Establecer la ruta
      fdDialog.setDirectory(gsPath);
      // Establecer la extensin del fichero
      fdDialog.setFile("*.eer");
      // Mostrar el dilogo
      fdDialog.show();

      // Obtener la ruta
      if (fdDialog.getFile() == null) {
         sRuta = "";
      } // if
      else {
         sFichero = fdDialog.getFile().trim();
         if (!sFichero.endsWith(".eer")) {
            sFichero += ".eer";
         } // if
         sRuta = fdDialog.getDirectory() + sFichero;
      } // else

      // Devolver la ruta
      return sRuta;
   } // Funcin: sObtenerFicheroEsquema

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Funcin: sObtenerFicheroScript                                          //
   // Parmetros: No                                                          //
   // Retorno: ndice del array de datos del siguiente elemento               //
   // Descripcin: Devuelve la ruta completa del fichero para el script del   //
   //              esquema                                                    //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   public String sObtenerFicheroScript() {

      FileDialog fdDialog; // Dilogo para indicar el fichero
      String sRuta; // Ruta del fichero
      String sFichero; // Nombre del fichero

      // Crear el dilogo
      fdDialog = new FileDialog(new Frame(), "Guardar Script", FileDialog.SAVE);
      // Establecer la ruta
      fdDialog.setDirectory(gsPath);
      // Establecer la extensin del fichero
      fdDialog.setFile("*.sql");
      // Mostrar el dilogo
      fdDialog.show();

      // Obtener la ruta
      if (fdDialog.getFile() == null) {
         sRuta = "";
      } // if
      else {
         sFichero = fdDialog.getFile().trim();
         if (!sFichero.endsWith(".sql")) {
            sFichero += ".sql";
         } // if
         sRuta = fdDialog.getDirectory() + sFichero;
      } // else

      // Devolver la ruta
      return sRuta;
   } // Funcin: sObtenerFicheroScript

} // Clase: clsFichero