package Derytas;

import java.util.*;
import java.awt.*;

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// Clase: clsGraficos                                                         //
// Mtodos: - public clsEntidad: Constructor de la clase                      //
//          - public void TrazarLineaDoble: Traza una lnea doble entre dos   //
//            elementos                                                       //
//          - public void TrazarLinea: Traza una lnea entre dos elementos    //
//          - private void PintarInclusion: Pinta el smbolo de inclusin     //
//          - private double dAngulo: Devuelve el ngulo en grados que forma  //
//            la horizontal con la recta formada por los puntos recibidos     //
//          - public void PintarEntidad: Pinta una entidad                    //
//          - public void PintarEntidadDebil: Pinta una entidad dbil         //
//          - public int PintarAtributo: Pintar un atributo y devolver el     //
//            nmero de atributos dibujados                                   //
//          - public void PintarRelacion: Pinta una relacin                  //
//          - public void PintarRelacionIdentificativa: Pinta una relacin    //
//            identificativa                                                  //
// Descripcin: Clase para crear y modificar entidades                        //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////
public class clsGraficos {

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: clsGraficos                                              //
   // Parmetros: No                                                          //
   // Descripcin: Constructor de la clase                                    //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   public clsGraficos() {
      // No hace nada
   } // Procedimiento: clsGraficos

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: TrazarLineaDoble                                         //
   // Parmetros: - g (Graphics): Objeto grfico                              //
   //             - pnAnchoLinea (int): Ancho de la lnea                     //
   //             - pnXOrigen (int): Posicin horizontal del origen           //
   //             - pnYOrigen (int): Posicin vertical del origen             //
   //             - pnAnchoOrigen (int): Ancho del origen                     //
   //             - pnAltoOrigen (int): Alto del origen                       //
   //             - pnXDestino (int): Posicin horizontal del destino         //
   //             - pnYDestino (int): Posicin vertical del destino           //
   //             - pnAnchoDestino (int): Ancho del destino                   //
   //             - pnAltoDestino (int): Alto del destino                     //
   //             - pbInclusion (boolean): Indica si se dibuja el smbolo de  //
   //               inclusin                                                 //
   //             - public void PintarEspecializacion: Pinta una              //
   //               especializacin                                           //
   //             - public void PintarTipo_U_I: Pinta un tipo unin o un tipo //
   //               interseccin                                              //
   // Descripcin: Traza una lnea doble entre dos elementos del esquema con  //
   //              la posibilidad de dibujar sobre ella el smbolo de         //
   //              inclusin                                                  //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   public void TrazarLineaDoble(Graphics g,
                                int pnAnchoLinea,
                                int pnXOrigen,
                                int pnYOrigen,
                                int pnAnchoOrigen,
                                int pnAltoOrigen,
                                int pnXDestino,
                                int pnYDestino,
                                int pnAnchoDestino,
                                int pnAltoDestino,
                                boolean pbInclusion) {

      double dPendiente; // Pendiente de la recta
      double dAngulo; // ngulo de la recta con la horizontal
      int nOffsetX; // Desplazamiento horizontal
      int nOffsetY; // Desplazamiento vertical

      // Inicializar los valores de las variables
      int nXCentroOrigen = pnXOrigen + pnAnchoOrigen / 2;
      int nYCentroOrigen = pnYOrigen + pnAltoOrigen / 2;
      int nXCentroDestino = pnXDestino + pnAnchoDestino / 2;
      int nYCentroDestino = pnYDestino + pnAltoDestino / 2;

      if (pnXOrigen > pnXDestino + pnAnchoDestino) {
         nXCentroDestino = pnXDestino + pnAnchoDestino;
         nYCentroDestino = pnYDestino + pnAltoDestino / 2;

         dPendiente = - ( (double) (nYCentroDestino - nYCentroOrigen) /
                         (double) (nXCentroDestino - nXCentroOrigen));
         dAngulo = Math.abs(Math.atan(dPendiente));
         nOffsetY = pnAnchoLinea + (int) Math.sin(dAngulo);
         nOffsetX = pnAnchoLinea + (int) Math.cos(dAngulo);

         dAngulo = dAngulo(nXCentroOrigen, nYCentroOrigen,
                           nXCentroDestino, nYCentroDestino);

         if (nYCentroOrigen < nYCentroDestino) {
            g.drawLine(nXCentroOrigen - nOffsetX +
                       (int) ( (clsConstantes.nANCHO_ESPGEN_UNCAT / 2) *
                              Math.cos(Math.toRadians(dAngulo))),
                       nYCentroOrigen - nOffsetY -
                       (int) ( (clsConstantes.nALTO_ESPGEN_UNCAT / 2) *
                              Math.sin(Math.toRadians(dAngulo))),
                       nXCentroDestino - nOffsetX,
                       nYCentroDestino - nOffsetY);
            g.drawLine(nXCentroOrigen + nOffsetX +
                       (int) ( (clsConstantes.nANCHO_ESPGEN_UNCAT / 2) *
                              Math.cos(Math.toRadians(dAngulo))),
                       nYCentroOrigen + nOffsetY -
                       (int) ( (clsConstantes.nALTO_ESPGEN_UNCAT / 2) *
                              Math.sin(Math.toRadians(dAngulo))),
                       nXCentroDestino + nOffsetX,
                       nYCentroDestino + nOffsetY);

         } // if
         else {
            g.drawLine(nXCentroOrigen + nOffsetX +
                       (int) ( (clsConstantes.nANCHO_ESPGEN_UNCAT / 2) *
                              Math.cos(Math.toRadians(dAngulo))),
                       nYCentroOrigen - nOffsetY -
                       (int) ( (clsConstantes.nALTO_ESPGEN_UNCAT / 2) *
                              Math.sin(Math.toRadians(dAngulo))),
                       nXCentroDestino + nOffsetX,
                       nYCentroDestino - nOffsetY);
            g.drawLine(nXCentroOrigen - nOffsetX +
                       (int) ( (clsConstantes.nANCHO_ESPGEN_UNCAT / 2) *
                              Math.cos(Math.toRadians(dAngulo))),
                       nYCentroOrigen + nOffsetY -
                       (int) ( (clsConstantes.nALTO_ESPGEN_UNCAT / 2) *
                              Math.sin(Math.toRadians(dAngulo))),
                       nXCentroDestino - nOffsetX,
                       nYCentroDestino + nOffsetY);

         } // else
      } // if
      else if (pnXOrigen + pnAnchoOrigen < pnXDestino) {
         nXCentroDestino = pnXDestino;
         nYCentroDestino = pnYDestino + pnAltoDestino / 2;

         dPendiente = - ( (double) (nYCentroDestino - nYCentroOrigen) /
                         (double) (nXCentroDestino - nXCentroOrigen));
         dAngulo = Math.abs(Math.atan(dPendiente));
         nOffsetY = pnAnchoLinea + (int) Math.sin(dAngulo);
         nOffsetX = pnAnchoLinea + (int) Math.cos(dAngulo);

         dAngulo = dAngulo(nXCentroOrigen, nYCentroOrigen,
                           nXCentroDestino, nYCentroDestino);

         if (nYCentroOrigen < nYCentroDestino) {
            g.drawLine(nXCentroOrigen - nOffsetX +
                       (int) ( (clsConstantes.nANCHO_ESPGEN_UNCAT / 2) *
                              Math.cos(Math.toRadians(dAngulo))),
                       nYCentroOrigen + nOffsetY -
                       (int) ( (clsConstantes.nALTO_ESPGEN_UNCAT / 2) *
                              Math.sin(Math.toRadians(dAngulo))),
                       nXCentroDestino - nOffsetX,
                       nYCentroDestino + nOffsetY);
            g.drawLine(nXCentroOrigen + nOffsetX +
                       (int) ( (clsConstantes.nANCHO_ESPGEN_UNCAT / 2) *
                              Math.cos(Math.toRadians(dAngulo))),
                       nYCentroOrigen - nOffsetY -
                       (int) ( (clsConstantes.nALTO_ESPGEN_UNCAT / 2) *
                              Math.sin(Math.toRadians(dAngulo))),
                       nXCentroDestino + nOffsetX,
                       nYCentroDestino - nOffsetY);

         } // if
         else {
            g.drawLine(nXCentroOrigen - nOffsetX +
                       (int) ( (clsConstantes.nANCHO_ESPGEN_UNCAT / 2) *
                              Math.cos(Math.toRadians(dAngulo))),
                       nYCentroOrigen - nOffsetY -
                       (int) ( (clsConstantes.nALTO_ESPGEN_UNCAT / 2) *
                              Math.sin(Math.toRadians(dAngulo))),
                       nXCentroDestino - nOffsetX,
                       nYCentroDestino - nOffsetY);
            g.drawLine(nXCentroOrigen + nOffsetX +
                       (int) ( (clsConstantes.nANCHO_ESPGEN_UNCAT / 2) *
                              Math.cos(Math.toRadians(dAngulo))),
                       nYCentroOrigen + nOffsetY -
                       (int) ( (clsConstantes.nALTO_ESPGEN_UNCAT / 2) *
                              Math.sin(Math.toRadians(dAngulo))),
                       nXCentroDestino + nOffsetX,
                       nYCentroDestino + nOffsetY);

         } // else
      } // else if
      else {
         if (pnYOrigen < pnYDestino) {
            nXCentroDestino = pnXDestino + pnAnchoDestino / 2;
            nYCentroDestino = pnYDestino;

            dPendiente = - ( (double) (nYCentroDestino - nYCentroOrigen) /
                            (double) (nXCentroDestino - nXCentroOrigen));
            dAngulo = Math.abs(Math.atan(dPendiente));
            nOffsetX = pnAnchoLinea + (int) Math.cos(dAngulo);
            nOffsetY = pnAnchoLinea + (int) Math.sin(dAngulo);

            dAngulo = dAngulo(nXCentroOrigen, nYCentroOrigen,
                              nXCentroDestino, nYCentroDestino);

            if (nXCentroOrigen < nXCentroDestino) {
               g.drawLine(nXCentroOrigen - nOffsetX +
                          (int) ( (clsConstantes.nANCHO_ESPGEN_UNCAT / 2) *
                                 Math.cos(Math.toRadians(dAngulo))),
                          nYCentroOrigen + nOffsetY -
                          (int) ( (clsConstantes.nALTO_ESPGEN_UNCAT / 2) *
                                 Math.sin(Math.toRadians(dAngulo))),
                          nXCentroDestino - nOffsetX,
                          nYCentroDestino + nOffsetY);
               g.drawLine(nXCentroOrigen + nOffsetX +
                          (int) ( (clsConstantes.nANCHO_ESPGEN_UNCAT / 2) *
                                 Math.cos(Math.toRadians(dAngulo))),
                          nYCentroOrigen - nOffsetY -
                          (int) ( (clsConstantes.nALTO_ESPGEN_UNCAT / 2) *
                                 Math.sin(Math.toRadians(dAngulo))),
                          nXCentroDestino + nOffsetX,
                          nYCentroDestino - nOffsetY);

            }
            else {
               g.drawLine(nXCentroOrigen - nOffsetX +
                          (int) ( (clsConstantes.nANCHO_ESPGEN_UNCAT / 2) *
                                 Math.cos(Math.toRadians(dAngulo))),
                          nYCentroOrigen - nOffsetY -
                          (int) ( (clsConstantes.nALTO_ESPGEN_UNCAT / 2) *
                                 Math.sin(Math.toRadians(dAngulo))),
                          nXCentroDestino - nOffsetX,
                          nYCentroDestino - nOffsetY);
               g.drawLine(nXCentroOrigen + nOffsetX +
                          (int) ( (clsConstantes.nANCHO_ESPGEN_UNCAT / 2) *
                                 Math.cos(Math.toRadians(dAngulo))),
                          nYCentroOrigen + nOffsetY -
                          (int) ( (clsConstantes.nALTO_ESPGEN_UNCAT / 2) *
                                 Math.sin(Math.toRadians(dAngulo))),
                          nXCentroDestino + nOffsetX,
                          nYCentroDestino + nOffsetY);

            } // else
         } // if
         else {
            nXCentroDestino = pnXDestino + pnAnchoDestino / 2;
            nYCentroDestino = pnYDestino + pnAltoDestino;

            dPendiente = - ( (double) (nYCentroDestino - nYCentroOrigen) /
                            (double) (nXCentroDestino - nXCentroOrigen));
            dAngulo = Math.abs(Math.atan(dPendiente));
            nOffsetX = pnAnchoLinea + (int) Math.cos(dAngulo);
            nOffsetY = pnAnchoLinea + (int) Math.sin(dAngulo);

            dAngulo = dAngulo(nXCentroOrigen, nYCentroOrigen,
                              nXCentroDestino, nYCentroDestino);

            if (nXCentroOrigen < nXCentroDestino) {
               g.drawLine(nXCentroOrigen - nOffsetX +
                          (int) ( (clsConstantes.nANCHO_ESPGEN_UNCAT / 2) *
                                 Math.cos(Math.toRadians(dAngulo))),
                          nYCentroOrigen - nOffsetY -
                          (int) ( (clsConstantes.nALTO_ESPGEN_UNCAT / 2) *
                                 Math.sin(Math.toRadians(dAngulo))),
                          nXCentroDestino - nOffsetX,
                          nYCentroDestino - nOffsetY);
               g.drawLine(nXCentroOrigen + nOffsetX +
                          (int) ( (clsConstantes.nANCHO_ESPGEN_UNCAT / 2) *
                                 Math.cos(Math.toRadians(dAngulo))),
                          nYCentroOrigen + nOffsetY -
                          (int) ( (clsConstantes.nALTO_ESPGEN_UNCAT / 2) *
                                 Math.sin(Math.toRadians(dAngulo))),
                          nXCentroDestino + nOffsetX,
                          nYCentroDestino + nOffsetY);

            } // if
            else {
               g.drawLine(nXCentroOrigen - nOffsetX +
                          (int) ( (clsConstantes.nANCHO_ESPGEN_UNCAT / 2) *
                                 Math.cos(Math.toRadians(dAngulo))),
                          nYCentroOrigen + nOffsetY -
                          (int) ( (clsConstantes.nALTO_ESPGEN_UNCAT / 2) *
                                 Math.sin(Math.toRadians(dAngulo))),
                          nXCentroDestino - nOffsetX,
                          nYCentroDestino + nOffsetY);
               g.drawLine(nXCentroOrigen + nOffsetX +
                          (int) ( (clsConstantes.nANCHO_ESPGEN_UNCAT / 2) *
                                 Math.cos(Math.toRadians(dAngulo))),
                          nYCentroOrigen - nOffsetY -
                          (int) ( (clsConstantes.nALTO_ESPGEN_UNCAT / 2) *
                                 Math.sin(Math.toRadians(dAngulo))),
                          nXCentroDestino + nOffsetX,
                          nYCentroDestino - nOffsetY);

            } // else
         } // else
      } // else

      // Comprobar si se pinta el smbolo de inclusin
      if (pbInclusion) {
         PintarInclusion(g, nXCentroOrigen, nYCentroOrigen, nXCentroDestino,
                         nYCentroDestino);
      } // if
   } // Procedimiento: TrazarLineaDoble

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: TrazarLineaDoble                                         //
   // Parmetros: - g (Graphics): Objeto grfico                              //
   //             - pnAnchoLinea (int): Ancho de la lnea                     //
   //             - pnXOrigen (int): Posicin horizontal del origen           //
   //             - pnYOrigen (int): Posicin vertical del origen             //
   //             - pnAnchoOrigen (int): Ancho del origen                     //
   //             - pnAltoOrigen (int): Alto del origen                       //
   //             - pnXDestino (int): Posicin horizontal del destino         //
   //             - pnYDestino (int): Posicin vertical del destino           //
   //             - pnAnchoDestino (int): Ancho del destino                   //
   //             - pnAltoDestino (int): Alto del destino                     //
   //             - psMin (String): Cardinalidad mnima                       //
   //             - psMax (String): Cardinalidad mxima                       //
   // Descripcin: Traza una lnea doble entre dos elementos del esquema con  //
   //              la cardinalidad con la que participan las entidades en una //
   //              relacin                                                   //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   public void TrazarLineaDoble(Graphics g,
                                int nAnchoLinea,
                                int pnXOrigen,
                                int pnYOrigen,
                                int pnAnchoOrigen,
                                int pnAltoOrigen,
                                int pnXDestino,
                                int pnYDestino,
                                int pnAnchoDestino,
                                int pnAltoDestino,
                                String psMin,
                                String psMax) {

      // Declaracin de variables

      int nXCentroOrigen;
      int nYCentroOrigen;
      int nXCentroDestino;
      int nYCentroDestino;
      double dPendiente;
      double dAngulo;
      int nOffsetX;
      int nOffsetY;
      int nAjusteX;
      int nAjusteY;

      if (pnXOrigen > pnXDestino + pnAnchoDestino) {
         nXCentroOrigen = pnXOrigen;
         nYCentroOrigen = pnYOrigen + pnAltoOrigen / 2;
         nXCentroDestino = pnXDestino + pnAnchoDestino;
         nYCentroDestino = pnYDestino + pnAltoDestino / 2;

         dPendiente = - ( (double) (nYCentroDestino - nYCentroOrigen) /
                         (double) (nXCentroDestino - nXCentroOrigen));
         dAngulo = Math.abs(Math.atan(dPendiente));
         nOffsetY = nAnchoLinea + (int) Math.sin(dAngulo);
         nOffsetX = nAnchoLinea + (int) Math.cos(dAngulo);

         dAngulo = dAngulo(nXCentroOrigen, nYCentroOrigen,
                           nXCentroDestino, nYCentroDestino);

         if (nYCentroOrigen < nYCentroDestino) {
            g.drawLine(nXCentroOrigen - nOffsetX, nYCentroOrigen - nOffsetY,
                       nXCentroDestino - nOffsetX, nYCentroDestino - nOffsetY);
            g.drawLine(nXCentroOrigen + nOffsetX, nYCentroOrigen + nOffsetY,
                       nXCentroDestino + nOffsetX, nYCentroDestino + nOffsetY);

            nAjusteX = 18 - (int) (18 * (Math.sin(Math.toRadians(dAngulo)))) +
               nOffsetX;
            nAjusteY = 9 + (int) (9 * Math.sin(Math.toRadians(dAngulo)));
         } // if
         else {
            g.drawLine(nXCentroOrigen + nOffsetX, nYCentroOrigen - nOffsetY,
                       nXCentroDestino + nOffsetX, nYCentroDestino - nOffsetY);
            g.drawLine(nXCentroOrigen - nOffsetX, nYCentroOrigen + nOffsetY,
                       nXCentroDestino - nOffsetX, nYCentroDestino + nOffsetY);

            nAjusteX = 18 - (int) (18 * (Math.sin(Math.toRadians(dAngulo)))) -
               nOffsetX;
            nAjusteY = 9 - (int) (9 * Math.sin(Math.toRadians(dAngulo)));
         } // else
      } // if
      else if (pnXOrigen + pnAnchoOrigen < pnXDestino) {
         nXCentroOrigen = pnXOrigen + pnAnchoOrigen;
         nYCentroOrigen = pnYOrigen + pnAltoOrigen / 2;
         nXCentroDestino = pnXDestino;
         nYCentroDestino = pnYDestino + pnAltoDestino / 2;

         dPendiente = - ( (double) (nYCentroDestino - nYCentroOrigen) /
                         (double) (nXCentroDestino - nXCentroOrigen));
         dAngulo = Math.abs(Math.atan(dPendiente));
         nOffsetY = nAnchoLinea + (int) Math.sin(dAngulo);
         nOffsetX = nAnchoLinea + (int) Math.cos(dAngulo);

         dAngulo = dAngulo(nXCentroOrigen,
                           nYCentroOrigen,
                           nXCentroDestino,
                           nYCentroDestino);

         if (nYCentroOrigen < nYCentroDestino) {
            g.drawLine(nXCentroOrigen - nOffsetX,
                       nYCentroOrigen + nOffsetY,
                       nXCentroDestino - nOffsetX,
                       nYCentroDestino + nOffsetY);
            g.drawLine(nXCentroOrigen + nOffsetX,
                       nYCentroOrigen - nOffsetY,
                       nXCentroDestino + nOffsetX,
                       nYCentroDestino - nOffsetY);

            nAjusteX = 18 + (int) (18 * (Math.sin(Math.toRadians(dAngulo)))) -
               nOffsetX;
            nAjusteY = 9 + (int) (9 * Math.sin(Math.toRadians(dAngulo)));

         } // if
         else {
            g.drawLine(nXCentroOrigen - nOffsetX,
                       nYCentroOrigen - nOffsetY,
                       nXCentroDestino - nOffsetX,
                       nYCentroDestino - nOffsetY);
            g.drawLine(nXCentroOrigen + nOffsetX,
                       nYCentroOrigen + nOffsetY,
                       nXCentroDestino + nOffsetX,
                       nYCentroDestino + nOffsetY);

            nAjusteX = 18 + (int) (18 * (Math.sin(Math.toRadians(dAngulo)))) +
               nOffsetX;
            nAjusteY = 9 - (int) (9 * Math.sin(Math.toRadians(dAngulo)));
         } // else
      } // else if
      else {
         if (pnYOrigen < pnYDestino) {
            nXCentroOrigen = pnXOrigen + pnAnchoOrigen / 2;
            nYCentroOrigen = pnYOrigen + pnAltoOrigen;
            nXCentroDestino = pnXDestino + pnAnchoDestino / 2;
            nYCentroDestino = pnYDestino;

            dPendiente = - ( (double) (nYCentroDestino - nYCentroOrigen) /
                            (double) (nXCentroDestino - nXCentroOrigen));
            dAngulo = Math.abs(Math.atan(dPendiente));
            nOffsetX = nAnchoLinea + (int) Math.cos(dAngulo);
            nOffsetY = nAnchoLinea + (int) Math.sin(dAngulo);

            dAngulo = dAngulo(nXCentroOrigen,
                              nYCentroOrigen,
                              nXCentroDestino,
                              nYCentroDestino);

            if (nXCentroOrigen < nXCentroDestino) {
               g.drawLine(nXCentroOrigen - nOffsetX,
                          nYCentroOrigen + nOffsetY,
                          nXCentroDestino - nOffsetX,
                          nYCentroDestino + nOffsetY);
               g.drawLine(nXCentroOrigen + nOffsetX,
                          nYCentroOrigen - nOffsetY,
                          nXCentroDestino + nOffsetX,
                          nYCentroDestino - nOffsetY);

               nAjusteX = 18 + (int) (18 * (Math.sin(Math.toRadians(dAngulo)))) -
                  nOffsetX;
               nAjusteY = 9 + (int) (9 * Math.sin(Math.toRadians(dAngulo)));
            }
            else {
               g.drawLine(nXCentroOrigen - nOffsetX,
                          nYCentroOrigen - nOffsetY,
                          nXCentroDestino - nOffsetX,
                          nYCentroDestino - nOffsetY);
               g.drawLine(nXCentroOrigen + nOffsetX,
                          nYCentroOrigen + nOffsetY,
                          nXCentroDestino + nOffsetX,
                          nYCentroDestino + nOffsetY);

               nAjusteX = 18 - (int) (18 * (Math.sin(Math.toRadians(dAngulo)))) +
                  nOffsetX;
               nAjusteY = 9 + (int) (9 * Math.sin(Math.toRadians(dAngulo)));
            } // else
         } // if
         else {
            nXCentroOrigen = pnXOrigen + pnAnchoOrigen / 2;
            nYCentroOrigen = pnYOrigen;
            nXCentroDestino = pnXDestino + pnAnchoDestino / 2;
            nYCentroDestino = pnYDestino + pnAltoDestino;

            dPendiente = - ( (double) (nYCentroDestino - nYCentroOrigen) /
                            (double) (nXCentroDestino - nXCentroOrigen));
            dAngulo = Math.abs(Math.atan(dPendiente));
            nOffsetX = nAnchoLinea + (int) Math.cos(dAngulo);
            nOffsetY = nAnchoLinea + (int) Math.sin(dAngulo);

            dAngulo = dAngulo(nXCentroOrigen,
                              nYCentroOrigen,
                              nXCentroDestino,
                              nYCentroDestino);

            if (nXCentroOrigen < nXCentroDestino) {
               g.drawLine(nXCentroOrigen - nOffsetX,
                          nYCentroOrigen - nOffsetY,
                          nXCentroDestino - nOffsetX,
                          nYCentroDestino - nOffsetY);
               g.drawLine(nXCentroOrigen + nOffsetX,
                          nYCentroOrigen + nOffsetY,
                          nXCentroDestino + nOffsetX,
                          nYCentroDestino + nOffsetY);

               nAjusteX = 18 + (int) (18 * (Math.sin(Math.toRadians(dAngulo)))) +
                  nOffsetX;
               nAjusteY = 9 - (int) (9 * Math.sin(Math.toRadians(dAngulo)));
            } // if
            else {
               g.drawLine(nXCentroOrigen - nOffsetX,
                          nYCentroOrigen + nOffsetY,
                          nXCentroDestino - nOffsetX,
                          nYCentroDestino + nOffsetY);
               g.drawLine(nXCentroOrigen + nOffsetX,
                          nYCentroOrigen - nOffsetY,
                          nXCentroDestino + nOffsetX,
                          nYCentroDestino - nOffsetY);

               nAjusteX = 18 - (int) (18 * (Math.sin(Math.toRadians(dAngulo)))) -
                  nOffsetX;
               nAjusteY = 9 - (int) (9 * Math.sin(Math.toRadians(dAngulo)));

            } // else
         } // else
      } // else

      // Escribir la cardinalidad
      g.drawString("(" + psMin + "," + psMax + ")",
                   (nXCentroOrigen + nXCentroDestino) / 2 - nAjusteX,
                   (nYCentroOrigen + nYCentroDestino) / 2 - nAjusteY);
   } // Procedimiento: TrazarLineaDoble

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: TrazarLinea                                              //
   // Parmetros: - g (Graphics): Objeto grfico                              //
   //             - pnXOrigen (int): Posicin horizontal del origen           //
   //             - pnYOrigen (int): Posicin vertical del origen             //
   //             - pnAnchoOrigen (int): Ancho del origen                     //
   //             - pnAltoOrigen (int): Alto del origen                       //
   //             - pnXDestino (int): Posicin horizontal del destino         //
   //             - pnYDestino (int): Posicin vertical del destino           //
   //             - pnAnchoDestino (int): Ancho del destino                   //
   //             - pnAltoDestino (int): Alto del destino                     //
   //             - pbInclusion (boolean): Indica si se dibuja el smbolo de  //
   //               inclusin                                                 //
   // Descripcin: Traza una lnea entre dos elementos del esquema con la     //
   //              posibilidad de dibujar sobre ella el smbolo de inclusin  //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   public void TrazarLinea(Graphics g,
                           int pnXOrigen,
                           int pnYOrigen,
                           int pnAnchoOrigen,
                           int pnAltoOrigen,
                           int pnXDestino,
                           int pnYDestino,
                           int pnAnchoDestino,
                           int pnAltoDestino,
                           boolean pbInclusion) {

      int nXCentroOrigen = pnXOrigen + pnAnchoOrigen / 2;
      int nYCentroOrigen = pnYOrigen + pnAltoOrigen / 2;
      int nXCentroDestino;
      int nYCentroDestino;

      double dAngulo;

      if (pnXOrigen > pnXDestino + pnAnchoDestino) {
         if (pnAnchoOrigen == clsConstantes.nANCHO_ENTIDAD) {
            nXCentroOrigen = pnXOrigen;
         } // if
         nXCentroDestino = pnXDestino + pnAnchoDestino;
         nYCentroDestino = pnYDestino + pnAltoDestino / 2;
      } // if
      else if (pnXOrigen + pnAnchoOrigen < pnXDestino) {
         if (pnAnchoOrigen == clsConstantes.nANCHO_ENTIDAD) {
            nXCentroOrigen = pnXOrigen + pnAnchoOrigen;
         } // if
         nXCentroDestino = pnXDestino;
         nYCentroDestino = pnYDestino + pnAltoDestino / 2;
      } // else if
      else {
         if (pnYOrigen < pnYDestino) {
            if (pnAnchoOrigen == clsConstantes.nANCHO_ENTIDAD) {
               nYCentroOrigen = pnYOrigen + pnAltoOrigen;
            } // if
            nXCentroDestino = pnXDestino + pnAnchoDestino / 2;
            nYCentroDestino = pnYDestino;
         } // if
         else {
            if (pnAnchoOrigen == clsConstantes.nANCHO_ENTIDAD) {
               nYCentroOrigen = pnYOrigen;
            } // if

            nXCentroDestino = pnXDestino + pnAnchoDestino / 2;
            nYCentroDestino = pnYDestino + pnAltoDestino;
         } // else
      } // else

      // Dibujar la lnea
      dAngulo = dAngulo(nXCentroOrigen,
                        nYCentroOrigen,
                        nXCentroDestino,
                        nYCentroDestino);

      if (pbInclusion) {
         PintarInclusion(g,
                         nXCentroOrigen,
                         nYCentroOrigen,
                         nXCentroDestino,
                         nYCentroDestino);
      } // if

      if (pnAnchoOrigen == clsConstantes.nANCHO_ENTIDAD) {
         g.drawLine(nXCentroOrigen,
                    nYCentroOrigen,
                    nXCentroDestino,
                    nYCentroDestino);
      } // if
      else {
         g.drawLine(nXCentroOrigen +
                    (int) ( (clsConstantes.nANCHO_ESPGEN_UNCAT / 2) *
                           Math.cos(Math.toRadians(dAngulo))),
                    nYCentroOrigen -
                    (int) ( (clsConstantes.nALTO_ESPGEN_UNCAT / 2) *
                           Math.sin(Math.toRadians(dAngulo))),
                    nXCentroDestino, nYCentroDestino);
      } // else
   } // Procedimiento: TrazarLinea

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: TrazarLinea                                              //
   // Parmetros: - g (Graphics): Objeto grfico                              //
   //             - pnXOrigen (int): Posicin horizontal del origen           //
   //             - pnYOrigen (int): Posicin vertical del origen             //
   //             - pnXDestino (int): Posicin horizontal del destino         //
   //             - pnYXDestino (int): Posicin vertical del destino          //
   // Descripcin: Dibuja el smbolo de inclusin                             //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private void PintarInclusion(Graphics g,
                                int pnXOrigen,
                                int pnYOrigen,
                                int pnXDestino,
                                int pnYDestino) {

      int x = (pnXOrigen + pnXDestino) / 2;
      int y = (pnYOrigen + pnYDestino) / 2;

      double dPendiente;
      double dAngulo;
      int nOffset;

      if (pnXDestino >= pnXOrigen) {
         nOffset = 90;
      }
      else {
         nOffset = 270;
      }

      dPendiente = - ( (double) (pnYDestino - pnYOrigen) /
                      (double) (pnXDestino - pnXOrigen));
      dAngulo = Math.atan(dPendiente);
      g.drawArc(x - 10, y - 10, 20, 20, nOffset + (int) Math.toDegrees(dAngulo),
                -180);
   } // Procedimiento: PintarInclusion

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: TrazarLineaDoble                                         //
   // Parmetros: - g (Graphics): Objeto grfico                              //
   //             - pnXOrigen (int): Posicin horizontal del origen           //
   //             - pnYOrigen (int): Posicin vertical del origen             //
   //             - pnAnchoOrigen (int): Ancho del origen                     //
   //             - pnAltoOrigen (int): Alto del origen                       //
   //             - pnXDestino (int): Posicin horizontal del destino         //
   //             - pnYDestino (int): Posicin vertical del destino           //
   //             - pnAnchoDestino (int): Ancho del destino                   //
   //             - pnAltoDestino (int): Alto del destino                     //
   //             - psMin (String): Cardinalidad mnima                       //
   //             - psMax (String): Cardinalidad mxima                       //
   // Descripcin: Traza una lnea entre dos elementos del esquema con la     //
   //              cardinalidad con la que participan las entidades en una    //
   //              relacin                                                   //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   public void TrazarLinea(Graphics g,
                           int pnXOrigen,
                           int pnYOrigen,
                           int pnAnchoOrigen,
                           int pnAltoOrigen,
                           int pnXDestino,
                           int pnYDestino,
                           int pnAnchoDestino,
                           int pnAltoDestino,
                           String psMin,
                           String psMax) {

      int nXCentroOrigen;
      int nYCentroOrigen;
      int nXCentroDestino;
      int nYCentroDestino;
      int nAjusteX;
      int nAjusteY;

      double dAngulo;

      if (pnXOrigen > pnXDestino + pnAnchoDestino) {
         nXCentroOrigen = pnXOrigen;
         nYCentroOrigen = pnYOrigen + pnAltoOrigen / 2;
         nXCentroDestino = pnXDestino + pnAnchoDestino;
         nYCentroDestino = pnYDestino + pnAltoDestino / 2;

         dAngulo = dAngulo(nXCentroOrigen, nYCentroOrigen,
                           nXCentroDestino, nYCentroDestino);

         if (nYCentroOrigen < nYCentroDestino) {
            nAjusteX = 18 - (int) (18 * (Math.sin(Math.toRadians(dAngulo))));
            nAjusteY = 7 + (int) (7 * Math.sin(Math.toRadians(dAngulo)));
         } // if
         else {
            nAjusteX = 18 - (int) (18 * (Math.sin(Math.toRadians(dAngulo))));
            nAjusteY = 7 - (int) (7 * Math.sin(Math.toRadians(dAngulo)));
         } // else

         g.drawString("(" + psMin + "," + psMax + ")",
                      ( (nXCentroOrigen + nXCentroDestino) / 2 - nAjusteX),
                      ( (nYCentroOrigen + nYCentroDestino) / 2) - nAjusteY);

      } // if
      else if (pnXOrigen + pnAnchoOrigen < pnXDestino) {
         nXCentroOrigen = pnXOrigen + pnAnchoOrigen;
         nYCentroOrigen = pnYOrigen + pnAltoOrigen / 2;
         nXCentroDestino = pnXDestino;
         nYCentroDestino = pnYDestino + pnAltoDestino / 2;

         dAngulo = dAngulo(nXCentroOrigen,
                           nYCentroOrigen,
                           nXCentroDestino,
                           nYCentroDestino);

         if (nYCentroOrigen < nYCentroDestino) {
            nAjusteX = 16 + (int) (16 * (Math.sin(Math.toRadians(dAngulo))));
            nAjusteY = 7 + (int) (7 * Math.sin(Math.toRadians(dAngulo)));
         } // if
         else {
            nAjusteX = 18 + (int) (18 * (Math.sin(Math.toRadians(dAngulo))));
            nAjusteY = 7 - (int) (7 * Math.sin(Math.toRadians(dAngulo)));
         } // else

         g.drawString("(" + psMin + "," + psMax + ")",
                      ( (nXCentroOrigen + nXCentroDestino) / 2 - nAjusteX),
                      ( (nYCentroOrigen + nYCentroDestino) / 2) - nAjusteY);

      } // else if
      else {
         if (pnYOrigen < pnYDestino) {
            nXCentroOrigen = pnXOrigen + pnAnchoOrigen / 2;
            nYCentroOrigen = pnYOrigen + pnAltoOrigen;
            nXCentroDestino = pnXDestino + pnAnchoDestino / 2;
            nYCentroDestino = pnYDestino;

            dAngulo = dAngulo(nXCentroOrigen,
                              nYCentroOrigen,
                              nXCentroDestino,
                              nYCentroDestino);

            if (nXCentroOrigen < nXCentroDestino) {
               nAjusteX = 16 + (int) (16 * (Math.sin(Math.toRadians(dAngulo))));
               nAjusteY = 7 + (int) (7 * Math.sin(Math.toRadians(dAngulo)));
            } // if
            else {
               nAjusteX = 18 - (int) (18 * (Math.sin(Math.toRadians(dAngulo))));
               nAjusteY = 7 + (int) (7 * Math.sin(Math.toRadians(dAngulo)));
            } // else
            g.drawString("(" + psMin + "," + psMax + ")",
                         ( (nXCentroOrigen + nXCentroDestino) / 2 - nAjusteX),
                         ( (nYCentroOrigen + nYCentroDestino) / 2) -
                         nAjusteY);

         } // if
         else {
            nXCentroOrigen = pnXOrigen + pnAnchoOrigen / 2;
            nYCentroOrigen = pnYOrigen;
            nXCentroDestino = pnXDestino + pnAnchoDestino / 2;
            nYCentroDestino = pnYDestino + pnAltoDestino;

            dAngulo = dAngulo(nXCentroOrigen, nYCentroOrigen,
                              nXCentroDestino, nYCentroDestino);

            if (nXCentroOrigen < nXCentroDestino) {
               nAjusteX = 18 + (int) (18 * (Math.sin(Math.toRadians(dAngulo))));
               nAjusteY = 7 - (int) (7 * Math.sin(Math.toRadians(dAngulo)));
            } // if
            else {
               nAjusteX = 18 - (int) (18 * (Math.sin(Math.toRadians(dAngulo))));
               nAjusteY = 7 - (int) (7 * Math.sin(Math.toRadians(dAngulo)));
            } // else
         } // else
      } // else

      // Dibujar la lnea
      g.drawLine(nXCentroOrigen,
                 nYCentroOrigen,
                 nXCentroDestino,
                 nYCentroDestino);

      g.drawString("(" + psMin + "," + psMax + ")",
                   ( (nXCentroOrigen + nXCentroDestino) / 2 - nAjusteX),
                   ( (nYCentroOrigen + nYCentroDestino) / 2) -
                   nAjusteY);
   } // Procedimiento: TrazarLinea

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Funcin: dAngulo                                                        //
   // Parmetros: - pnXOrigen (int): Posicin horizontal del origen           //
   //             - pnYOrigen (int): Posicin vertical del origen             //
   //             - pnXDestino (int): Posicin horizontal del destino         //
   //             - pnYDestino (int): Posicin vertical del destino           //
   // Retorno: Angulo en grados                                               //
   // Descripcin: Calcula el ngulo entre la recta formada por los dos       //
   //              y la horizontal                                            //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private double dAngulo(int pnXOrigen,
                          int pnYOrigen,
                          int pnXDestino,
                          int pnYDestino) {

      double dPendiente; // Pendiente del vector formado por los puntos
      double dAngulo; // ngulo con la horizontal del vector formado por los puntos

      // Calcular la pendiente del vector formado por los puntos
      dPendiente = - ( (double) (pnYDestino - pnYOrigen) /
                      (double) (pnXDestino - pnXOrigen));
      // Calcular el ngulo con la horizontal del vector formado por los puntos
      dAngulo = Math.toDegrees(Math.atan(dPendiente));

      if (pnXDestino < pnXOrigen) {
         dAngulo = 180 + dAngulo;
      } // if

      // Devolver el ngulo
      return dAngulo;
   } // Funcin: dAngulo

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: PintarEntidad                                            //
   // Parmetros: - g (Graphics): Objeto grfico                              //
   //             - c (Color): Color para pintar                              //
   //             - pnX (int): Posicin horizontal del elemento               //
   //             - pnY (int): Posicin vertical del elemento                 //
   //             - psNombre (String): Nombre del elemento                    //
   //             - pnVerAtributos (int): Opcin para ver los atributos       //
   //             - pvAtributos (Vector): Vector de atributos                 //
   //             - pbAtribIzq (boolean): Indicador si los atributos se       //
   //               se pintan a la izquierda de la entidad                    //
   // Descripcin: Pinta una entidad                                          //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   public void PintarEntidad(Graphics g,
                             Color c,
                             int pnX,
                             int pnY,
                             String psNombre,
                             int pnVerAtributos,
                             Vector pvAtributos,
                             boolean pbAtribIzq) {

      int iIndice; // ndice para recorrer el vector de atributos
      clsElemento elmElemento;
      clsEntRel erEntidad;
      int nYFin;
      int nNumAtribs = 1;

      // Establecer el color para pintar
      g.setColor(c);
      // Pintar la representacin de la entidad
      g.clearRect(pnX,
                  pnY,
                  clsConstantes.nANCHO_ENTIDAD,
                  clsConstantes.nALTO_ENTIDAD);
      g.drawRect(pnX,
                 pnY,
                 clsConstantes.nANCHO_ENTIDAD,
                 clsConstantes.nALTO_ENTIDAD);
      // Escribir el nombre de la entidad
      g.drawString(psNombre,
                   pnX + clsConstantes.nSEP_COM + 2 +
                   ( (7 - (psNombre.length() / 2)) * 7),
                   pnY + clsConstantes.nALTO_ENTIDAD / 2 +
                   clsConstantes.nSEP_COM);

      // Pintar los atributos de la entidad
      if (pnVerAtributos != clsConstantes.nNINGUNO) {
         nYFin = pnY + clsConstantes.nALTO_ENTIDAD +
            clsConstantes.nALTO_LINEA_ATRIBUTO;

         if (!pbAtribIzq) {
            pnX += clsConstantes.nANCHO_ENTIDAD;
         } // if

         for (iIndice = 0; iIndice < pvAtributos.size(); iIndice++) {
            g.drawLine(pnX, pnY, pnX, nYFin);
            nNumAtribs += PintarAtributo(g,
                                         c,
                                         (clsElemento) pvAtributos.elementAt(iIndice),
                                         pnX,
                                         pnY,
                                         nYFin,
                                         pnVerAtributos);
            pnY = nYFin;
            nYFin += nNumAtribs * clsConstantes.nALTO_LINEA_ATRIBUTO;
            nNumAtribs = 1;
         } // for
      } // if
   } // Procedimiento: PintarEntidad

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: PintarEntidadDebil                                       //
   // Parmetros: - g (Graphics): Objeto grfico                              //
   //             - c (Color): Color para pintar                              //
   //             - pnX (int): Posicin horizontal del elemento               //
   //             - pnY (int): Posicin vertical del elemento                 //
   //             - psNombre (String): Nombre del elemento                    //
   //             - pnVerAtributos (int): Opcin para ver los atributos       //
   //             - pvAtributos (Vector): Vector de atributos                 //
   //             - pbAtribIzq (boolean): Indicador si los atributos se       //
   //               se pintan a la izquierda de la entidad                    //
   // Descripcin: Pinta una entidad                                          //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   public void PintarEntidadDebil(Graphics g,
                                  Color c,
                                  int pnX,
                                  int pnY,
                                  String psNombre,
                                  int pnVerAtributos,
                                  Vector pvAtributos,
                                  boolean pbAtribIzq) {

      int iIndice;
      int nYFin;
      int nNumAtribs = 1;

      // Establecer el color para pintar
      g.setColor(c);
      // Dibujar la representacin de la entidad
      g.clearRect(pnX, pnY, clsConstantes.nANCHO_ENTIDAD,
                  clsConstantes.nALTO_ENTIDAD);
      g.drawRect(pnX, pnY, clsConstantes.nANCHO_ENTIDAD,
                 clsConstantes.nALTO_ENTIDAD);
      g.drawRect(pnX + 4, pnY + 4, clsConstantes.nANCHO_ENTIDAD - 8,
                 clsConstantes.nALTO_ENTIDAD - 8);
      // Escribir el nombre de la entidad
      g.drawString(psNombre,
                   pnX + clsConstantes.nSEP_COM + 2 +
                   ( (7 - (psNombre.length() / 2)) * 7),
                   pnY + clsConstantes.nALTO_ENTIDAD / 2 +
                   clsConstantes.nSEP_COM);

      // Pintar los atributos de la entidad
      if (pnVerAtributos != clsConstantes.nNINGUNO) {
         nYFin = pnY + clsConstantes.nALTO_ENTIDAD +
            clsConstantes.nALTO_LINEA_ATRIBUTO;

         if (!pbAtribIzq) {
            pnX += clsConstantes.nANCHO_ENTIDAD;
         } // if

         for (iIndice = 0; iIndice < pvAtributos.size(); iIndice++) {
            g.drawLine(pnX, pnY, pnX, nYFin);
            nNumAtribs += PintarAtributo(g,
                                         c,
                                         (clsElemento) pvAtributos.elementAt(iIndice),
                                         pnX,
                                         pnY,
                                         nYFin,
                                         pnVerAtributos);
            pnY = nYFin;
            nYFin += nNumAtribs * clsConstantes.nALTO_LINEA_ATRIBUTO;
            nNumAtribs = 1;
         } // for
      } // if
   } // Procedimiento: PintarEntidadDebil

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Funcin: PintarAtributo                                                 //
   // Parmetros: - g (Graphics): Objeto grfico                              //
   //             - c (Color): Color para pintar                              //
   //             - oAtributo (clsElemento): Atributo para dibujar            //
   //             - pnX (int): Posicin horizontal para dibujar               //
   //             - pnYo (int): Posicin vertical original de los atributos   //
   //             - pnY (int): Posicin vertical para dibujar                 //
   //             - psNombre (String): Nombre del elemento                    //
   //             - pnVerAtributos (int): Opcin para ver los atributos       //
   // Retorno: Posicin vertical donde debe dibujarse el siguiente atributo   //
   // Descripcin: Pinta un atributo y devuelve la posicin vertical donde    //
   //              debe dibujarse el siguiente atributo                       //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   public int PintarAtributo(Graphics g,
                             Color c,
                             clsElemento oAtributo,
                             int pnX,
                             int pnYo,
                             int pnY,
                             int pnVerAtributos) {
      int nXFin;
      int nYFin;
      int nAlto;
      int iIndice;
      int nNumAtribs = 1;

      nXFin = pnX + clsConstantes.nSEP_COM;

      // Establecer el color para dibujar el atributo
      g.setColor(c);

      // Dibujar las lneas para el atributo
      g.drawLine(pnX,
                 pnY,
                 nXFin,
                 pnY);
      g.drawLine(pnX,
                 pnYo,
                 pnX,
                 pnY);

      // Dibujar la forma del tipo de atributo
      if (oAtributo.nTipo == clsConstantes.nATRIBUTO_SIMPLE) {
         g.drawOval(nXFin, pnY - (clsConstantes.nALTO_ATRIBUTO / 2),
                    clsConstantes.nANCHO_ATRIBUTO,
                    clsConstantes.nALTO_ATRIBUTO);
      } // if
      else if (oAtributo.nTipo == clsConstantes.nATRIBUTO_DERIVADO) {
         for (int i = 0; i <= 360; i += 20) {
            g.drawArc(nXFin, pnY - (clsConstantes.nALTO_ATRIBUTO / 2),
                      clsConstantes.nANCHO_ATRIBUTO,
                      clsConstantes.nALTO_ATRIBUTO, i, 15);
         } // for
      } // else if
      else if (oAtributo.nTipo == clsConstantes.nATRIBUTO_LLAVE) {
         g.fillOval(nXFin, pnY - (clsConstantes.nALTO_ATRIBUTO / 2),
                    clsConstantes.nANCHO_ATRIBUTO,
                    clsConstantes.nALTO_ATRIBUTO);
      }
      else if (oAtributo.nTipo == clsConstantes.nATRIBUTO_LLAVE_PARCIAL) {
         g.drawOval(nXFin, pnY - (clsConstantes.nALTO_ATRIBUTO / 2),
                    clsConstantes.nANCHO_ATRIBUTO,
                    clsConstantes.nALTO_ATRIBUTO);
         for (int i = clsConstantes.nSEP_COM;
              i <= clsConstantes.nANCHO_ATRIBUTO - clsConstantes.nSEP_COM;
              i += 4) {
            g.drawLine(nXFin + i, pnY, nXFin + i + 2, pnY);
         } // for
      } // else if
      else if (oAtributo.nTipo == clsConstantes.nATRIBUTO_MULTIVALUADO) {
         g.drawOval(nXFin, pnY - (clsConstantes.nALTO_ATRIBUTO / 2),
                    clsConstantes.nANCHO_ATRIBUTO,
                    clsConstantes.nALTO_ATRIBUTO);
         g.drawOval(nXFin + 4,
                    pnY - (clsConstantes.nALTO_ATRIBUTO / 2) + 4,
                    clsConstantes.nANCHO_ATRIBUTO - 8,
                    clsConstantes.nALTO_ATRIBUTO - 8);
      } // else if

      // Escribir el nombre del atributo
      g.drawString(oAtributo.sNombre,
                   nXFin + clsConstantes.nANCHO_ATRIBUTO +
                   clsConstantes.nSEP_COM, pnY + 3);

      // Pintar los atributos del atributo
      nYFin = pnY + clsConstantes.nALTO_LINEA_ATRIBUTO;
      int nNumAtribsTot = 0;
      if (pnVerAtributos == clsConstantes.nTODOS) {
         for (iIndice = 0; iIndice < oAtributo.vAtributos.size();
              iIndice++) {
            nNumAtribs = 1;
            //g.drawLine(pnX, pnY, pnX, nYFin);
            if (iIndice == 0) {
               pnY += clsConstantes.nALTO_ATRIBUTO / 2;
            } // if
            nNumAtribs += PintarAtributo(g, c,
                                         (clsElemento) oAtributo.vAtributos.elementAt(iIndice),
                                         pnX +
                                         clsConstantes.nANCHO_ATRIBUTO / 2 +
                                         clsConstantes.nSEP_COM, pnY,
                                         nYFin, pnVerAtributos);
            pnY = nYFin;
            nYFin += nNumAtribs * clsConstantes.nALTO_LINEA_ATRIBUTO;
            nNumAtribsTot += nNumAtribs;
         } // for
      } // if

      // Devolver el nmero de atributos
      return nNumAtribsTot;
   } // Funcin: PintarAtributo

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: PintarRelacion                                            //
   // Parmetros: - g (Graphics): Objeto grfico                              //
   //             - c (Color): Color para pintar                              //
   //             - pnX (int): Posicin horizontal del elemento               //
   //             - pnY (int): Posicin vertical del elemento                 //
   //             - psNombre (String): Nombre del elemento                    //
   //             - pnVerAtributos (int): Opcin para ver los atributos       //
   //             - pvAtributos (Vector): Vector de atributos                 //
   // Descripcin: Pinta una relacin                                         //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   public void PintarRelacion(Graphics g,
                              Color c,
                              int pnX,
                              int pnY,
                              String psNombre,
                              int pnVerAtributos,
                              Vector pvAtributos) {

      clsEntRel erEntidad;
      clsElemento elmEntidad;
      int iIndice;
      int nYFin;
      int nNumAtribs = 1;

      // Coordenadas X para pintar la Relacin
      int[] nXs = {
         pnX,
         pnX + clsConstantes.nANCHO_RELACION / 2,
         pnX + clsConstantes.nANCHO_RELACION,
         pnX + clsConstantes.nANCHO_RELACION / 2};
      // Coordenadas Y para pintar la Relacin
      int[] nYs = {
         pnY + clsConstantes.nALTO_RELACION / 2,
         pnY,
         pnY + clsConstantes.nALTO_RELACION / 2,
         pnY + clsConstantes.nALTO_RELACION};

      // Dibujar la representacin de la relacin
      g.setColor(clsConstantes.COLOR_FONDO_APPLET);
      g.fillPolygon(nXs, nYs, 4);
      // Establecer el color para pintar
      g.setColor(c);
      g.drawPolygon(nXs, nYs, 4);
      g.drawString(psNombre,
                   pnX + 6 * clsConstantes.nSEP_COM + 2 +
                   ( (7 - (psNombre.length() / 2)) * 7),
                   pnY + clsConstantes.nALTO_RELACION / 2 +
                   clsConstantes.nSEP_COM);

      // Pintar los atributos de la relacin
      if (pnVerAtributos != clsConstantes.nNINGUNO) {
         nYFin = pnY + clsConstantes.nALTO_RELACION +
            clsConstantes.nALTO_LINEA_ATRIBUTO;
         for (iIndice = 0; iIndice < pvAtributos.size(); iIndice++) {
            if (iIndice == 0) {
               pnY += clsConstantes.nALTO_RELACION;

            }
            //g.drawLine(pnX, pnY, pnX, nYFin);
            nNumAtribs += PintarAtributo(g,
                                         c,
                                         (clsElemento) pvAtributos.elementAt(iIndice),
                                         pnX +
                                         clsConstantes.nANCHO_RELACION / 2,
                                         pnY,
                                         nYFin,
                                         pnVerAtributos);
            pnY = nYFin;
            nYFin += nNumAtribs * clsConstantes.nALTO_LINEA_ATRIBUTO;
            nNumAtribs = 1;
         } // for
      } // if
   } // Procedimiento: PintarRelacion

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: PintarRelacionIdentificativa                             //
   // Parmetros: - g (Graphics): Objeto grfico                              //
   //             - c (Color): Color para pintar                              //
   //             - pnX (int): Posicin horizontal del elemento               //
   //             - pnY (int): Posicin vertical del elemento                 //
   //             - psNombre (String): Nombre del elemento                    //
   //             - pnVerAtributos (int): Opcin para ver los atributos       //
   //             - pvAtributos (Vector): Vector de atributos                 //
   // Descripcin: Pinta una relacin                                         //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   public void PintarRelacionIdentificativa(Graphics g,
                                            Color c,
                                            int pnX,
                                            int pnY,
                                            String psNombre,
                                            int pnVerAtributos,
                                            Vector pvAtributos) {

      int nYFin;
      int iIndice;
      int nNumAtribs = 1;

      // Coordenadas X para pintar el borde exterior de la Relacin
      int[] nXs = {
         pnX,
         pnX + clsConstantes.nANCHO_RELACION / 2,
         pnX + clsConstantes.nANCHO_RELACION,
         pnX + clsConstantes.nANCHO_RELACION / 2};
      // Coordenadas Y para pintar el borde exterior de la Relacin
      int[] nYs = {
         pnY + clsConstantes.nALTO_RELACION / 2,
         pnY,
         pnY + clsConstantes.nALTO_RELACION / 2,
         pnY + clsConstantes.nALTO_RELACION};
      // Coordenadas X para pintar el borde interior de la Relacin Identificativa
      int[] nXsRD = {
         pnX + 8,
         pnX + clsConstantes.nANCHO_RELACION / 2,
         pnX + clsConstantes.nANCHO_RELACION - 8,
         pnX + clsConstantes.nANCHO_RELACION / 2};
      // Coordenadas X para pintar el borde interior de la Relacin Identificativa
      int[] nYsRD = {
         pnY + clsConstantes.nALTO_RELACION / 2,
         pnY + 3,
         pnY + clsConstantes.nALTO_RELACION / 2,
         pnY + clsConstantes.nALTO_RELACION - 3};

      g.setColor(clsConstantes.COLOR_FONDO_APPLET);
      g.fillPolygon(nXs, nYs, 4);
      g.setColor(c);
      g.drawPolygon(nXs, nYs, 4);
      g.drawPolygon(nXsRD, nYsRD, 4);
      g.drawString(psNombre,
                   pnX + 6 * clsConstantes.nSEP_COM + 2 +
                   ( (7 - (psNombre.length() / 2)) * 7),
                   pnY + clsConstantes.nALTO_RELACION / 2 +
                   clsConstantes.nSEP_COM);

      // Pintar los atributos de la relacin
      if (pnVerAtributos != clsConstantes.nNINGUNO) {
         nYFin = pnY + clsConstantes.nALTO_RELACION +
            clsConstantes.nALTO_LINEA_ATRIBUTO;
         for (iIndice = 0; iIndice < pvAtributos.size(); iIndice++) {
            if (iIndice == 0) {
               pnY += clsConstantes.nALTO_RELACION;

            }
            nNumAtribs += PintarAtributo(g,
                                         c,
                                         (clsElemento) pvAtributos.elementAt(
               iIndice),
                                         pnX +
                                         clsConstantes.nANCHO_RELACION / 2,
                                         pnY,
                                         nYFin,
                                         pnVerAtributos);

            pnY = nYFin;
            nYFin += nNumAtribs * clsConstantes.nALTO_LINEA_ATRIBUTO;
            nNumAtribs = 1;
         } // for
      } // if
   } // Procedimiento: PintarRelacionIdentificativa

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: PintarEspecializacion                                    //
   // Parmetros: - g (Graphics): Objeto grfico                              //
   //             - c (Color): Color para pintar                              //
   //             - pnX (int): Posicin horizontal del elemento               //
   //             - pnY (int): Posicin vertical del elemento                 //
   //             - psTipo (String): Tipo de especializacin                  //
   //             - psAtributoEspecializacion (String): Atributo de la        //
   //               especializacin                                           //
   // Descripcin: Pinta una especializacin                                  //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   public void PintarEspecializacion(Graphics g,
                                     Color c,
                                     int pnX,
                                     int pnY,
                                     String psTipo,
                                     String psAtributoEspecializacion) {

      int nX; // Posicin horizontal a partir de la cul pintar
      int nY; // Posicin vertical a partir de la cul pintar

      // Pintar la representacin de la especializacin
      g.setColor(clsConstantes.COLOR_FONDO_APPLET);
      g.fillOval(pnX, pnY, clsConstantes.nANCHO_ESPGEN_UNCAT,
                 clsConstantes.nALTO_ESPGEN_UNCAT);
      g.setColor(c);
      g.drawOval(pnX, pnY, clsConstantes.nANCHO_ESPGEN_UNCAT,
                 clsConstantes.nALTO_ESPGEN_UNCAT);

      if (psTipo.equals("o")) {
         g.drawString(psTipo, pnX + 10, pnY + 17);
      } // if
      else {
         g.drawString(psTipo, pnX + 9, pnY + 18);
      } // else

      // Pintar el atributo de las especializacin, si lo tiene

      // Obtener la posicin horizontal a partir de la cul pintar
      nX = pnX + clsConstantes.nANCHO_ESPGEN_UNCAT;
      // Obtener la posicin horizontal a partir de la cul pintar
      nY = pnY + (clsConstantes.nALTO_ESPGEN_UNCAT / 2);

      // Escirbir el nombre del atributo
      g.drawString(psAtributoEspecializacion,
                   nX + clsConstantes.nSEP_COM, nY + 3);
   } // Procedimiento: PintarEspecializacion

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: PintarTipo_U_I                                           //
   // Parmetros: - g (Graphics): Objeto grfico                              //
   //             - c (Color): Color para pintar                              //
   //             - pnX (int): Posicin horizontal del elemento               //
   //             - pnY (int): Posicin vertical del elemento                 //
   //             - pnTipo (int): Tipo de elemento                            //
   // Descripcin: Pinta un tipo unin o interseccin                         //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   public void PintarTipo_U_I(Graphics g,
                              Color c,
                              int pnX,
                              int pnY,
                              int pnTipo) {

      // Pintar la representacin del elemento
      g.setColor(clsConstantes.COLOR_FONDO_APPLET);
      g.fillOval(pnX, pnY, clsConstantes.nANCHO_ESPGEN_UNCAT,
                 clsConstantes.nALTO_ESPGEN_UNCAT);
      g.setColor(c);
      g.drawOval(pnX, pnY, clsConstantes.nANCHO_ESPGEN_UNCAT,
                 clsConstantes.nALTO_ESPGEN_UNCAT);

      if ( (pnTipo == clsConstantes.nTIPO_INTERSECCION_PARCIAL) ||
          (pnTipo == clsConstantes.nTIPO_INTERSECCION_TOTAL)) {
         g.drawArc(pnX + clsConstantes.nANCHO_ESPGEN_UNCAT / 4,
                   pnY + clsConstantes.nANCHO_ESPGEN_UNCAT / 4,
                   clsConstantes.nANCHO_ESPGEN_UNCAT / 2,
                   clsConstantes.nALTO_ESPGEN_UNCAT, 180, -180);
      } // if
      else {
         g.drawArc(pnX + clsConstantes.nANCHO_ESPGEN_UNCAT / 4,
                   pnY - clsConstantes.nANCHO_ESPGEN_UNCAT / 4,
                   clsConstantes.nANCHO_ESPGEN_UNCAT / 2,
                   clsConstantes.nALTO_ESPGEN_UNCAT, 180, 180);

      } // else
   } // Procedimiento:PintarTipo_U_I

} // Clase: clsGraficos