package Derytas;

import java.util.*;
import java.awt.*;
import java.awt.List;
import java.awt.event.*;

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// Clase: clsRelacion                                                         //
// Mtodos: - public clsRelacion: Constructor de la clase                     //
//          - private void Ocultar: Cierra la ventana                         //
//          - private void Mostrar: Muestra la ventana segn el modo de       //
//            apertura indicado                                               //
//          - private void AgregarEntidad: Agregar una entidad participante a //
//            la relacin                                                     //
//          - private void EliminarEntRel: Elimina una entidad participante   //
//            de la relacin                                                  //
//          - private void HabilitarCrear: Habilita/Deshabilita el botn      //
//            Crear                                                           //
//          - private void CargarCardinalidad: Carga los valores posibles de  //
//            cardinalidades en una lista desplegable                         //
//          - private void LimpiarVentana: Establece los valores iniciales de //
//            la ventana                                                      //
//          - private void ActivarAgregar: Habilita el botn para agregar     //
//            entidades                                                       //
//          - private void ActivarQuitar. Habilita el botn para agregar      //
//            entidades                                                       //
//          - private void CrearRelacion: Crea la relacin                    //
//          - private void QuitarRelacionEntidades: Quita la relacin de las  //
//            entidades participantes                                         //
//          - private boolean bValidar: Comprueba que los datos introducidos  //
//            para la creacin de la relacin son correctos                   //
//          - private void ActualizarTitulo: Actualiza el ttulo de la        //
//            ventana                                                         //
//          - private void AgregarRelacionAEntidades: Agrega la relacin a    //
//            entidades participantes                                         //
//          - private void AbrirAtributo: Abre una nueva ventana para crear   //
//            un atributo                                                     //
// Descripcin: Clase para crear y modificar relaciones                       //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////
public class clsRelacion
   extends Frame {

   // Variables de clase
   private clsGeneral oGeneral;
   private clsConfig oConfig;
   private clsMensajes oMensajes;
   private int nModo; // Modo de apertura de la ventana

   // Componentes de la ventana

   private Label lblNombre;
   private TextField txtNombre;
   private Checkbox chbIdentificativa;
   private Label lblEntidades;
   private List lstEntidades;
   private Label lblEntRels;
   private List lstEntRels;
   private Label lblAtributos;
   private Button btnSubir;
   private Button btnBajar;
   private List lstAtributos;
   private Label lblMinMax;
   private Choice choMin;
   private Choice choMax;
   private Label lblQuitarAgregar;
   private Button btnAgregar;
   private Button btnQuitar;
   private Button btnAtributos;
   private Button btnElimAtrib;
   private Button btnCrear;
   private Button btnCerrar;
   private Checkbox chbCerrar;
   private Vector vEntRels;
   private Vector vAtributos;
   private String sNombreAntiguo = "";
   private clsEntRel erEntidad;

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: clsRelacion                                              //
   // Parmetros: - poGeneral (clsGeneral): Objeto de la clase clsGeneral     //
   //               para poder acceder a los mtodos de carcter general      //
   //             - poConfig (clsConfig): Objeto de la clase clsConfig para   //
   //               poder acceder a las variables de configuracin            //
   //             - pnModo (int): Modo de acceso a la ventana                 //
   // Descripcin: Constructor de la clase                                    //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   public clsRelacion(clsGeneral poGeneral,
                      clsConfig poConfig,
                      int pnModo) {

      // Inicializacin de variables
      oGeneral = poGeneral;
      oConfig = poConfig;
      oMensajes = new clsMensajes(oGeneral);
      nModo = pnModo;

      // Ancho local para los Label lblMinMax
      int nAncho_lblMinMax = 80;
      // Ancho local para los Choice choMin y choMax
      int nAncho_choMin_choMax = 40;
      // Ancho local para los botones Quitar/Agregar
      int nAnchoButton = clsConstantes.nANCHO_BUTTON / 2;

      // Crear los componentes de la ventana
      lblNombre = new Label("Nombre de la Relacin");
      txtNombre = new TextField();
      chbIdentificativa = new Checkbox("Es Relacin de Dependencia");
      lblEntidades = new Label(
         "Elegir Entidades para aadir a la Relacin");
      lstEntidades = new List();
      lblMinMax = new Label("   MIN       MAX");
      choMin = new Choice();
      choMax = new Choice();
      lblQuitarAgregar = new Label("Quitar/Agregar");
      btnAgregar = new Button(">>");
      btnQuitar = new Button("<<");
      lblEntRels = new Label("Entidades que participan en la Relacin");
      lstEntRels = new List();
      lblAtributos = new Label("Lista de Atributos");
      btnSubir = new Button("Subir");
      btnBajar = new Button("Bajar");
      lstAtributos = new List();
      btnAtributos = new Button("Crear Atributos");
      btnElimAtrib = new Button("Eliminar Atributo");
      if (nModo == clsConstantes.nMODO_ALTA) {
         vEntRels = new Vector();
      } // if
      else {
         vEntRels = oGeneral.oModificar.vEntRel;
      } // else
      if (nModo == clsConstantes.nMODO_ALTA) {
         vAtributos = new Vector();
      } // if
      else {
         vAtributos = oGeneral.oModificar.vAtributos;
      } // else
      btnCrear = new Button("Crear Relacin");
      btnCerrar = new Button("Cancelar");
      chbCerrar = new Checkbox("Cerrar despus de Crear");

      // Propiedades de los componentes de la ventana

      // lblNombre
      // Establecer tamao y posicin
      lblNombre.setBounds(new Rectangle(clsConstantes.nSEP_VHOR,
                                        clsConstantes.nSEP_VVER,
                                        clsConstantes.nANCHO_LABEL,
                                        clsConstantes.nALTO_COM));

      // txtNombre
      // Establecer tamao y posicin
      txtNombre.setBounds(new Rectangle(clsConstantes.nSEP_VHOR,
                                        lblNombre.getY() +
                                        clsConstantes.nALTO_COM,
                                        clsConstantes.nANCHO_TEXTFIELD,
                                        clsConstantes.nALTO_COM));
      // Controlar el teclado
      txtNombre.addKeyListener(new KeyListener() {

         String sNombre; // Nombre de la entidad

         public void keyTyped(KeyEvent e) {
            //  No hace falta
         }; // keyTyped

         public void keyPressed(KeyEvent e) {
            if ( (e.getKeyCode() == e.VK_ENTER) &&
                (! (txtNombre.getText().equals("")))) {
               // Crear la relacin cuando se pulsa ENTER sobre el nombre
               CrearRelacion();
            } // if
            else {
               // Habilitar/Deshabilitar el botn Crear
               HabilitarCrear();
               if (txtNombre.getText().length() <= clsConstantes.nMAX_NOMBRE) {
                  // Guardar el nombre escrito hasta el momento
                  sNombre = txtNombre.getText();
                  // Actualizar el ttulo de la ventana
                  ActualizarTitulo();
               } // if
               else {
                  txtNombre.setText(sNombre);
               } // else
            } // else
         } // keyPressed

         public void keyReleased(KeyEvent e) {
            // Habilitar/Deshabilitar el botn Crear
            HabilitarCrear();
            if (txtNombre.getText().length() <= clsConstantes.nMAX_NOMBRE) {
               // Guardar el nombre escrito hasta el momento
               sNombre = txtNombre.getText();
               // Actualizar el ttulo de la ventana
               ActualizarTitulo();
            } // if
            else {
               txtNombre.setText(sNombre);
            } // else
         } // keyReleased
      }); // addKeyListener

      // chbIdentificativa
      // Establecer tamao y posicin
      chbIdentificativa.setBounds(new Rectangle(txtNombre.getX() +
                                                clsConstantes.nANCHO_TEXTFIELD +
                                                clsConstantes.nSEP_COM,
                                                txtNombre.getY(),
                                                clsConstantes.nANCHO_CHECKBOX,
                                                clsConstantes.nALTO_COM));

      // lblEntidades
      // Establecer tamao y posicin
      lblEntidades.setBounds(new Rectangle(clsConstantes.nSEP_VHOR,
                                           txtNombre.getY() +
                                           clsConstantes.nSEP_COM +
                                           clsConstantes.nALTO_COM,
                                           clsConstantes.nANCHO_LABEL + 5,
                                           clsConstantes.nALTO_COM));

      // lstEntidades
      // Establecer tamao y posicin
      lstEntidades.setBounds(new Rectangle(clsConstantes.nSEP_VHOR,
                                           lblEntidades.getY() +
                                           clsConstantes.nALTO_COM,
                                           clsConstantes.nANCHO_LIST,
                                           clsConstantes.nALTO_LIST));
      // Cargar la lista con las entidades
      oGeneral.CargarLista(lstEntidades, clsConstantes.nENTIDAD);
      // Cargar la lista con las entidades dbiles
      oGeneral.CargarLista(lstEntidades, clsConstantes.nENTIDAD_DEBIL);
      // Aadir el componente al listener para cuando se cambie la seleccin
      lstEntidades.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            // Llamar al mtodo que gestiona el evento
            ActivarAgregar();
         } // Evento: actionPerformed
      }); // addActionListener

      // lblMinMax
      // Establecer tamao y posicin
      lblMinMax.setBounds(new Rectangle(lstEntidades.getX() +
                                        clsConstantes.nANCHO_LIST +
                                        clsConstantes.nSEP_COM,
                                        lstEntidades.getY(),
                                        nAncho_lblMinMax,
                                        clsConstantes.nALTO_COM));

      // choMin
      // Establecer tamao y posicin
      choMin.setBounds(new Rectangle(lblMinMax.getX(),
                                     lblMinMax.getY() +
                                     clsConstantes.nALTO_COM +
                                     clsConstantes.nSEP_COM,
                                     nAncho_choMin_choMax,
                                     clsConstantes.nALTO_COM));
      // Cargar la lista desplegable con las cardinalidades mnimas
      CargarCardinalidad(choMin, false);

      // choMax
      // Establecer tamao y posicin
      choMax.setBounds(new Rectangle(choMin.getX() +
                                     nAncho_choMin_choMax +
                                     clsConstantes.nSEP_COM,
                                     choMin.getY(),
                                     nAncho_choMin_choMax,
                                     clsConstantes.nALTO_COM));
      // Cargar la lista desplegable con las cardinalidades mximas
      CargarCardinalidad(choMax, true);

      // lblQuitarAgregar
      // Establecer tamao y posicin
      lblQuitarAgregar.setBounds(new Rectangle(choMin.getX(),
                                               choMin.getY() +
                                               choMin.getHeight() +
                                               clsConstantes.nSEP_COM,
                                               90,
                                               clsConstantes.nALTO_COM));

      // btnQuitar
      // Establecer tamao y posicin
      btnQuitar.setBounds(new Rectangle(lblQuitarAgregar.getX(),
                                        lblQuitarAgregar.getY() +
                                        choMin.getHeight(),
                                        nAnchoButton,
                                        clsConstantes.nALTO_BUTTON));
      // Deshabilitar el botn
      btnQuitar.setEnabled(false);
      // Aadir el componente al listener para cuando se pulse el botn
      btnQuitar.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            // Llamar al mtodo que gestiona el evento
            EliminarEntRel();
         } // Evento: actionPerformed
      }); // addActionListener

      // btnAgregar
      // Establecer tamao y posicin
      btnAgregar.setBounds(new Rectangle(btnQuitar.getX() +
                                         btnQuitar.getWidth() +
                                         clsConstantes.nSEP_COM,
                                         btnQuitar.getY(),
                                         nAnchoButton,
                                         clsConstantes.nALTO_BUTTON));
      // Deshabilitar el botn
      btnAgregar.setEnabled(false);
      // Aadir el componente al listener para cuando se pulse el botn
      btnAgregar.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            // Llamar al mtodo que gestiona el evento
            AgregarEntidad();
         } // Evento: actionPerformed
      }); // addActionListener

      // lstEntRels
      // Establecer tamao y posicin
      lstEntRels.setBounds(new Rectangle(btnAgregar.getX() +
                                         btnAgregar.getWidth() +
                                         clsConstantes.nSEP_COM,
                                         lstEntidades.getY(),
                                         clsConstantes.nANCHO_LIST,
                                         clsConstantes.nALTO_LIST));
      // Aadir el componente al listener para cuando se cambie la seleccin
      lstEntRels.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            // Llamar al mtodo que gestiona el evento
            ActivarQuitar();
         } // Evento: actionPerformed
      }); // addActionListener

      // lblEntRels
      // Establecer tamao y posicin
      lblEntRels.setBounds(new Rectangle(lstEntRels.getX(),
                                         lblEntidades.getY(),
                                         clsConstantes.nANCHO_LABEL,
                                         clsConstantes.nALTO_COM));

      // lblAtributos
      // Establecer tamao y posicin
      lblAtributos.setBounds(new Rectangle(lblEntRels.getX() +
                                           lblEntRels.getWidth() +
                                           clsConstantes.nSEP_COM,
                                           lblEntRels.getY(),
                                           clsConstantes.nANCHO_LABEL,
                                           clsConstantes.nALTO_COM));
      // btnBajar
      // Establecer tamao y posicin
      btnBajar.setBounds(new Rectangle(lblAtributos.getX() +
                                       lblAtributos.getWidth() -
                                       clsConstantes.nANCHO_BUTTON / 2,
                                       lblAtributos.getY() -
                                       clsConstantes.nSEP_COM,
                                       clsConstantes.nANCHO_BUTTON / 2,
                                       clsConstantes.nALTO_BUTTON));
      // Deshabilitar el botn
      btnBajar.setEnabled(false);
      // Aadir el componente al listener para cuando se pulse el botn
      btnBajar.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            // Llamar al mtodo que gestiona el evento
            oGeneral.Bajar(lstAtributos,
                           vAtributos,
                           btnSubir,
                           btnBajar);
         } // Evento: actionPerformed
      }); // addActionListener

      // btnSubir
      // Establecer tamao y posicin
      btnSubir.setBounds(new Rectangle(btnBajar.getX() -
                                       clsConstantes.nANCHO_BUTTON / 2 -
                                       clsConstantes.nSEP_COM,
                                       lblAtributos.getY() -
                                       clsConstantes.nSEP_COM,
                                       clsConstantes.nANCHO_BUTTON / 2,
                                       clsConstantes.nALTO_BUTTON));
      // Deshabilitar el botn
      btnSubir.setEnabled(false);
      // Aadir el componente al listener para cuando se pulse el botn
      btnSubir.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            // Llamar al mtodo que gestiona el evento
            oGeneral.Subir(lstAtributos,
                           vAtributos,
                           btnSubir,
                           btnBajar);
         } // Evento: actionPerformed
      }); // addActionListener

      // lstAtributos
      // Establecer tamao y posicin
      lstAtributos.setBounds(new Rectangle(lblAtributos.getX(),
                                           lblAtributos.getY() +
                                           clsConstantes.nALTO_COM,
                                           clsConstantes.nANCHO_LIST,
                                           clsConstantes.nALTO_LIST));
      // Aadir el componente al listener para cuando se cambie la seleccin
      lstAtributos.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            // Llamar al mtodo que gestiona el evento
            oGeneral.ListaClick(btnElimAtrib,
                                lstAtributos,
                                btnSubir,
                                btnBajar);
         } // Evento: actionPerformed
      }); // addActionListener

      // btnAtributos
      // Establecer tamao y posicin
      btnAtributos.setBounds(new Rectangle(lstAtributos.getX(),
                                           lstAtributos.getY() +
                                           lstAtributos.getHeight() +
                                           clsConstantes.nSEP_COM,
                                           clsConstantes.
                                           nANCHO_BUTTON_LIST,
                                           clsConstantes.nALTO_BUTTON));
      // Deshabilitar el botn
      btnAtributos.setEnabled(false);
      // Aadir el componente al listener para cuando se pulse el botn
      btnAtributos.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            // Llamar al mtodo que gestiona el evento
            AbrirAtributo();
         } // Evento: actionPerformed
      }); // addActionListener

      // btnElimAtrib
      // Establecer tamao y posicin
      btnElimAtrib.setBounds(new Rectangle(btnAtributos.getX() +
                                           btnAtributos.getWidth() +
                                           clsConstantes.nSEP_COM,
                                           btnAtributos.getY(),
                                           clsConstantes.
                                           nANCHO_BUTTON_LIST,
                                           clsConstantes.nALTO_BUTTON));
      // Deshabilitar el botn
      btnElimAtrib.setEnabled(false);
      // Aadir el componente al listener para cuando se pulse el botn
      btnElimAtrib.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            // Llamar al mtodo que gestiona el evento
            oGeneral.EliminarAtributo(lstAtributos,
                                      vAtributos,
                                      btnElimAtrib);
         } // Evento: actionPerformed
      }); // addActionListener

      // btnCerrar
      // Establecer tamao y posicin
      btnCerrar.setBounds(new Rectangle(lstAtributos.getX() +
                                        lstAtributos.getWidth() -
                                        clsConstantes.nANCHO_BUTTON,
                                        btnAtributos.getY() +
                                        btnAtributos.getHeight() +
                                        clsConstantes.nSEP_COM,
                                        clsConstantes.nANCHO_BUTTON,
                                        clsConstantes.nALTO_BUTTON));
      // Aadir el componente al listener para cuando se pulse el botn
      btnCerrar.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            // Llamar al mtodo que gestiona el evento
            Ocultar();
         } // Evento: actionPerformed
      }); // addActionListener

      // btnCrear
      // Establecer tamao y posicin
      btnCrear.setBounds(new Rectangle(btnCerrar.getX() -
                                       clsConstantes.
                                       nANCHO_BUTTON_CREAR -
                                       clsConstantes.nSEP_COM,
                                       btnCerrar.getY(),
                                       clsConstantes.
                                       nANCHO_BUTTON_CREAR,
                                       clsConstantes.nALTO_BUTTON));
      // Habilitar/Deshabilitar el botn segun el modo de acceso
      btnCrear.setEnabled(nModo == clsConstantes.nMODO_MODIFICAR);
      // Aadir el componente al listener para cuando se pulse el botn
      btnCrear.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            // Llamar al mtodo que gestiona el evento
            CrearRelacion();
         } // Evento: actionPerformed
      }); // addActionListener

      // chbCerrar
      // Establecer tamao y posicin
      chbCerrar.setBounds(new Rectangle(clsConstantes.nSEP_VHOR,
                                        btnCerrar.getY() +
                                        clsConstantes.nALTO_ATRIBUTO +
                                        clsConstantes.nSEP_COM,
                                        clsConstantes.nANCHO_CHECKBOX,
                                        clsConstantes.nALTO_COM));
      // Obtener la opcin para cerrar de la variable de configuracin
      chbCerrar.setState(oConfig.bCerrar);

      // Propiedades de la ventana
      // Establecer el tamao
      setSize(oGeneral.nAnchoVentana(btnCerrar.getX()),
              oGeneral.nAltoVentana(chbCerrar.getY()));
      // Establecer tipo de Layout
      setLayout(null);
      // Impedir que se redimensione la ventana
      setResizable(false);
      // Establecer el icono de la ventana
      setIconImage(oGeneral.imgLogo);
      // Centrar la ventana
      oGeneral.CentrarVentana(this);
      // Indicar que sta es la ventana actual
      oGeneral.EstablecerActual(this);
      // Aadir los componenentes a la ventana
      add(lblNombre);
      add(txtNombre);
      add(chbIdentificativa);
      add(lblEntidades);
      add(lblMinMax);
      add(choMin);
      add(choMax);
      add(lblQuitarAgregar);
      add(btnAgregar);
      add(btnQuitar);
      add(btnAtributos);
      add(btnElimAtrib);
      add(lstEntidades);
      add(lstAtributos);
      add(btnSubir);
      add(btnBajar);
      add(lblAtributos);
      add(btnAtributos);
      add(btnCerrar);
      add(lblEntRels);
      add(lstEntRels);
      add(btnCrear);
      add(chbCerrar);
      // Aadir la ventana al listener para cuando se cierre la ventana
      addWindowListener(new WindowAdapter() {
         public void windowClosing(WindowEvent e) {
            // Llamar al mtodo que gestiona el evento
            Ocultar();
         } // windowClosing
      }); // addWindowListener
      // Mostrar los datos de la ventana segn el modo de acceso
      Mostrar();
      // Mostrar la ventana
      setVisible(true);
   } // Procedimiento: clsRelacion

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: Ocultar                                                  //
   // Parmetros: No                                                          //
   // Descripcin: Lgica necesaria  para cerrar la ventana                   //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private void Ocultar() {
      // Limpiar la ventana actual
      oGeneral.EstablecerActual(null);
      // Indicar que no hay ninguna ventana abierta
      oGeneral.bVentanaAbierta = false;
      // Ocultar la ventana
      this.dispose();
   } // Procedimiento: Ocultar

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: Mostrar                                                  //
   // Parmetros: No                                                          //
   // Descripcin: Limpia los campos de la ventana o los informa con los      //
   //              datos de una relacin segun el modo de acceso              //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private void Mostrar() {
      if (nModo == clsConstantes.nMODO_ALTA) {
         // Limpiar la ventana
         LimpiarVentana();
      } // if
      else {
         // Cambiar el ttulo del botn de crear
         btnCrear.setLabel("Modificar Relacin");
         // Cambiar el ttulo de la check de cerrar la ventana
         chbCerrar.setLabel("Cerrar despus de Modificar");
         // Cerrar la ventana tras modificar
         chbCerrar.setState(true);
         // Impedir que no se cierre la ventana, inhabilitando la check
         chbCerrar.setEnabled(false);
         // Guardar el Nombre original
         sNombreAntiguo = oGeneral.oModificar.sNombre;
         // Mostrar el Nombre
         txtNombre.setText(sNombreAntiguo);
         // Identificar si es Relacin Identificativa
         if (oGeneral.oModificar.nTipo ==
             clsConstantes.nRELACION_IDENTIFICATIVA) {
            chbIdentificativa.setState(true);
         }
         // Mostrar las entidades participantes en la relacin y quitarlas de las disponibles
         for (int i = 0; i < vEntRels.size(); i++) {
            erEntidad = (clsEntRel) vEntRels.elementAt(i);
            lstEntRels.add(erEntidad.sNombre + " (" + erEntidad.sMin +
                           "," + erEntidad.sMax + ")");
            lstEntidades.remove(erEntidad.sNombre);
         } // for
         // Mostrar los atributos de la relacin
         for (int i = 0; i < vAtributos.size(); i++) {
            lstAtributos.add( ( (clsElemento) vAtributos.elementAt(i)).
                             sNombre);
         } // for
         btnAtributos.setEnabled(true);
         // Cambiar el ttulo de la ventana
         this.setTitle("Relacin: " + oGeneral.oModificar.sNombre);
      } // if
   } // Procedimiento: Mostrar

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: AgregarEntidad                                           //
   // Parmetros: No                                                          //
   // Descripcin: Agregar una entidad participante a la relacin             //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private void AgregarEntidad() {

      String sNombre;
      String sMin;
      String sMax;

      sNombre = lstEntidades.getSelectedItem();
      sMin = choMin.getSelectedItem();
      sMax = choMax.getSelectedItem();

      vEntRels.addElement(new clsEntRel(sNombre, sMin, sMax));
      lstEntRels.add(sNombre + " (" + sMin + ", " + sMax + ")");
      lstEntidades.remove(lstEntidades.getSelectedIndex());
      btnAgregar.setEnabled(false);
      HabilitarCrear();
   } // Procedimiento: AgregarEntidad

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: EliminarEntRel                                           //
   // Parmetros: No                                                          //
   // Descripcin: Elimina una entidad participante a la relacin             //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private void EliminarEntRel() {

      int iIndice;

      iIndice = lstEntRels.getSelectedIndex();
      lstEntRels.remove(iIndice);
      lstEntidades.add( ( (clsEntRel) vEntRels.elementAt(iIndice)).sNombre);
      vEntRels.removeElementAt(iIndice);
      btnQuitar.setEnabled(false);
      HabilitarCrear();
   } // Procedimiento: EliminarEntRel

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: HabilitarCrear                                           //
   // Parmetros: No                                                          //
   // Descripcin: Habilita/Deshabilita el botn Crear                        //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private void HabilitarCrear() {
      btnCrear.setEnabled(!txtNombre.getText().equals("") &&
                          (vEntRels.size() > 1));
      btnAtributos.setEnabled(!txtNombre.getText().equals(""));
   } // Procedimiento: HabilitarCrear

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: CargarCardinalidad                                       //
   // Parmetros: - pchoChoice (Choice): Lista desplegable donde cargar los   //
   //               datos                                                     //
   //             - pbParticipacionTotal (boolean): Indicador de              //
   //               participacin total                                       //
   // Descripcin: Carga la lista desplegable con los valores de las          //
   //              cardinalidades                                             //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private void CargarCardinalidad(Choice pchoChoice,
                                   boolean pbParticipacionTotal) {
      if (!pbParticipacionTotal) {
         pchoChoice.add("0");
      } // if
      pchoChoice.add("1");
      pchoChoice.add("N");
      pchoChoice.add("M");
   } // Procedimiento: CargarCardinalidad

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: LimpiarVentana                                           //
   // Parmetros: No                                                          //
   // Descripcin: Establece los valores iniciales de la ventana              //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private void LimpiarVentana() {
      txtNombre.setText("");
      chbIdentificativa.setState(false);
      choMin.select(0);
      choMax.select(0);
      this.setTitle("Nueva Relacin");
      lstAtributos.removeAll();
      btnSubir.setEnabled(false);
      btnBajar.setEnabled(false);
      lstEntidades.removeAll();
      oGeneral.CargarLista(lstEntidades, clsConstantes.nENTIDAD);
      oGeneral.CargarLista(lstEntidades, clsConstantes.nENTIDAD_DEBIL);
      lstEntRels.removeAll();
      vAtributos.removeAllElements();
      btnAtributos.setEnabled(false);
      btnElimAtrib.setEnabled(false);
      btnAgregar.setEnabled(false);
      btnQuitar.setEnabled(false);
      btnCrear.setEnabled(false);
      txtNombre.requestFocus();
   } // Procedimiento: LimpiarVentana

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: ActivarAgregar                                           //
   // Parmetros: No                                                          //
   // Descripcin: Habilita el botn para agregar entidades                   //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private void ActivarAgregar() {
      btnAgregar.setEnabled(!lstEntidades.getSelectedItem().equals(""));
   } // Procedimeinto: ActivarAgregar

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: ActivarQuitar                                            //
   // Parmetros: No                                                          //
   // Descripcin: Habilita el botn para quitar entidades                    //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private void ActivarQuitar() {
      btnQuitar.setEnabled(!lstEntRels.getSelectedItem().equals(""));
   } // Procedimeinto: ActivarQuitar

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: CrearRelacion                                            //
   // Parmetros: No                                                          //
   // Descripcin: Crea la relacin con los datos introducidos en la ventana  //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private void CrearRelacion() {

      String sNombre; // Nombre de la relacin
      int nTipo; // Tipo de relacin
      clsElemento oRelacion; // Objeto para una relacin

      // Obtener el nombre de la relacin
      sNombre = oGeneral.sFiltrarNombre(txtNombre.getText());

      if (!chbIdentificativa.getState()) {
         nTipo = clsConstantes.nRELACION;
      } // if
      else {
         nTipo = clsConstantes.nRELACION_IDENTIFICATIVA;
      } // else
      if (bValidar(sNombre, nTipo)) {
         if (oGeneral.oModificar == null) {
            oRelacion = new clsElemento(oGeneral, oConfig.nVerAtributos);

            // Asigna el Nombre del elemento
            oRelacion.sNombre = sNombre;
            // Asigna el Tipo de elemento
            oRelacion.nTipo = nTipo;
            // Asigna la coordenada X de la posicin del elemento
            oRelacion.nX = oGeneral.nXInicial;
            // Asigna la coordenada Y de la posicin del elemento
            oRelacion.nY = oGeneral.nYInicial;
            // Buscar una posicin libre en el esquema
            while (oGeneral.bEnElemento(oRelacion.nX,
                                        oRelacion.nY,
                                        false)) {
               // Asigna la coordenada X de la posicin del elemento
               oRelacion.nX += clsConstantes.nSEP_COM;
               // Asigna la coordenada Y de la posicin del elemento
               oRelacion.nY += clsConstantes.nSEP_COM;
            } // while
            // Asigna el Ancho del elemento
            oRelacion.nAncho = clsConstantes.nANCHO_RELACION;
            // Asigna el Alto del elemento
            oRelacion.nAlto = clsConstantes.nALTO_RELACION;
            // Inicializar el vector de Entidades Relacionadas
            oRelacion.vEntRel = (Vector) vEntRels.clone();
            oRelacion.vAtributos = (Vector) vAtributos.clone();
            AgregarRelacionAEntidades(oRelacion);
            oGeneral.vElementos.add(oRelacion);
         } // if
         else {
            oGeneral.oModificar.sNombre = sNombre;
            oGeneral.oModificar.nTipo = nTipo;
            oGeneral.oModificar.vEntRel = (Vector) vEntRels.clone();
            oGeneral.oModificar.vAtributos = (Vector) vAtributos.clone();
            QuitarRelacionEntidades(sNombreAntiguo);
            AgregarRelacionAEntidades(oGeneral.oModificar);

         } // else
         oGeneral.oModificar = null;
         // Actualizar la variable global bCerrar con el valor de la ventana
         oConfig.bCerrar = chbCerrar.getState();

         // Asignar la relacin como propietaria de los atributos
         oGeneral.AsignarPropietario(sNombre, vAtributos);

         // Comprobar si se debe ocultar la ventana
         if (oConfig.bCerrar) {
            // Ocultar la ventana
            Ocultar();
         } // if
         else {
            // Limpiar la ventana
            LimpiarVentana();
         } // else

         // Actualizar el esquema
         oGeneral.Repintar();
      } // if
   } // Procedimiento: CrearRelacion

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: QuitarRelacionEntidades                                  //
   // Parmetros: - psNombreAntiguo (String): Nombre de la entidad            //
   // Descripcin: Crea la relacin con los datos introducidos en la ventana  //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private void QuitarRelacionEntidades(String psNombreAntiguo) {

      int iElemento; // ndice para recorrer el vector de elementos
      clsElemento oElemento; // Elemento actual a tratar
      int iRelacion; // ndice para recorrer el vector de relaciones
      boolean bEncontrada;

      for (iElemento = 0; iElemento < oGeneral.vElementos.size(); iElemento++) {
         oElemento = (clsElemento) oGeneral.vElementos.elementAt(iElemento);
         if ( (oElemento.nTipo == clsConstantes.nENTIDAD) ||
             (oElemento.nTipo == clsConstantes.nENTIDAD_DEBIL)) {
            bEncontrada = false;
            iRelacion = 0;
            while ( (!bEncontrada) && (iRelacion < oElemento.vRelaciones.size())) {
               if ( ( (String) oElemento.vRelaciones.elementAt(iRelacion)).
                   equals(psNombreAntiguo)) {
                  oElemento.vRelaciones.removeElementAt(iRelacion);
               } // if
               iRelacion++;
            } // while
         } // if
      } // for
   } // Procedimiento: QuitarRelacionEntidades

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Funcin: bValidar                                                       //
   // Parmetros: - psNombre (String): Nombre del atributo                    //
   //             - pnTipo (int): Tipo de relacin                            //
   // Retorno: Devuelve un valor lgico que indica si se han pasado o no      //
   //          todas las validaciones                                         //
   // Descripcin: Comprueba que los datos para crear la entidad son          //
   //              correctos, mostrando un mensaje de error para los datos    //
   //              que no sean vlidos                                        //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private boolean bValidar(String sNombre,
                            int nTipo) {

      boolean bRetorno = true; // Valor a devolver con el resultado de las validaciones
      clsElemento oEntidad; // Objeto para una entidad
      clsEntRel erEntidad;
      int iIndice;
      clsElemento elmAtributo;

      // Validar que el elemento no exista ya (si no es modificacin)
      if ( (oGeneral.oModificar == null) ||
          ( (oGeneral.oModificar != null) && (!sNombre.equals(sNombreAntiguo)))) {
         if (oGeneral.bExisteElemento(sNombre, nTipo, oGeneral.vElementos)) {
            oMensajes.MostrarMensaje(clsConstantes.sERROR_ELEMENTO_YA_EXISTE,
                                     sNombre,
                                     this);
            return false;
         } // if
      } // if

      // Validar que los tipos de atributos son correctos para el tipo de entidad
      for (iIndice = 0; iIndice < vAtributos.size(); iIndice++) {
         elmAtributo = (clsElemento) vAtributos.elementAt(iIndice);
         if (elmAtributo.nTipo == clsConstantes.nATRIBUTO_LLAVE_PARCIAL) {
            oMensajes.MostrarMensaje(clsConstantes.sERROR_LLAVE_PARCIAL,
                                     this);
            return false;
         } // if
         if (elmAtributo.nTipo == clsConstantes.nATRIBUTO_LLAVE) {
            oMensajes.MostrarMensaje(clsConstantes.sERROR_LLAVE_PRIMARIA,
                                     this);
            return false;
         } // if
      } // for

      for (int i = 0; i < vEntRels.size(); i++) {
         erEntidad = (clsEntRel) vEntRels.elementAt(i);
         oEntidad = oGeneral.elmElemento(erEntidad.sNombre);
         if ( (oEntidad.nTipo == clsConstantes.nENTIDAD_DEBIL) &&
             (nTipo == clsConstantes.nRELACION_IDENTIFICATIVA) &&
             ( (!erEntidad.sMin.equals("1")) ||
              (!erEntidad.sMax.equals("1")))) {
            oMensajes.MostrarMensaje(clsConstantes.sERROR_LLAVE_COMPUESTA,
                                     this);
            bRetorno = false;
         } // if
      } // for

      // Devolver el resultado de las validaciones
      return bRetorno;
   } // Funcin: bValidar

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: ActualizarTitulo                                         //
   // Parmetros: No                                                          //
   // Descripcin: Actualiza el ttulo de la ventana con el valor introducido //
   //              para el nombre despus de aplicarle el filtro              //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private void ActualizarTitulo() {
      if (txtNombre.getText().equals("")) {
         this.setTitle("Nueva Relacin");
      } // if
      else {
         this.setTitle("Relacin: " +
                       oGeneral.sFiltrarNombre(txtNombre.getText()));
      } // else
   } // Procedimiento: ActualizarTitulo

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: AgregarRelacionAEntidades                                //
   // Parmetros: poElemento (clsElemento): Relacin                          //
   // Descripcin: Agrega la relacin a las entidades participantes           //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private void AgregarRelacionAEntidades(clsElemento poElemento) {

      clsEntRel erEntidad;
      clsElemento oEntidad;

      for (int i = 0; i < poElemento.vEntRel.size(); i++) {
         erEntidad = (clsEntRel) poElemento.vEntRel.elementAt(i);
         oEntidad = oGeneral.elmElemento(erEntidad.sNombre);
         oEntidad.vRelaciones.addElement(poElemento.sNombre);
      } // for
   } // Procedimiento: AgregarRelacionAEntidades

   /////////////////////////////////////////////////////////////////////////////
   //                                                                         //
   // Procedimiento: AbrirAtributo                                            //
   // Parmetros: No                                                          //
   // Descripcin: Abre una nueva ventana para crear un atributo para la      //
   //              relacin                                                   //
   //                                                                         //
   /////////////////////////////////////////////////////////////////////////////
   private void AbrirAtributo() {
      new clsAtributo(this,
                      vAtributos,
                      lstAtributos,
                      oGeneral,
                      oConfig);
   } // Procedimiento: AbrirAtributo

} // Clase: clsRelacion
